
; Program:  Atari_01.asm  Copyright (C) February 1994 By:  Dallas E. Overturf
;
; STATUS:  Main code derived from  my blytst03.asm file 2-20-94.
;		Attempting to port to atari Superman processor.
;
;
; This program may not be sold, used in a commerical environment, packaged
; with a commerical product, or used for profit in any way without prior
; written permission from the author except as noted in the "User Aggreement"
; section below.
; 
; The author makes no warranties as to the contents of this program or related
; documentation and specifically disclaims any implied warranties of
; merchantability or fitness for any particular purpose.  In no event shall the
; author be liable for loss of profit or any other commercial damage, including
; but not limited to special, incidental, consequential or other damages.
; 
; The author reserves the right to modify the program and/or documentation
; without obligation to notify any person or organization of these changes.
;
;
; Written by Dallas E. Overturf
;
; Copyright 1994 by Dallas E. Overturf.
;
; User Aggrement:
;
; You (the user) may freely modify, copy, and or change the code provided
; that:
; 
; You retain the original copyright notice; in all or any part of the code
; that you use, copy, or modify.
;
; You may sell the code (binary and or source) provided that the price
; charged is no more than your cost of the media (disk, EEPROM etc...) that
; it is contained on.  Additional fees such as postage, shipping, and handling
; are not to exceed a total greater than Five Dollars ($5.00) US currency;
; or the equivalent in foreign (non-US currency).
;
;
; Questions Comments etc... On this program may be directed to the
; author at the following address (If you wish a reply please send a SASE)
;
; Dallas E. Overturf
; 267 Main St
; Ashland, MA 01721 (USA)
;
;
; Comments, pleas etc... from the Author:
;
; If you decide to make your own version of the code either by patching
; the binary or by changing the source; please be considerate and patch
; patch the revision byte to a value of 01, 02, or 03 to indicate that the
; copy is a hacked up copy.  I will not use revisions 01, 02, or 03 in a
; released copy.  The first released copy will be revision 04.
; If you modify the source; then please keep the Modification History 
; and or Bug history sections updated as appropriate.
;
; The code:  This code was not written for efficiency and was done in
; my spare time.  No doubt there are things I did that I could have done
; better. It does work the way I intended it to so I'm not planning
; on re-writing it!  If you submit a bug to me that I deem to be a valid
; bug; I will probably fix it; if it can be reasonably fixed (my opinion).
;
; If you come up with something "great" you want added to this code I may
; be willing to consider doing it if it seems reasonable (my opinion).
;
;		Happy Pinballing!
;					Dallas...
;
;
; Modification history
;
; 12-10-92:  DEO  Rev 4.0. Initial Release.
;		  Base file: atari_01.asm.
;
;
;
; End Modification History
;
;
; Bug History
;
; 06-15-93:  DEO Rev 4.0.  None known at this time.
;
; Testing Notes:
;
;	The Atari board accepts 2716 EEPROM.  Thus this test prom
;	should be a 2716; or it may be a 2732 by programming only
;	the upper half of the 2732 (addresses 0x0800-0x0FFF) in 
;	which case the 2732 may be used as a 2716 in the board.
;
;	This test prom is placed into socket ROM3 (location J7) to run.
;	Additionally to run the test you MUST tie to ground the
;	"WATCHDOG KILL" test point on the board to disable the Watchdog
;	timer.  If this prom runs without doing this then this indicates
;	that the watchdog timer is not counting up and issuing a board
;	reset which should occur when the board is not running code that
;	deals with this timer; thus if you see reset pulsing when the
;	NORMAL game rom are in the board; that is an indication that the
;	board is for one reason or another not running it's code.
;	In this case should this test appear to run okay; you should verify
;	that you are getting both the clear watchdog signal, the IACK
;	signal, and also the IRQ signal itself up to the processor.  Note that
;	both IACK and the watchdog clear are software driven by the normal
;	game rom if your game rom are good and can be accessed properly.
;
;
;	Programming the code into an EPROM:
;	For a 2716 just program it in normal (IE: start address is 0000).
;	If your burner requires an end address, it is 07FF.
;
;	For a 2732 the start address is 0800 (this means 0000-07FF
;	remain blank.  If your burner requires an end address, it is 0FFF.
;
;	Note:  If the binary file of this program is larger than 2048 bytes;
;	and the file starts at address 0000; then you will need to give your
;	prom programmer a starting address within the binary file as well to
;	start taking data from.  In this case you need to give it address
;	3800 which will be the offset (in hexadecimal) of the start of code
;	in the binary file (assuming the file starts from address 0x0000.
;
;
;	It is recommended that you verify the prom in a known good board
;	prior to attempting to use it on a bad board!
;
;	Things that may keep you from comming up and working correctly
;	on the game rom that are untested in this prom.
;	INTERRUPTS!!  This test program currently does not do any
;	checking of program interrupts (IRQ lines to the 6800).
;
;
;	Some basic suggestions when trouble shooting a board:
;
;	If the test prom does not work at all; any remaning game rom
;	should also be removed.  Thus this can help prove that
;	there is not a bad game rom on the board.  If this test prom can
;	get through it's set of tests on your mpu, and you do not
;	ever see IACK or watchdog clear occuring then it is recommended
;	that you try a different set of KNOWN GOOD game rom in your mpu to
;	verify that your game rom are good.
;
;	This test program was designed with the intent of finding problems
;	that are due to the battery corroding on the board.  This normally
;	means an OPEN (broken) etch or bad connection within a socket.
;	It is also designed to deal with dead board type problems; however
;	to use this in most dead board conditions you will need to have
;	a 24 channel logic analyzer at your disposal (8 data, 16 address)
;	and of course this listing so you can see what went wrong.
;
;	This program checks the main address/data paths on the 
;	Atari pinball processor board used in Superman and Hercules.
;	PCB numbers; 020839-01 D and 020839-01 E.
;	
;
;	Assuming the program code can be read out of the prom:
;
;	This prom will loop on errors and continue if the error
;	goes away.  Thus it is not usefull for running overnight to see
;	if there is a single intermittent error, as if the error goes away
;	there will be no indication of it as the test will continue
;	(note you could do this with a logic analzer attached).
;	Note:  This is the default setup of the program.  This can
;	be changed by changing the "error_action" byte.  See below.
;
;
;	Since the program in the prom must go over the same bus to the
;	mpu as anything else on the board it is not very usefull for
;	trouble shooting shorts on a data line or address line (EXCEPT
;	with a logic analyzer and this listing) as the test program 
;	probably will not be able to run.
;
;	The design of this test prom was for three main purposes:
;
;	1.  Allow troubleshooting of bus problems relatively quickly
;		and easily using a 24 channel logic analyzer.  
;
;	2.  Also attempt to actively loop on an error so that the error
;		may be more easily seen using an osciloscope or logic probe.
;
;	3.  Attempt (when possible) to be able to give the user a more 
;		definitive visual clue to what was wrong with the bus.
;		Ie:  data and or control was bad or the address lines
;		are bad.
;		Note:  The data test is always run first on a part followed
;		by addressing tests.  The initial data test will not fail
;		due to an open address line to a component under test.
;		It will fail due to a bad data line, or a bad control
;		line.  Control lines are chip selects; R/W and Power/Ground.
;		Thus if you pass the data test and fail the first addressing
;		test you know you will most likely be looking for a bad
;		address line.  If you fail a component uniqueness test
;		then you are looking for a bad address/control line
;		to one of the components that failed.  This is assuming
;		that you have replaced the suspect part with a known good
;		part.
;
;		Note that this test Flashes all four LEDs at once.
;		
;
;
;	Note:  Upon start of the program there will be two short flashes.
;	This is done as a sanity check so that the user can see that the 
;	actual test code has in fact started to run or is re-running.
;
;	Note:  for those using a logic analyzer the following occurs prior
;	to seeing the two sanity check flashes:
;
;	Four data patterns are written out immediately after reset;
;	they are 00, FF, 55, and AA; and are written to address 0000.  
;	They are immediately followed by four reads from the following
;	addresses:  0000, FFFF, 5555, and AAAA.  (data here is random and 
;	of no interest.)
;	This is to allow analyzer users to verify basic data and address bus
;	integrity.  If either is being held down or up; it should be
;	immediately visable here using an analyzer.  The next thing that
;	occurs will be the two short flashes that act as a sanity check for
;	the user to verify that the program is running followed by the first
;	test.
;
;
;	The "error_action" byte settings located at program address 5800 or
;	physical EPROM addresses 0000 (for 2716) or 0800 for (2732/2532).
;
;	If set to "restart_val" (01):
;	Then all errors will, after writing to error_check in; jmp to an 
;	error_flash routine which is responsible for flashing the LED 5
;	times quickly and then restarting the test (based on further decode
;	of the data) from the beginning.   This setting may be usefull for
;	looking at intermittent problems.
;
;	If set to "halt_val" (02):
;	Then all errors will, after writing to error_check in; jmp to an 
;	error_flash routine which is responsible for flashing the LED 5
;	times quickly and then the 6800 processor will be put into an
;	infinite, do nothing loop (simulates a halt).
;	This would allow the user to see that the test failed 
;	an overnight run.  Since the test is no longer running the normal
;	tests; the LEDs will not be flashing.
;
;	If set to loop_val (00):  
;	The test will not flash five times nor will it restart.  Instead it
;	attempts to loop on any error found until the error goes away.
;	This setting is the most usefull for trouble shooting with a scope
;	or logic probe if a logic analyzer is unavailble.  It is also fine
;	for using a logic analyzer as well.
;
;
;	The error_action byte will by default, be set to 00 (loop_val)
;	when the EPROM is made.  The user may wish to make an archival
;	copy of this prom and or set this byte to whatever he/she desires.
;	It may be preferrable to keep two copies of the test prom handy;
; 	each with a different setting of this byte.
;
;
;	Error Check In Feature:   (For use with a logic analyzer)
;
;	On an error the program will attempt to write to the reset_vector
;	of the 6800 (Address FFFE) in an attempt to allow triggering of
;	an analyzer on an intermittent error.  This is known as the
;	"error check in" location, and is READ ONLY.  Set the anlyzer to
;	End trace on any write to address FFFE.  Data should be
;	set to a don't care state.  The data you see is of no interest.
;	The only thing that counts is that you do a write to address FFFE.
;	You should be able to see what failed shortly before the error
;	check in location is written.
;
;	Another good qualifier to use for any trace if you have more than 24
;	channels available, is to trace on the address and or data you
;	are interested in and qualify it with reset being false.
;	Note:  Both Write (R/W) and Reset are low true signals; thus a
;	logic "1" high indicates you are reading or not reset respectively;
;	a 0 (low) would indicate you are writing or reseting respectively.
;
;
;	
;		Current # of tests/flashes and their meaning:
;
;		(The short double flash at start does NOT count as a flash
;		and neither does the one at the end. Currently there are
;		Seven (7) long flashes one per test.
;
;		Note:  A flash indicates that a test passed, prior to a flash,
;		you are running the testing associated with that flash.
;		Thus seeing two short flashes at the start and not the "first"
;		long flash would indicate you are failing the U7 data test.
;
;
;		All tests below will attempt to loop on an error; assuming
;		the error_action byte is set to loop_val (00).
;			
;		0.  Two short flashes of LED indicate program start. **
;		    Whether it be from power up or from the test finishing
;		    a pass and simply looping back to the start.
;
;		    ** Note:  Will be preceeded by short flashes indicating
;		    the Revision Number of the program if you momentarily
;		    short the NMI signal on the 6800 processor to ground.
;
;			
;		If the program successfully started you will see two short
;		flashes prior to the start of the first test.
;		Note:  In counting the number of flashes do not count these
;		two short flashes.  They simply mean the program is active
;		and also indicate the start and end of the program.
;		Every time the program loops through 7 flashes successfully,
;		you should see these two short flashes indicating the end of
;		the test; a pause, and another two short flashes indicating
;		the beginning of the test.
;
;		The test program will run forever and these two short
;		flashes can be used to identify the start of another pass
;		as well as indicate that the program has not gone haywire
;		due to some problem; watch for them!
;
;		The only other times short flashes are used currently are:
;
;		1.  If NMI is activated on the 6800 as mentioned above.
;		
;		2.  For an error report (based on the "error_action byte;
;		described previously) 
;
;		3.  During certain tests as described in the test descriptions
;		below, all indicate an error and all will be 10 flashes.
;
;
;		If the LED did not blink twice you should suspect that the
;		program is not running.
;
;		If it did blink two short times then you should now
;		be running test 1.  If the test passes succesfully; 
;		you should see the first real flash.
;
;
;		Flash 1.  Page zero data test passed:
;			All data patterns checked in all locations
;			in the 2101 ram.
;
;			Note:  As this is the first test run, any failure
;			can get hit here.  An open address line to a ram,
;			may cause this test to fail if it is floating
;			at a border line state.  Indications of this would
;			be failing data/address changing during different
;			passes through this test.  If the address line is
;			floating steadily high or steadily low then this
;			problem will normally be caught in the address tests.
;
;			Probable cause of failure:  almost anything at this
;			point.  Check data lines,  enables, and R/W line 
;			of both ram; and finally check address lines.
;			Enables on U7; should be active as well as R/W 
;			if error_action = loop_val.
;
;			Data and address can most easily be checked via a logic
;			analyzer or may be ohmed out to their destinations
;			
;			Now running test 2.
;
;		Flash 2. The first of two page0 address uniqueness tests passed.
;			All addresses checked in page0 for uniqness to
;			pg0_min address and pg0_min+1 address.
;
;
;			Note:  This is a weird way to do it but it is done
;			before address equal data test since due to the lack
;			of any stack space (we cannot trust U7 yet) and due
;			to lack of an extra register in the 6800 we cannot
;			get into an error loop from address equal data
;			until we have two "trusted" bytes in the 6810.
;			This will catch almost any address uniqness problem
;			at the 6810, except possibly a bad 6810.  That should
;			be caught by the address equal data test that follows.
;
;			Probable cause of failure is a faulty address line.
;
;			Now running test 3.
;
;		3.  Flash 3.  Address equal data test of page0 ram.  passed.
;			This is the second address uniqueness test of page0.
;
;			A loop is entered on failure which will write differing
;			data into two different locations in page0 ram that
;			conflict with each other.
;
;			Probable cause of failure:  
;			Open or shorted address line to page0 ram.
;
;			Now running test 4.
;
;		4.  Flash 4.  U7 uniq from U8 test 1 of 2 passed.
;
;			Write 00 to page1 check page0 is untouched.
;
;			Probable cause of failure:
;			Check page0 enable circuit followed by checking
;			for short between page1 enable signals and page0
;			enable circuit.  Also verify
;			R/W line is not shorted to page0 enable circuitry.
;			Possible though unlikely, a data line is shorted.
;
;			Now running test 5.
;
;		5.  Flash 5.  page0 uniq from page1 test 2 of 2 passed.
;
;			Write FF to page1 check page0 untouched.
;
;			Probable cause of failure:
;			open or shorted chip selects feeding page0 an or page1.
;			Check page0 and page1 enable circuits followed by 
;			checking for short between the page0 and page1 chip
;			selects.
;			Also verify R/W line is not shorted to page0 enable 
;			circuitry.  Possible though unlikely shorted data line.
;
;			Now running test 6.
;
;		6.  Flash 6.  page1 data test passed.
;
;			Check all data patterns to page1 CMOS RAM.
;
;			Probable cause of failure:
;			Open data line, open or shorted chip select or 
;			open or shorted R/W line to page1 CMOS RAM.
;
;			Now running test 7.
;
;		7.  Flash 7.  page1 address uniqueness test passed.
;
;			All addresses checked on page1.  Test passed.
;
;			Note:  The address uniqueness section of this test
;			will attempt to find the conflicting address to loop
;			on the error.  If it is unsuccessfull; the error
;			routine will short flash the LED 10 times and the
;			program will attempt to restart itself from scratch.
;
;			Probable cause of error:  open address line,
;			open or shorted chip select to page1.
;
;			Now running test 8.
;
;
;		Note:  All of the above "Probable cause" suggestions ASSUME
;			that you have  "Known Good" Parts in the board; if
;			not the part itself is the first most likely cause.
;
;		Program re-start may be accomplished by activating the
;		reset pushbutton (on the mpu board) at any time.
;		It may also be accomplished via the NMI signal on the
;		processor.  If done via NMI then an interrupt whose vector
;		points back to "show_revision".  Show_revision will cause 
;		the LED
;		To blink n times rapidly to correspond with the revision
;		number of the program.  This will be followed by a wait
;		of 2-3 seconds.  After which the program will restart.
;		Program restart can be recognized by its two short flashes.
;		
;
;
;
;File ATARI_04.ASM:
;two short flashes indicating the beginning!
;
;flash #1.  page0 data bits are unique.
;flash #2.  page0 addresses unique part 1.
;flash #3.  page0 addr uniq part 2. page0 is good.
;flash #4.  Writing page1 did not affect page0.
;flash #5.  page0/page1 uniq. Stack is now useable.
;flash #6.  page1 data bits are unique.
;flash #7.  page1 addresses are unique.
;
;switch_test containing possible two long flashes where the LEDs represent
;the external switch inputs in two nibbles.  May not be seen if inputs all off.
;
;two short flashes indicating The End.
;


; Notes on the Watchdog circuit:
;
; The watchdog timer causes reset to occur after a period of time.
; It is kept from going off after reset as long as A13 is set true
; (while we are in ROM 1).  Once out of ROM 1 the counter will start.
; and if software does not issue the watchdog reset signal.
; The interrupt routine must also issue an INTACK to clear the pending
; interrupt.  This means that if the normal game software is not running
; correctly that the result will be a pulsing reset.
;
; This diagnostic prom will be inserted into the ROM 3 socket.
; This diagnostic can be run with or without the watchdog timer disabled.
; If it runs disabled but not enabled then; then you should check out the
; WATCHDOG, INTACK and IRQ signals are working correctly.

;                       THE MEMORY MAP
;DECODER
;SIGNAL    7 DATA 0 R/W ADDRESS    15  ADDR BITS 00        FUNCTION
;==============================================================================
;PAGE0     DDDDDDDD R/W 0000-00FF  XX000XXXAAAAAAAA  WORKING RAM
;PAGE1         DDDD R/W 0800-08FF  XX001XXXAAAAAAAA  BATTERY BACKED RAM
;INREAD    D         R  1000       XX010XXXXXXXXXXX  SELF TEST INPUT
;INREAD     D        R  1000       XX010XXXXXXXXXXX  DISABLED SOLENOID INPUT
;INREAD      DDDDDD  R  1000-1007  XX010XXXXXXXXAAA  EXTERNAL SWITCH INPUTS 
;AUDWR0    D         W  1800       XX011XXX000XXXXX  AUDIO NOISE ENABLE
;AUDWR0     D        W  1800       XX011XXX000XXXXX  AUDIO WAVEFORM ENABLE
;AUDWR0      DD      W  1800       XX011XXX000XXXXX  AUDIO OCTAVE SELECT
;AUDWR0        DDDD  W  1800       XX011XXX000XXXXX  AUDIO WAVEFORM SELECT
;AUDWR1    DDDD      W  1820       XX011XXX001XXXXX  AUDIO FREQUENCY DIVISOR
;AUDWR1        DDDD  W  1820       XX011XXX001XXXXX  AUDIO AMPLITUDE CONTROL
;DISPWR        DDDD  W  1840-1846  XX011XXX010XXAAA  DISPLAY DATA OUTPUTS
;DISPWR         DDD  W  1847       XX011XXX010XX111  DISPLAY DIGIT ENABLE
;LAMPWR    DDDDDDDD  W  1860-1867  XX011XXX011XXAAA  LAMP OUTPUT
;SOLWR0        DDDD  W  1880       XX011XXX100XXXXX  SOLENOID OUTPUT
;SOLWR1           D  W  18A0-18A5  XX011XXX101XXAAA  INDEPENDENT CONTR0L OUTPUT
;SOLWR1           D  W  18A7       XX011XXX101XX111  SOLENOID ENABLE
;WATCHDOG            W  18C0       XX011XXX110XXXXX  WATCHDOG RESET
;INTACK              W  18E0       XX011XXX111XXXXX  INTERRUPT ACKNOWLEDGE
;DIPSWRD   DDDDDDDD  R  2000-2003  XX100XXXXXXXXXAA  DIP SWITCH INPUTS
;ROM1      DDDDDDDD  R  A800-AFFF  XX101AAAAAAAAAAA  ROM 1 SELECT
;ROM2      DDDDDDDD  R  3000-37FF  XX110AAAAAAAAAAA  ROM 2 SELECT
;ROM3      DDDDDDDD  R  3800-3FFF  XX111AAAAAAAAAAA  ROM 3 SELECT
;=============================================================================


yes		equ	$1	;
no		equ	$0	;
rom1strt	equ	$A800	;start addr for atari Rom 1.
rom1end		equ	$AFFF	;end addr for atari Rom 1.
rom2strt	equ	$3000	;start addr for atari Rom 2.
rom2end		equ	$37FF	;end addr for atari Rom 2.
rom3strt	equ	$3800	;start addr for atari Rom 3.
rom3end		equ	$3FFF	;end addr for atari Rom 3.
pg0_min		equ	$0000	;low addr for both 2101 RAM.
pg0_max		equ	$00FF	;high addr for both 2101 RAM.
pg1_min		equ	$0800	;low addr for page1 battery backed ram.
pg1_max		equ	$08FF	;high addr for page1 battery backed ram.
audwr0		equ	$1800	;Audio waveform sel and LED latch address.
inread		equ	$1000	;Address to read switch inputs from driver bd.
intack		equ	$18E0	;Any write to this will clear IRQ flop.
watchdog	equ	$18C0	;Any write to this will clear the timer.
led_mask	equ	$0F	;
led_on		equ	$0F	;set all four LEDs on.
led_off		equ	$00	;set all four LEDs off.
irq_vector	equ	$FFF8	;FFF8,FFF9  IRQ interrupt.
swi_vector	equ	$FFFA	;FFFA,FFFB SWI	interrupt.
nmi_vector	equ	$FFFC	;FFFC,FFFD NMI CLEAR SWITCH (SW 33).
reset_vector	equ	$FFFE	;FFFE,FFFF RESET vector.
loop_val	equ	$0	;see error_action settings, or flash_error.
restart_val	equ	$1	;see error_action settings, or flash_error.
halt_val	equ	$2	;see error_action settings, or flash_error.
revision_val	equ	$4	;Revision.
stack_strt	equ	$9F	;stack will be from 9F to 0, with the


;
; THE FOLLOWING LOCATIONS ARE RESERVED IN THE pg_0 ram AS PROGRAM VARIABLES
; THEY MAY NOT BE ACCESSED UNTIL AFTER WE HAVE DECIDED THE pg_0 ram IS OK.
;
; Variable region exists from addr $A0 to addr $FF.
var_1		equ	$A0	; two bytes for first variable in call
var_2		equ	$A2	; two bytes for second variable in call
var_3		equ	$A4	; two bytes for third variable in call
x_place		equ	$A6	; two bytes for allowing push of x.
temp		equ	$A8	; two bytes used for local temp storage.
disp_bcd	equ	$AA	; two bytes for display info.
blank_cnt	equ	$AC	; current blanking status
disp_cnt	equ	$AE	; for looping.
led_val		equ	$B0	; byte for led_value which is only a nibble.

; define  some important space for address uniqness testing of pg_0 ram RAM.
temp_0		equ	pg0_min	; two bytes of space at 00 and 01 in RAM.


	org 	rom3strt		;start of program.

;	Note:  "error_action" is intended to always be at rom3strt
;	for convenience and is the first byte of rom3.
;	The next byte contains the revision number.
;	Then a few bytes which contain a copyright notice.


error_action:
	fcb	#loop_val	;set to loop_val will cause loop on error.
 				;See previous description of error_action.

revision:
	fcb	#revision_val	;Put rev in rom also in case we want to know.


;		Now put in my copyright notice.
copyright:
	fcc	" COPYRIGHT 1994 BY DALLAS E. OVERTURF "

	fcc	" MY ATARI TEST PROM "

;	Issuing a Non Maskable Interrupt (NMI) will restart at show_revision.
;	Normal RESET will start at "start:"


show_revision:

	ldx	#30000		;wait a moment prior to flashing rev.
wait_rev:
	dex			;dec count.
	bne	wait_rev	;keep waiting.

	ldab	#revision_val	;b now has revision.
	ldx	#revision_ret	;x has return address.
	jmp	sflash_sp_svc	;go flash the rev.
revision_ret:
	ldaa	#2		;two times will be a very long wait.
revision_lp:
	ldx	#30000		;long wait.
rev_x_lp:
	dex			;dec wait count
	bne	rev_x_lp	;
	deca			;dec loop count.
	bne	#revision_lp	;branch if not done or fall through to start.

start:
	;1st write and or read some data and address patterns to see
	;that the bus is ok on the analyzer.

	ldx	#$0000	;An address to write to.
	ldaa	#$00		;some data to write= 00.
	staa	0,x		;unchecked data only to look at bus.
	coma			;some data to write= FF.
	staa	0,x		;unchecked data only to look at bus.
	ldaa	#$55		;some data to write= 55.
	staa	0,x		;unchecked data only to look at bus.
	coma			;some data to write= AA.
	staa	0,x		;unchecked data only to look at bus.
	ldx	#$0000		;address pattern to read from.
	ldaa	0,x		;read address, don't care about data.
	ldx	#$FFFF		;address pattern to read from.
	ldaa	0,x		;read address, don't care about data.
	ldx 	#$5555		;address pattern to read from.
	ldaa	0,x		;read address, don't care about data.
	ldx 	#$AAAA		;address pattern to read from.
	ldaa	0,x		;read address, don't care about data.

	;Now turn off LED so we know we are running code.

	ldaa	#led_off	;turn all four LEDs OFF.
	staa	audwr0		;LEDs should turn on now.

	ldx	#32700		;hold off so we can see the LED is off.

start_delay:
	dex
	bne	#start_delay

pass:

	ldx	#pass_return	;x has return address.
	ldab	#2		;flash count.  Double quick blink so we know
	jmp	sflash_sp_svc	;we are in code and where start is on a pass.
pass_return:

	ldx	#pg0_min		; Start addr of pg_0 ram.
	ldaa	#$0
pg0_data:

;	pg0_data attempts to check that each address can hold any data
;	pattern in the pg_0 ram.  Address uniqueness has not been checked yet.

	ldx	#pg0_min

pg0_d_nextadr:
	ldaa	#$0		;aa will be expected data
	ldab	#$ff		;ab will be actual. Init to anything else; we
				;start at zero so this must get zero on a read
	bra	#pg0_d_datloop	;

pg0_d_errloop:
	clr	#reset_vector	;failure check in.  Write to read only space.
	ldab	error_action	;sets CCs.
	beq	#pg0_d_datloop	;no flash error.
	jmp	flash_error	;

pg0_d_datloop:
	staa	0,x		;write to the pg_0 ram.
	ldab	0,x		;read from the pg_0 ram.
	cba			;compare a and b accumulators.
	bne	#pg0_d_errloop	;loop on the error till it clears.
	inca			;increment a.
	bne	#pg0_d_datloop	;data is 00-FF on FF we roll to 00 indicating
				;we have checked all data on current address.
	inx			;inc x to next addr.
	cpx	#pg0_max+1	;if past max addr then we are done.
	bne	#pg0_d_nextadr	;not done; do next address.
	ldab	#$1		;flash #1.  pg0 data bits are unique.

	ldx	#pg0_d_return	;x now has return address.
	jmp	#flash_sp_svc	;go flash, then fall through to next test.
pg0_d_return:			;This label must be here to return.

;===========================================================================

;	This test will verify that locations zero and one in pg0 are unique
;	from all other locations in pg0.  Thus we will have identified two
;	bytes of memory that can be used as stack space or temp storage in
;	the full pg0 address uniqueness test which is an address equal data
;	test.
;	By using all data patterns we verify that there is no data line(s)
;	shorted to an address line which would later confuse things for the
;	four addresses in question.  Probably most address uniqueness problems
;	will be caught here; however we will do adress equal data test later
;	to be very safe.  Note:  Between this test and the pg0_data_test; we
;	really should not see data shorted to an address.


	clra			;aa holds pattern to write.  patterns. 00-FF.
pg0_au0_part:
	ldx	#pg0_min+2
	bra	#pg0_au0_loop

pg0_au0_err:
	clr	#reset_vector	;failure check in.  Write to read only space.
	ldab	error_action	;sets CCs.
	beq	#pg0_au0_loop	;no flash error.
	jmp	flash_error	;

pg0_au0_loop:
	clrb			;to clear temp locs. We need some temp space.
	stab	pg0_min		;temp loc 0.  Part of temp_0.  Used for
	stab	pg0_min+1	;temp loc 1.  Part of temp_0.  Future storage.
	staa	0,x		;write test data to all other pg0 locations.
	ldab	pg0_min		;temp loc 0.  This will set CCs.
	bne	pg0_au0_err	;loop on error.
	ldab	pg0_min+1	;temp loc 1.  This will set CCs.
	bne	pg0_au0_err	;loop on error.
	inca			;inc aa to next data pattern.
	beq	#pg0_au0_ckdone
	bra	#pg0_au0_loop	;Do next data pattern.

pg0_au0_ckdone:
	cpx	#pg0_max		; check if really done.
	beq	pg0_au0_done	; yes we are really done.
	inx	
	clra			; Start pattern at 00 again.
	bra	#pg0_au0_loop	;not done continue looping.

pg0_au0_done:
	ldab	#$1		;flash #2.  pg0 addresses unique part 1.
				;we don't call flash_svc yet till we check
				;that all addresses are unique which occurs
				;next.
	ldx	#pg0_au0_return	;x has return address.
	jmp	#flash_sp_svc	;go flash.
pg0_au0_return:			;This label must be here to return.

	ldx	#20000		; since we passed hold off a bit so we can 
				; see the next flash occur for next test.
pg0_au0_delay:
	dex
	bne	#pg0_au0_delay


;===========================================================================

;	pg0 Address Equal Data test.  Final adress uniqness test of pg0.
;	It is not expected that this test will normally fail except
;	in the case of a bad pg_0 ram or an intermittent board.  However
;	this test can loop on error in case strange circumstances are
;	encountered, and we fail due to something other than a bad pg_0 ram.

pg0_ed_test:
	ldaa	#pg0_max
	ldx	#pg0_max
pg0_ed_dec:
	staa	0,x		;write pattern to address.
	deca			;next pattern.
	dex			;next address.
	cpx	#temp_0+1	;leave our two byte space alone!
	bne	pg0_ed_dec	;done if equal else continue.

;	pg0 should now be completely address equal data less first two locs.

	ldaa	#temp_0+2	;1st address in range we write to.
	ldx	#temp_0+2	;1st address in range we write to.
pg0_ed_inc:
	ldab	0,x		;read 1st address.
	cba			;compare expected in a, to actual in b.
	bne	pg0_ed_find_loc	;Try to find the conflict to loop on error.
	inca			;next pattern.
	inx			;next address.
	cpx	#pg0_max+1	;done when we reach pg0_max.
	bne	pg0_ed_inc	;continue if not done or fall through and pass.

;	pg0 should now be completely address equal data, less first two locs.

	jmp	pg0_ed_passed	;no error, we passed.

;	On entry x is the location in error found as follows:
;	The routine prior to this is expected to write data from max to
;	min addresses and then do the check from min to max so that 
;	temp_0 - 1 will be the highest address needed to find the error.
;	If this policy is not adhered to then this routine's ability to
;	localize the error will be compromised.
;	Note:  Temp_0 occupies the first two bytes of pg0 thus it MUST BE
;	gauranteed to to be uniqe to pg0 prior to using this routine.

pg0_ed_find_loc:

	clra			;
	staa	0,x		;clear error location.
	stx	temp_0		;store away error loc for now.
	ldaa	#$AA		;Compliment of first pattern.  We do two.

pg0_ed_next_dat:
	coma			;compliment so 1st patter will be 55.
	ldx	#temp_0+2	;initial loc we write to

pg0_ed_next_adr:
	staa	0,x		;write to location then check error loc.
	txs			;store x in sp for now.
	ldx	temp_0		;get error loc into x.
	ldab	0,x		;check it.  This will set CCs.
	tsx			;restore x from sp. This will not change CCs.
	bne	pg0_ed_found	;we will now go and try to loop on the error.
	inx			;next address.
	cpx	temp_0		;If >= temp_0, we've not yet found the problem.
	blt	pg0_ed_next_adr	;continue to next addr.
	cmpa	#$AA		;if AA then we are done.
	bne	pg0_ed_next_dat	;if 55 then we need to do AA still.
	bra	pg0_ed_unot	;If here we could not isolate the error.
				;in such a case let them know.
	
;	pg0_f_loop expects the bad address in temp_0 and conflict addr in x.
;	data written to find error will be in aa still.  aa has the data.
;	Note:  we enter at found:  we loop to found_lp.  This allows us to
;	see the problem occur immediately before error check in.
pg0_ed_foundlp:
	clr	#reset_vector	;failure check in.  Write to read only space.
	ldab	error_action	;sets CCs.
	beq	#pg0_ed_found	;no flash error.
	jmp	flash_error	;Set to flash on error.

pg0_ed_found:			;This is initial entry point.
	txs			;hold x in sp.	
	clrb
	ldx	temp_0		;x has error loc.
	stab	0,x		;clear error loc.
	tsx			;restore x.
	staa	0,x		;write to x, we expect to overwrite error loc.
	txs			;hold x in sp.
	ldx	temp_0		;x has error loc.
	ldab	0,x		;this will set CCs.
	tsx			;restore x.  This does not affect CCs.
	bne	pg0_ed_foundlp	;if not equal we are seeing the error.

pg0_ed_unot:

	ldab	#10		;setup to flash 10 times so we know that
				;we could not locate the error.
				;We will restart.
	ldx	pg0_edf_return	;x now has return address.
	jmp	sflash_sp_svc	;Let them know we cannot isolate error.
pg0_edf_return:

	ldx	#20000		;delay count.

pg0_ed_delay:
	dex
	bne	#pg0_ed_delay
	jmp	start		;There was an error so restart from scratch.


;	The next test expects that pg0 has addr equal data pattern in it.
;	make it so.

pg0_ed_passed:
	ldaa	#pg0_min
	ldx	#pg0_min
set_loop:
	staa	0,x
	inx			;next address.
	inca			;next pattern.
	cpx	#pg0_max+1	;if equal then we are done.
	bne	#set_loop	;go write pattern.

;	pg0 should now be completely address equal data.

	ldab	#$1		;flash #3.  pg0 addr uniq part 2. pg0 is good.
				;we don't call flash svc yet till we check
				;the page1 addressing; as we want to check 
				;that the page1 addressing does not affect
				;the pg_0 ram at pg0.
	ldx	#pg0_ed_return	;x now has return address.
	jmp	#flash_sp_svc	;go flash.
pg0_ed_return:

;	No delay loop is needed here.  We can see the LED just fine.

;===========================================================================

;	Verify that writing to pg1 does not write to pg0.

pg0pg1_uniq:
	ldx	#pg0_min		; start of pg0.
	ldaa	#$0		; zero to clear.

pg0pg1_clr:			; setup pg0 data to known 00 is simplest.
	staa	0,x		; clear pg0.
	inx
	cpx	#pg0_max+1	; check if done.
	bne	#pg0pg1_clr	; continue or fall through.
	lds	#pg1_max	; high addr of device we are checking against.
				; sp grows from high to low.
pg0pg1_lp:
	ldaa	#$FF		; Any pattern besides zero will do here.
	psha			; This will write a byte to pg1.
	tsx			; x now has a copy of sp+1. sp is unchanged.
	cpx	#pg1_min	; because got sp+1 this is a straight cmpare.
	beq	#pg0pg1_pass	; if done then pass.

pg0pg1_chk:
	ldx	#pg0_min	; Setup to check out pg0, it should still be 00.

pg0pg1_chklp:
	ldab	0,x		; read device location.  This will set CCs.
	bne	#pg0pg1_err	; if not zero goto error routine.

pg0pg1_reent:			; if error goes away in errloop then re-enter.
	inx			; inc x.
	cpx	#pg0_max+1	; check if done.
	bne	#pg0pg1_chklp	; loop if not done
	bra	#pg0pg1_lp	; current did not overwrite, next pg1 address.

pg0pg1_err:
	clr	#reset_vector	;failure check in.  Write to read only space.
	ldab	error_action	;sets CCs.
	beq	#pg0pg1_errlp	;no flash error.
	jmp	flash_error	;

pg0pg1_errlp:
	clr	0,x		; clear overwritten location.
	des			; sp will always be fail addr+1. fix it.
	psha			; write to suspect location.
	ldab	0,x		; check for failure.  This sets CCs.
	bne	#pg0pg1_errlp	; continue to loop on error if failing.
	bra	#pg0pg1_reent	;failure not solid;continue.

pg0pg1_pass:

	ldab	#$1		;flash #4.  pg0 is unique from other devices.

	ldx	#pg0pg1_return	;x now has return address.
	jmp	#flash_sp_svc	;go flash.
pg0pg1_return:

	ldx	#20000		; since we passed hold off a bit so we can 
				; see the next flash occur for next test.
pg0pg1_delay:
	dex
	bne	#pg0pg1_delay ; Delay to see flash.

;===========================================================================

	lds	#stack_strt	;initialize the stack for use.

;	This will be the first time we use the stack.  We shall hope
;	that it is in good enough shape to use!

	jsr	one_flash	;flash #5.  pg0/pg1 uniq. Stack is now useable.
				;note: pg0 has not been proven unique against
				;U10 at this point.  Will not likely matter.

	ldx	#20000		;since we passed hold off a bit so we can
				;see the next flash occur for next test.
pg08_s_delay:
	dex			
	bne	#pg08_s_delay

;===========================================================================

	; From this point we will call (jsr) the rest of the tests
	; for convenience.  Hopefully the stack has been tested well
	; enough that it is reliable!


;===========================================================================

do_calls:

; Test pg1 data.

	jsr	pg1_data_test	;
	jsr	one_flash	;flash #6.  pg1 data bits are unique.

;===========================================================================

; Test pg1 addressing unique.

	jsr	pg1_addr_test	;
	jsr	one_flash	;flash #7.  pg1 addresses are unique.

;===========================================================================
; Test switch inputs from driver board and flash values on the LEDs.
; All LEDs must have previously come on for this to mean anything.
; We do not want the driver board hooked up for this test.
; See switch_test for more info on how this is used.

	jsr	switch_test	; Contains 2 flashes one for low nibble
				; and one for high.  Note that one or both
				; flashes may not be seen if inputs are 
				; all low for their respective nibbles.
				; This should be the last test run.
;===========================================================================

fake_flash_08:	

	ldx	#20000		;delay to differentiate from two flash start
fake_08a:			;of test marker.
	dex			;
	bne	fake_08a	;
	ldab	#led_on		;All four LEDs on.
	stab	led_val		;Put in proper place.
	ldab	#1		;One flash.
	jsr	sflash_svc	;flash #8.  Short "fake" flash.

	ldx	#5000		;delay to differentiate from two flash start
fake_08b:			;of test marker.
	dex			;
	bne	fake_08b	;

fake_flash_09:
	ldab	#led_on		;All four LEDs on.
	stab	led_val		;Put in proper place.	
	ldab	#1		;One flash.
	jsr	sflash_svc	;flash #9.  Short "fake" flash.

	ldx	#30000		;delay to differentiate from two flash start
fake_09:			;of test marker.
	dex			;
	bne	fake_09		;

;===========================================================================

;	End of testing; restart the test from the beginning.

	jmp	start		;For now we are done; start all over again.

;===========================================================================


;	This is the start of all common routines.


	; This is the pg1 test.  It will verify data and address uniqueness.
	; because the page1 CMOS Ram is only 4 bits wide; we take some
	; great pains to see to it that we find conflicting addresses
	; in the address uniqueness test for this part so we can loop on
	; error.  The pain however is well worth the gain
	; as it may make using a scope easier; and certainly makes things
	; crystal clear on a logic analyzer.

pg1_data_test:

;	pg1_data attempts to check that each cell can hold any
;	data pattern in the page1.  address has not been qualified yet.


	ldx	#pg1_min

pg1_d_nextadr:
	ldaa	#$0		;aa will be expected data
	ldab	#$ff		;ab will be actual. Init to anything else; we
				;start at zero so this must get zero on a read
	bra	pg1_d_datloop	

pg1_d_errloop:
	clr	#reset_vector	;failure check in.  Write to read only space.
	ldab	error_action	;sets CCs.
	beq	#pg1_d_datloop	;no flash error.
	jmp	flash_error	;

pg1_d_datloop:
	anda	#$0f		;page1 is only 4 bits wide.  Mask unused bits.
	staa	0,x		;write to the page1.
	ldab	0,x		;read from the page1.
	andb	#$0f		;page1 is only 4 bits wide.  Mask unused bits.
	cba			;compare a and b accumulators.
	bne	#pg1_d_errloop	;loop on the error till it clears.
	adda	#$01		;inca on MSB side, lsb 4 bits stay zero.
	anda	#$0f		;just to be sure and we can set CCs here also.
	bne	#pg1_d_datloop	;data is 00-FF on FF we roll to 00 indicating
				;we have checked all data on current address.
	inx			;inc x to next addr.
	cpx	#pg1_max+1	;if past max addr then we are done.
	bne	#pg1_d_nextadr	;not done; do next address.

	rts			;return to caller.


pg1_addr_test:
	ldx	#pg1_min		;page1 starting address
	ldaa	#$0		;aa will be expected data
	staa	var_1		;Clear variable located in pg_0 ram used for uniq.
	ldab	#$ff		;ab will be actual. Init to anything else; we
				;start at zero so this must get zero on a read
	bra	#pg1_a_datloop

pg1_a_errloop:
	clr	#reset_vector	;failure check in.  Write to read only space.
	ldab	error_action	;sets CCs.
	beq	#pg1_a_datloop	;no flash error.
	jmp	flash_error	;

pg1_a_datloop:
	adda	var_1		;A+M->A var_1 allows us to do uniqueness.
	anda	#$0f		;page1 is only 4 bits wide.  Mask unused bits.
	staa	0,x		;write to the page1.
	ldab	0,x		;read/verify data.
	andb	#$0f		;page1 is only 4 bits wide.  Mask unused bits.
	cba			;compare a and b accumulators.
	bne	#pg1_a_errloop	;loop on the error till it clears.
	adda	#$01		;inca. msb, already has the uniq cnt offset.
	anda	#$0f		;page1 is only 4 bits wide.  Mask unused bits.
	bne 	#pg1_continue	;on 0 we will increment var_1.
	ldab	var_1		;get current cnt in b.
	addb	#$01		;inc b (really cnt) used to provide addr uniq.
	andb	#$0f		;page1 is only 4 bits wide.  Mask unused bits.
	stab	var_1		;put new count back in cnt.
pg1_continue:
	inx			;inc x to next addr.
	cpx	#pg1_max+1	;if past max addr then we are done.
	bne	#pg1_a_datloop	;not done; do next address.

pg1_addr_uniq:

pg1_uclr:
	ldx	#pg1_min		;Start addr of page1.
	ldaa	#$0

pg1_uclr_addr:
	staa	0,x		;clear page1.
	inx			;inc x to next addr.
	cpx	#pg1_max+1	;if past max addr then we are done.
	bne	#pg1_uclr_addr	;not done.

	ldx	#pg1_min
	ldaa	#$0f		;pattern to write to page1.

pg1_uloop:
	ldab	0,x		;check before writing.
	andb	#$0f		;mask out unused bits.
	cmpb	#$0		;verify loc is zero.	
	bne	pg1_uerror	;go try to loop on error.
	staa	0,x		;loc was zero; write pattern to loc.
	inx			;next loc.
	cpx	#pg1_max+1	;check if done.
	bne	pg1_uloop	;not done.

	rts			;return to caller.


pg1_uerror:
	stx	var_1		;loc that was in error.
	clr	0,x		;clear error loc.
	ldx	#pg1_min		;setup to find location that overwrote
				;the location just stored in var_1.
	ldaa	#$0f		;reload a with same data.

pg1_find_loc:

	staa	0,x
	txs			;hold x in sp for a moment.
	ldx	var_1		;now get fail addr to check.
	ldab	0,x		;
	tsx			;put x held in sp back into x.
	andb	#$0f		;mask out unused bits.
	stx	var_2		;Addr in var_2 confilicts with addr in var_1.
	cmpb	#$0		;check error loc was still zero.  If not,
	bne	pg1_uerr_loop	;then try again.
	inx			;try next loc.
	cpx	var_1		;don't need to go past addr in var_1.
	beq	pg1_unot		;did not find bad address.	

	ldaa	#$0f		;reload a to be safe.

;	Note:  We enter at pg1_uerr_loop, we loop to checkin. This allows us
;	to see the problem occur immediately before error check in.
pg1_checkin_lp:
	clr	#reset_vector	;failure check in.  Write to read only space.
	ldab	error_action	;sets CCs.
	beq	#pg1_uerr_loop	;no flash error.
	jmp	flash_error	;Set to flash on error.

pg1_uerr_loop:
	clr	0,x		;clear the addr in var_2.
	ldx	var_1		;address that destroyed data in loc var_2.
	staa	0,x		;write to var_1 should wipe data in var_2 addr.
	ldx	var_2		;check we destroyed data in var_2 addr.
	ldab	0,x		;read the data in var_2 addr.
	andb	#$0f		;mask unused.
	bne	#pg1_checkin_lp	;stay in as long as we are failing.

pg1_unot:
	ldab	#10		;Flash 10 times short so we know the error was 
	jsr	sflash_svc	;not located.
	jmp	start		;Error not located, re-start the whole thing.

;===========================================================================
;			This is the switch test.
;
;  This allows us to verify that the switch lines into the mpu can
;  be properly seen.  This is run with the mpu not connected to the
;  driver board and the inputs may be forced high or low at H7; the
;  74LS244 input buffer on the MPU board.
;

switch_test:

	ldaa	#inread		;get mpu switch inputs into aa.
	psha			;save switch value on stack.
	anda	#led_mask	;mask out top byte.
	staa	led_val		;set up to flash LEDs for low nibble.
	ldab	#$1		;flash once.
	jsr	flash_svc
	pula			;restore switch value.
	lsra	#$4		;move high nibble into place.
	anda	#led_mask	;mask out top byte.
	staa	led_val		;set up to flash LEDs for next four bits.
	ldab	#$1		;flash once.
	jsr	flash_svc	
	rts			;return to caller




;===========================================================================

;		The following are support routines.


;===========================================================================

	; flash_error.
	; This will set the b register to 5 flashes and jmp to sflash_sp_svc.
	; This routine does not return, it restarts.


flash_error:			;Currently we do not return so we do much.
				;and the heck with the stack.  It matters not.
	ldx	#20000		;delay so we can tell the difference.
				;when looping.
flash_err_delay:
	dex
	bne	#flash_err_delay ; Delay to see flashes clearly.

	ldab	#$5		;b now holds the number of times to flash.
	ldx	#flash_err_ret	;x has return address.
	jmp	sflash_sp_svc	;Go flash the light 5 short times.
				;sflash_sp returns to our caller.
flash_err_ret:
	ldaa	error_action	;Find out what to do here.
	cmpa	#halt_val	;
	beq	#halt_loop	;effectively halt the testing.
	cmpa	#restart_val	;
	beq	#flash_error_rst	;go restart tests.
				;any other value will currently restart also,
				;but is not guaranteed to do so in later
				;revisions.
flash_error_rst:
	jmp	start		;flash complete, now restart from scratch.

;===========================================================================

		;This loop does nothing and is an infinite loop on itself
		;as there is no halt instruction for the 6800.

halt_loop:
	clr	#reset_vector	;failure check in.  Write to read only space.
halt_lp:
	nop
	nop
	bra	#halt_lp	;loop on nothing to simulate a halted program.

;===========================================================================
		;common_return will allow us to save some space rather
		;than repeat code for the restore/rts of some calls.
		;It also makes us consistent.

common_return:

	pulb			;restoring x.
	pula			;restoring x.
	stab	x_place+1	;restoring x.
	staa	x_place		;restoring x.
	ldx	x_place		;x is now restored.
	pulb			;restore b.
	pula			;restore a.
	rts			;return to caller.

;===========================================================================

one_sflash:

	pshb			;save b register.
	ldab	#led_on		;All four LEDs on.
	stab	led_val		;Put in proper place.	
	ldab	#$1		;b now holds the number of times to flash.
	jsr	sflash_svc	;Go flash the light.

	pulb			;restore b register.
	rts			;return to caller.

;===========================================================================

	; one_flash is a called routine.  It will flash the led
	; N number of times based on the value in the b register.
	; there are no passed parameters.
one_flash:

	pshb			;save b register.
	ldab	#led_on		;All four LEDs on.
	stab	led_val		;Put in proper place.	

	ldab	#$1		;b now holds the number of times to flash.
	jsr	flash_svc	;Go flash the light.

	pulb			;restore b register.
	rts			;return to caller.

;===========================================================================

	; flash_svc:
	; flash the LED N times based on contents of b register.
	; Which LEDs are turned on are controlled by the contents
	; of led_val.  This allows us to take advantage of the four
	; LEDs on the atari system to show us things such as switch
	; input values etc... should we wish.  Only the normal flash_svc
	; and sflash_svc routines may do this since they utilize the stack.
	;
	; Registers are preserved.

flash_svc:

	psha			;Save A.
	pshb			;Save B.
	stab	temp		;b has count need to hold b reg temporarily.
	stx	x_place		; setup to push x also.
	ldaa	x_place		; 
	ldab	x_place+1	;
	psha			; x high onto stack
	pshb			; x low onto stack
	ldab	temp		; b should now have the flash count again.

flash_loop:

	ldaa	#led_val	;aa set to turn on LEDs, based on led_val.
	staa	audwr0		;LED should turn on now.
	ldx	#30000		;LED On count.

flash_on:

	dex			; should decrement x by 1.
	bne	#flash_on	; keep on for LED on count. 

;	setup to turn off now to complete flash.

	ldaa	#led_off	;aa set to turn all four LEDs off.
	staa	audwr0		;LED should turn off now.
	ldx	#30000		;LED Off count.

flash_off:
	dex			;should decrement x by 1.
	bne	#flash_off	;keep off for LED off count. 
	decb
	bne	#flash_loop	;go till done flashing.

	jmp	common_return	;common return routine to save some space.
				;and also keep things simple.

;===========================================================================

	; sflash_svc:
	; same as flash_svc except this does short flashes.
	; flash the LED N times based on contents of b register.
	; Registers are preserved.


sflash_svc:

	psha			;Save A.
	pshb			;Save B.
	stab	temp		;b has count need to hold b reg temporarily.
	stx	x_place		; setup to push x also.
	ldaa	x_place		; 
	ldab	x_place+1	;
	psha			; x high onto stack
	pshb			; x low onto stack
	ldab	temp		; b should now have the flash count again.

sflash_loop:

	ldaa	#led_on		;aa set to turn all four LEDs on.
	staa	audwr0		;LED should turn on now.
	ldx	#9000		;LED On count.

sflash_on:

	dex			; should decrement x by 1.
	bne	#sflash_on	; keep on for LED on count. 

;	setup to turn off now to complete flash.

	ldaa	#led_off	;aa set to turn all four LEDs off.
	staa	audwr0	;LED should turn off now.
	ldx	#22000		;LED Off count.

sflash_off:
	dex			; should decrement x by 1.
	bne	#sflash_off	; keep off for LED off count. 
	decb
	bne	#sflash_loop	; go till done flashing.

	jmp	common_return	;common return routine to save some space.
				;and also keep things simple.		

;===========================================================================

	; flash_sp_svc
	; flash the LED N times based on contents of b register.
	; On Entry this routine expects the return address in x.
	;
	; This routine should be used prior to stack availability.
	; flash_svc is a called routine that should be used once 
	; the stack has been tested good if you need to save registers.
	; This routine does not save any registers!!

flash_sp_svc:

	txs			; Hold return address in sp.

flash_sp_loop:

	ldaa	#led_on
	staa	audwr0		;LEDs should turn on now.
	ldx	#30000		;LED On count.

flash_sp_on:

	dex			; should decrement x by 1.
	bne	#flash_sp_on	; keep on for LED on count. 

;	setup to turn off now to complete flash.

	ldaa	#led_off	;set all four LEDs OFF.
	staa	audwr0		;LEDs should turn on now.
	ldx	#30000		;LED Off count.

flash_sp_off:
	dex			;should decrement x by 1.
	bne	#flash_sp_off	;keep off for LED off count. 
	decb
	bne	#flash_sp_loop	;go till done flashing.
	tsx			;restore return address from sp.
	jmp	0,x		;return to caller.

;===========================================================================

	; sflash_sp_svc
	; flash the LED N times based on contents of b register.
	; On Entry this routine expects the return address in x.
	;
	; This routine should be used prior to stack availability.
	; sflash_svc is a called routine that should be used once 
	; the stack has been tested good if you need to save registers.
	; This routine does not save any registers!!

sflash_sp_svc:

	txs			; Hold return address in sp.

sflash_sp_loop:

	ldaa	#led_on		;set all four LEDs ON.
	staa	audwr0		;LEDs should turn on now.
	ldx	#9000		;LED On count.

sflash_sp_on:

	dex			; should decrement x by 1.
	bne	#sflash_sp_on	; keep on for LED on count. 

;	setup to turn off now to complete flash.


	ldaa	#led_off	;set all four LEDs OFF.
	staa	audwr0		;LEDs should turn on now.
	ldx	#22000		;LED Off count.

sflash_sp_off:
	dex			; should decrement x by 1.
	bne	#sflash_sp_off	; keep off for LED off count. 
	decb
	bne	#sflash_sp_loop	; go till done flashing.
	tsx			;restore return address to x.
	jmp	0,x		;return to caller.

;===========================================================================

	; hw_intrpt
	; This routine is entered on a hardware interrupt from IRQ.
	; We will issue WATCHDOG RESET  then INTACK and then return.
	; Note that the writes issued to by intack and watchdog
	; are don't care on the data.  Thus we do not setup data
	; in the A register prior to the write and we don't need
	; to save anything to the stack.
	; 

hw_intrpt:

	staa	watchdog	; clear watchdog counter to avoid RESET.
	staa	intack		; clear the pending interrupt.
	rti			; return from interrupt.

;===========================================================================

	; Vector setup for various interrupts should they occur.

	org	rom3end-7	; start of 6800 vector table.

	; Point unused interrupts to halt_loop.
	; Some day we may toy some more with them given a reason.

	fdb	hw_intrpt	; FFF8,FFF9 IRQ	CURRENTLY NOT USED HERE.
	fdb	halt_loop	; FFFA,FFFB SWI	CURRENTLY NOT USED HERE.
	fdb	show_revision	; FFFC,FFFD NMI CLEAR SWITCH (SW 33).
	fdb	start		; FFFE,FFFF RESET.

;===========================================================================
