
; Program:  Atariint.asm  Written February 1994 By:  Dallas E. Overturf
;           Public Domain Code;
;
; 		Attempting to allow basic testing of
;		the atari driver board and possibly using interrupts.
;		This code is being written to run without interrupts
;		but is also intending to try and allow them to be
;		used at a later time. Note:  Currently the cpu code
;		will run with or without interrupts enabled. Thus if you see
;		a pulsing reset line to the 6800 try grounding the watchdog
;		disable test point.  If this then runs partway you know
;		that your problem is in the watchdog circuit.  If not
;		The code got lost.  Also note that driver code is untested.
;		I got bored with this program and it is as far as I have gone.
;		Feel free to send me a copy if you add any good stuff to it.
;		I just don't have any Atari games so this is no big deal.
;		I wrote this to bring up a friend's game. 
;
; STATUS:  Under development push/pull X calls in place.  Code has been
;		tried out thus far and appears to be working.
;		Solenoid and lamp tests are coded and appear to
;		function; at least to the point where we do not
;		go into hyperspace or get hung up.  Need to verify
;		solenoids and lamps with a logic probe.
;		Interrupts are working.  This prom will run with 
;		or without the watchdog disabled!
;
;		Switch test is coded and ready to try out.
;
;		Other tests are commented out for now.
;
;
;
; This program may not be sold, used in a commerical environment, packaged
; with a commerical product, or used for profit in any way without prior
; written permission from the author except as noted in the "User Aggreement"
; section below.
; 
; The author makes no warranties as to the contents of this program or related
; documentation and specifically disclaims any implied warranties of
; merchantability or fitness for any particular purpose.  In no event shall the
; author be liable for loss of profit or any other commercial damage, including
; but not limited to special, incidental, consequential or other damages.
; 
; The author reserves the right to modify the program and/or documentation
; without obligation to notify any person or organization of these changes.
;
;
; Written by Dallas E. Overturf
;
; Copyright 1994 by Dallas E. Overturf.
;
; User Aggrement:
;
; You (the user) may freely modify, copy, and or change the code provided
; that:
; 
; You retain the original copyright notice; in all or any part of the code
; that you use, copy, and or modify.
;
; You may sell the code (binary and or source) provided that the total cost
; charged (including all charges such as shipping and handling) is no greater
; than ten dollars ($10.00) U.S currency; or the equivalent in foreign 
; (non-U.S.A.) currency.
; 
;
;
; Questions Comments etc... On this program may be directed to the
; author at the following address (If you wish a reply please send a SASE)
;
; Dallas E. Overturf
; 267 Main St
; Ashland, MA 01721 (USA)
;
;
; Comments, pleas etc... from the Author:
;
; If you decide to make your own version of the code either by patching
; the binary or by changing the source; please be considerate and patch
; patch the revision byte to a value of 01, 02, or 03 to indicate that the
; copy is a hacked up copy.  I will not use revisions 01, 02, or 03 in a
; released copy.  The first released copy will be revision 04.
; If you modify the source; then please keep the Modification History 
; and or Bug history sections updated as appropriate.
;
; The code:  This code was not written for efficiency and was done in
; my spare time.  No doubt there are things I did that I could have done
; better. It does work the way I intended it to so I'm not planning
; on re-writing it!  If you submit a bug to me that I deem to be a valid
; bug; I will probably fix it; if it can be reasonably fixed (my opinion).
;
; If you come up with something "great" you want added to this code I may
; be willing to consider doing it if it seems reasonable (my opinion).
;
;		Happy Pinballing!
;					Dallas...
;
;
; Modification history
;
; 12-10-92:  DEO  Rev 1.0. Initial Release.
;		  Base file: atari_04.asm.
;
;	Status:  Currently under development.  This program is intended
;		to allow basic testing of the atari driver board.
;		Problems:  Encountering difficulty in dealing with
;		the lack of registers on the 6800.  Need to find
;		a good way of pushing X onto the stack; or saving it.
;		The way may be what we did in the svc subs.
;		the switch test needs to inc an offset or something.
;		this is a real pain.  Trouble is we will end up wasting
;		time pushing and popping etc...
;
;
; End Modification History
;
;
; Bug History
;
; 03-25-94:  DEO Rev 1.0.  None known at this time.
;
; Testing Notes:
;
;	The Atari board accepts 2716 EEPROM.  Thus this test prom
;	should be a 2716; or it may be a 2732 by programming only
;	the upper half of the 2732 (addresses 0x0800-0x0FFF) in 
;	which case the 2732 may be used as a 2716 in the board.
;
;	This test prom is placed into socket ROM3 (location J7) to run.
;	Additionally to run the test you MUST tie to ground the
;	"WATCHDOG KILL" test point on the board to disable the Watchdog
;	timer.  If this prom runs without doing this then this indicates
;	that the watchdog timer is not counting up and issuing a board
;	reset which should occur when the board is not running code that
;	deals with this timer; thus if you see reset pulsing when the
;	NORMAL game rom are in the board; that is an indication that the
;	board is for one reason or another not running it's code.
;	In this case should this test appear to run okay; you should verify
;	that you are getting both the clear watchdog signal, the IACK
;	signal, and also the IRQ signal itself up to the processor.  Note that
;	both IACK and the watchdog clear are software driven by the normal
;	game rom if your game rom are good and can be accessed properly.
;
;
;	Programming the code into an EPROM:
;	For a 2716 just program it in normal (IE: start address is 0000).
;	If your burner requires an end address, it is 07FF.
;
;	For a 2732 the start address is 0800 (this means 0000-07FF
;	remain blank.  If your burner requires an end address, it is 0FFF.
;
;	Note:  If the binary file of this program is larger than 2048 bytes;
;	and the file starts at address 0000; then you will need to give your
;	prom programmer a starting address within the binary file as well to
;	start taking data from.  In this case you need to give it address
;	3800 which will be the offset (in hexadecimal) of the start of code
;	in the binary file (assuming the file starts from address 0x0000.
;
;	It is recommended that you verify the prom in a known good board
;	prior to attempting to use it on a bad board!
;
;	Things that may keep you from comming up and working correctly
;	on the game rom that are untested in this prom.
;	INTERRUPTS!!  This test program currently does not do any
;	checking of program interrupts (IRQ lines to the 6800).
;
;
;	Some basic suggestions when trouble shooting a board:
;
;	If the test prom does not work at all; any remaning game rom
;	should also be removed.  Thus this can help prove that
;	there is not a bad game rom on the board.  If this test prom can
;	get through it's set of tests on your mpu, and you do not
;	ever see IACK or watchdog clear occuring then it is recommended
;	that you try a different set of KNOWN GOOD game rom in your mpu to
;	verify that your game rom are good.
;
;	This test program was designed with the intent of finding problems
;	that are due to the battery corroding on the board.  This normally
;	means an OPEN (broken) etch or bad connection within a socket.
;	It is also designed to deal with dead board type problems; however
;	to use this in most dead board conditions you will need to have
;	a 24 channel logic analyzer at your disposal (8 data, 16 address)
;	and of course this listing so you can see what went wrong.
;
;	Note:  This prom should only be run once the Processor PCB has been
;	checked	out and is known to be basically good.
;
;	This program allows you to check out the various I/O PCB;
;	functions on Atari pinball I/O PCB used in Superman and Hercules
;	Via a scope and or logic probe or logic analyzer.
;	This includes lamps, solenoids which may be basically verified with
;	as little as a simple home brew logic probe.  Additionally it will
;	allow you to verify the switch lines are working by grounding the
;	switch line.  The program will blink the binary coding of the 
;	switches in the LEDs.  Since the switch lines occupy 8 bits (1 byte)
;	of data and there are only four LEDs; the program will blink the
;	low nibble (four bits) of the byte (data bits 0-3) then it will blink
;	the high nibble (data bits 4-7).  There are eight 8 lines to one line
;	switch multiplexers that can each select one of 8 inputs the 8
;	multiplexers each put out a bit onto the data bus to form the full
;	byte.  Thus you will see two flashes per byte (one per nibble) times
;	8 possible selections for a total of 16 flashes of the four LEDs to
;	read all 64 switch lines once time.
;	
;	
;
;
;	Note:  Upon start of the program there will be two short flashes
;	of all four of the LEDs.
;	This is done as a sanity check so that the user can see that the 
;	actual test code has in fact started to run or has made a pass and
;	is starting again.
;
;	Note:  In the case of a reset or an NMI interrupt; these two short
;		flashes will be preceeded by a revision a pause, then
;		the two short flashes will occur. the program will not
;		show the revision every time it makes a pass through.
;		You will not normally see an NMI interrupt on this system
;		unless you short it to ground.  Thus Reset via the reset
;		button or a power on will be the only time you see the 
;		revision.  The lowest revision for this program will be
;		4 (four).  Your modified versions of the code should have
;		their revisions set to 1, 2 or 3 so we know that they are
;		modified.
;
;	
;		Current # of tests, flashes and their meaning:
;
;		(The short double flash at start does NOT count as a flash
;		nor does the preceeding revision count on power up or reset.
;
;		Program re-start may be accomplished by activating the
;		RESET pushbutton (on the mpu board) at any time.
;		
;
;
;
;File ATARIINT.ASM:
;
; Notes on the Watchdog circuit:
;
; The watchdog timer causes reset to occur after a period of time.
; It is kept from going off after reset as long as A13 is set true
; (while we are in ROM 1).  Once out of ROM 1 the counter will start.
; and if software does not issue the watchdog reset signal.
; The interrupt routine must also issue an INTACK to clear the pending
; interrupt.  This means that if the normal game software is not running
; correctly that the result will be a pulsing reset.
;
; This diagnostic prom will be inserted into the ROM 3 socket.
; This diagnostic must be run with the watchdog timer disabled at this time.
; A later version will attempt to run with interrupts.  In fact this version
; was written to run with them but has not been checked against them yet.
; 
; If it runs disabled but not enabled then; then you should check out the
; WATCHDOG, INTACK and IRQ signals are working correctly.

;                       THE MEMORY MAP
;DECODER
;SIGNAL    7 DATA 0 R/W ADDRESS    15  ADDR BITS 00        FUNCTION
;==============================================================================
;PAGE0     DDDDDDDD R/W 0000-00FF  XX000XXXAAAAAAAA  WORKING RAM
;PAGE1         DDDD R/W 0800-08FF  XX001XXXAAAAAAAA  BATTERY BACKED RAM
;INREAD    D         R  1000       XX010XXXXXXXXXXX  SELF TEST INPUT
;INREAD     D        R  1000       XX010XXXXXXXXXXX  DISABLED SOLENOID INPUT
;INREAD      DDDDDD  R  1000-1007  XX010XXXXXXXXAAA  EXTERNAL SWITCH INPUTS 
;AUDWR0    D         W  1800       XX011XXX000XXXXX  AUDIO NOISE ENABLE
;AUDWR0     D        W  1800       XX011XXX000XXXXX  AUDIO WAVEFORM ENABLE
;AUDWR0      DD      W  1800       XX011XXX000XXXXX  AUDIO OCTAVE SELECT
;AUDWR0        DDDD  W  1800       XX011XXX000XXXXX  AUDIO WAVEFORM SELECT
;AUDWR1    DDDD      W  1820       XX011XXX001XXXXX  AUDIO FREQUENCY DIVISOR
;AUDWR1        DDDD  W  1820       XX011XXX001XXXXX  AUDIO AMPLITUDE CONTROL
;DISPWR        DDDD  W  1840-1846  XX011XXX010XXAAA  DISPLAY DATA OUTPUTS
;DISPWR         DDD  W  1847       XX011XXX010XX111  DISPLAY DIGIT ENABLE
;LAMPWR    DDDDDDDD  W  1860-1867  XX011XXX011XXAAA  LAMP OUTPUT
;SOLWR0        DDDD  W  1880       XX011XXX100XXXXX  SOLENOID OUTPUT
;SOLWR1           D  W  18A0-18A5  XX011XXX101XXAAA  INDEPENDENT CONTR0L OUTPUT
;SOLWR1           D  W  18A7       XX011XXX101XX111  SOLENOID ENABLE
;WATCHDOG            W  18C0       XX011XXX110XXXXX  WATCHDOG RESET
;INTACK              W  18E0       XX011XXX111XXXXX  INTERRUPT ACKNOWLEDGE
;DIPSWRD   DDDDDDDD  R  2000-2003  XX100XXXXXXXXXAA  DIP SWITCH INPUTS
;ROM1      DDDDDDDD  R  A800-AFFF  XX101AAAAAAAAAAA  ROM 1 SELECT
;ROM2      DDDDDDDD  R  3000-37FF  XX110AAAAAAAAAAA  ROM 2 SELECT
;ROM3      DDDDDDDD  R  3800-3FFF  XX111AAAAAAAAAAA  ROM 3 SELECT
;=============================================================================


yes		equ	$1	;
no		equ	$0	;
rom1strt	equ	$A800	;start addr for atari Rom 1.
rom1end		equ	$AFFF	;end addr for atari Rom 1.
rom2strt	equ	$3000	;start addr for atari Rom 2.
rom2end		equ	$37FF	;end addr for atari Rom 2.
rom3strt	equ	$3800	;start addr for atari Rom 3.
rom3end		equ	$3FFF	;end addr for atari Rom 3.
pg0_min		equ	$0000	;low addr for both 2101 RAM.
pg0_max		equ	$00FF	;high addr for both 2101 RAM.
pg1_min		equ	$0800	;low addr for page1 battery backed ram.
pg1_max		equ	$08FF	;high addr for page1 battery backed ram.
audwr0		equ	$1800	;Audio waveform sel and LED latch address.
inread		equ	$1000	;Base address for reading switch inputs from.
inread_max	equ	$1007	;Mux selects 1 of 8 (0-7) on driver board.
lamp_loop_cnt	equ	30000	;Test loop count so we can see on scope.
lampwr		equ	$1860	;Base addr to write lamp data to.
lampwr_max	equ	$1867	;Mux selects 1 of 8 (0-7) on driver board.
display_data	equ	$1840	;Start address of display data.
display_max	equ	$1846	;Mux selects 1 of 7 (0-6).
disp_digit_en	equ	$1847	;Address of digit enables for displays.
lamp_data	equ	$1860	;Start address of lamps.
dipswrd		equ	$2000	;Start address of dip switches.
dipswrd_max	equ	$2003	;Mux selects 1 of 4 (0-3).
intack		equ	$18E0	;Any write to this will clear IRQ flop.
watchdog	equ	$18C0	;Any write to this will clear the timer.
led_mask	equ	$0F	;The LEDs occupy only a nibble.
leds_on		equ	$0F	;set all four LEDs on.
led_off		equ	$00	;set all four LEDs off.
irq_vector	equ	$FFF8	;FFF8,FFF9  IRQ interrupt.
sol_0_high	equ	$0F	;Max 16 outputs for solwr0, this is data!
solenwr1        equ	$18A7	;set/clear solenoid enable based on D0.
solwr0		equ	$1880	;Address to most solenoids based on D0-D3.
solwr1          equ	$18A0   ;18A0-18A5, independent control solenoids set
solwr1_max	equ	$18A5	;and cleared based on state of D0.  via solwr1.
sol_loop_cnt	equ	30000	;Test loop count so we can see on scope.
swi_vector	equ	$FFFA	;FFFA,FFFB SWI	interrupt.
nmi_vector	equ	$FFFC	;FFFC,FFFD NMI CLEAR SWITCH (SW 33).
reset_vector	equ	$FFFE	;FFFE,FFFF RESET vector.
allow_ints	equ	$FF	;Non zero allows interrupts.
loop_val	equ	$0	;error_action setting, not used.
restart_val	equ	$1	;error_action setting, not used.
halt_val	equ	$2	;error_action setting, not used.
revision_val	equ	$4	;Revision.
stack_strt	equ	$9F	;stack will be from 9F to 0, with the
				;upper section reserved for variables.

;
; THE FOLLOWING LOCATIONS ARE RESERVED IN THE pg_0 ram AS PROGRAM VARIABLES
; THEY MAY NOT BE ACCESSED UNTIL AFTER WE HAVE DECIDED THE pg_0 ram IS OK.
;
; Variable region exists from addr $A0 to addr $FF and resides in page zero.

		org	$A0

acc_place	rmb	2	; two bytes reserved by push/pull X routines.

lamp_offset	rmb	2	; two bytes of offset into lamps.
p_x_place	rmb	2	; two bytes reserved by push/pull X routines.
sol_1_offset	rmb	2	; two bytes of offset into solwr1 solenoids.
sp_place	rmb	2	; two bytes reserved by push/pull X routines.
switch_sel	rmb	2	; two bytes of offset into switches.
temp		rmb	2	; two bytes used for local temp storage.
test_loop_cnt	rmb	2	; two bytes for a test for a loop count.
var_1		rmb	2	; two bytes for first variable in call
var_2		rmb	2	; two bytes for second variable in call
var_3		rmb	2	; two bytes for third variable in call
wait_cnt	rmb	2	; two bytes for wait count.
x_place		rmb	2	; two bytes for holding X.
int_status	rmb	1	; a byte indicating we will allow interrupts 
				; when safe.  Ie: setup is complete.  Note this
				; is NOT intened to relfect the state of the
				; interrupt mask!!!!!
led_val		rmb	1	; a byte for led_value which is only a nibble.


;	Note:  "error_action" is intended to always be at rom3strt
;	for convenience and is the first byte of rom3.
;	The next byte contains the revision number.
;	Then a few bytes which contain a copyright notice.

	org 	rom3strt		;start of program.

error_action:
	fcb	#loop_val	;set to loop_val will cause loop on error.
 				;See previous description of error_action.
revision:
	fcb	#revision_val	;Put rev in rom also in case we want to know.


;		Now put in my copyright notice.
copyright:
	fcc	" WRITTEN 1994 BY DALLAS E. OVERTURF "

	fcc	" MY ATARI DRIVER TEST PROM "

;	Issuing a Non Maskable Interrupt (NMI) will restart at show_revision.
;	Normal RESET will start at "start:"


show_revision:

	clr	watchdog	; clear watchdog counter to avoid RESET.
	clr	intack		; clear any pending interrupt.
	ldaa	#allow_ints	; we will set int_status to allow interrupts.
	staa	int_status	; we are allowing interrupts.
	lds	#stack_strt	; initialize the stack for use.
	jsr	restore_mask	; enable interrupts if we are allowing them.

	ldaa	#leds_on	;
	staa	led_val		; turn on all four LEDs.
	ldab	#$1		; One long flash and also leaves LEDs off.
	jsr	flash_svc	; 
	ldx	#30000		;
	stx	wait_cnt	;pause before revision!
	jsr	wait_svc	;
	ldab	#revision_val	;b now has revision.
	staa	led_val		;a should still have #leds_on value.
	jsr	sflash_svc	;go flash the rev in short flashes.
	stx	wait_cnt	;another pause after revision.
	jsr	wait_svc	;

start:
	sei			; set mask, disallow IRQ interrupts.
	clr	watchdog	; clear watchdog counter to avoid RESET.
	clr	intack		; clear any pending interrupt.
	ldaa	#allow_ints
	staa	int_status	; we are allowing interrupts.
	lds	#stack_strt	; initialize the stack for use.
	clr	audwr0		; Turn the LEDs OFF.
	ldx	#30000		; 
	stx	wait_cnt	;
	clr	watchdog	; clear watchdog counter to avoid RESET.
	jsr	restore_mask	; set enable interrupts if they are allowed.
	jsr	wait_svc	; Let's see LEDs in the off state.
	ldab	#2		; Flash count.
	ldaa	#leds_on	; Set all four LEDs ON.
	staa	led_val		;
	jsr	sflash_svc	; short double flash indicates start.

;===========================================================================

do_calls:

;===========================================================================
; Test switch inputs from driver board and flash values on the LEDs.
; All LEDs must have previously come on for this to mean anything.
; We do not want the driver board hooked up for this test.
; See switch_test for more info on how this is used.


;	jsr	lamp_test	; Go test the lamps.
;	jsr	sol_0_test	; Go test solenoids associated with solwr0.
;	jsr	sol_1_test	; Go test solenoids associated with solwr1.
	jsr	switch_test	; Contains 2 flashes one for low nibble
				; and one for high.  Note that one or both
				; flashes may not be seen if inputs are 
				; all low for their respective nibbles.
				; This should be the last test run.
	ldab	#2		; we will flash 2 short for end of test;	jsr	sflash_svc	;
	ldx	#30000		;wait count.
	stx	wait_cnt	;
	jsr	wait_svc	;wait a bit so we can tell it is start of test.

;	End of testing; restart the test from the beginning.

	jmp	start		;For now we are done; start all over again.

;===========================================================================


;===========================================================================
;			This is the switch test.
;
;  This allows us to verify that the switch lines into the mpu can
;  be properly seen.  This is run with the mpu not connected to the
;  driver board and the inputs may be forced high or low at H7; the
;  74LS244 input buffer on the MPU board.
;
;  Registers A,B,X are saved by this routine.
;


switch_test:
	
	psha			;Save A.
	pshb			;Save B.
	jsr	pshx_svc	;Save X.

	ldx	#inread		;base address of switches.
	stx	switch_sel	;switch_sel now at base addr of switches.
	ldx	#10000		;some time to wait between tests.
	stx	wait_cnt	;
	jsr	wait_svc	;wait a bit between nibbles.

switch_next:
	ldx	switch_sel	;get switch select val.
	ldaa	0,x		;get mpu switch inputs into aa.
	psha			;save switch value on stack.
	staa	led_val		;set up to flash LEDs for low nibble.
	ldab	#$1		;flash once then do 2nd half of byte.
	jsr	flash_svc	;go flash the LEDs.
	ldx	#10000		;some time to wait.
	stx	wait_cnt	;
	jsr	wait_svc	;wait a bit between nibbles.
	pula			;restore switch value.
	lsra			;moving high nibble into place.
	lsra			;moving high nibble into place.
	lsra			;moving high nibble into place.
	lsra			;high nibble now in place.
	staa	led_val		;set up to flash LEDs for next four bits.
	ldab	#$1		;flash once.
	jsr	flash_svc	;go flash the LEDs.
	ldx	#30000		;some time to wait.
	stx	wait_cnt	;
	jsr	wait_svc	;wait after the second nibble.	
	ldx	switch_sel	;
	inx			;increment switch sel.
	stx	switch_sel	;
	cpx	#inread_max+1	;Check if we've read all the switches.
	bne	switch_next	;Re-read switches based on new mux selects.

	jsr	pulx_svc	;Restore X.
	pulb			;Restore B.
	pula			;Restore A.
	rts			;Done; return to caller


;===========================================================================
;			This is the lamp test.
;
; Purpose:  Verification of lamp driver logic.
;
; You may check the lamp outputs at the connectors or at the "4099" muxes
; with a scope and or logic probe.  You should see all outputs toggle.
;
; This test must be run with the mpu connected to the driver board.
; ALL lamp lines should be toggling high and low out of the 4099 MUXs.
; 
; Registers A,B,X are preserved.
;

lamp_test:
	
	psha			;Save A.
	pshb			;Save B.
	jsr	pshx_svc	;Save X.
	ldx	#lamp_loop_cnt	;number of times to loop through test.
	stx	test_loop_cnt	;Do enough times to see with scope or probe.

lamp_contin:
	ldx	#lampwr		;Base addr of lamps.
	stx	lamp_offset	;Init. lamp_offset particularly high byte.

lamp_next:
	ldx	lamp_offset	;X now has select count.
	ldab	#$FF		;set up to turn on 8 lamp drivers.
	stab	0,x		;set the 8 bits.
	nop			;give us some ON time.
	clr	0,x		;Turn OFF 8 lamp drivers.
	inx			;inc lamp offset to next set.
	stx	lamp_offset	;
	cpx	#lampwr_max	;Check if we've written all the lamps.
	ble	lamp_next	;Go do the next 8 lamp drivers.
	ldx	test_loop_cnt	;
	dex			;Check the test's loop count, zero if done.
	stx	test_loop_cnt	;This will set CCs.
	bne	lamp_contin	;Not done do another pass.

	jsr	pulx_svc	;Restore X.
	pulb			;Restore B.
	pula			;Restore A.
	rts			;Done; return to caller

;===========================================================================
;		This is the solenoid test for solwr1.
;
; Purpose:  Verification of solenoid driver logic.
;
; You may check the solenoid outputs at the connectors or at the applicable
; muxes with a scope and or logic probe.  You should see all outputs toggle.
;
; This test will toggle all solenoid outputs related to solwr1 as shown on
; the schematics.
;
; This test must be run with the mpu connected to the driver board.  All
; solenoid lines should be toggling high and low at their respective muxes.
; This includes the solenable and soldisable signals.
;
; Registers A,B,X are preserved.
;

sol_1_test:
	
	psha			;Save A.
	pshb			;Save B.
	jsr	pshx_svc	;Save X.
	ldx	#sol_loop_cnt	;number of times to loop through test.
	stx	test_loop_cnt	;Do enough times to see with scope or probe.

sol_1_contin:
	ldx	#solenwr1	;We will turn on/off/on solen here.
	ldab	$1		;set up to turn on solenoid enable.
	stab	0,x		;Turn ON solenable based on D0 at solwr1 mux.
	clr	0,x		;Turn OFF solenable based on D0 at solwr1 mux.
	nop			;Give us some OFF time on this one.
	stab	0,x		;Leave ON solenable during test.
	ldx	#solwr1		;Base addr of solwr1 solenoids.
	stx	sol_1_offset	;Init. sol_1_offset particularly high byte.

sol_1_next:
	ldx	sol_1_offset	;X now has select count.
	ldab	#$1		;set up to turn on a solenoid with solwr1.
	stab	0,x		;Turn on sol based on D0 at solwr1 mux.
	nop			;Give us some ON time.
	clr	0,x		;Turn OFF sol based on D0 at solwr1 mux. 
	inx			;inc sol_1 offset to next set.
	stx	sol_1_offset	;
	cpx	#solwr1_max	;Check we've written to all solwr1 outputs.
	ble	sol_1_next	;Go do the next solwr1 output.
	ldx	test_loop_cnt	;
	dex			;Check the test's loop count, zero if done.
	stx	test_loop_cnt	;This will set CCs.
	bne	sol_1_contin	;Not done do another pass.

	jsr	pulx_svc	;Restore X.
	pulb			;Restore B.
	pula			;Restore A.
	rts			;Done; return to caller

;===========================================================================

;===========================================================================
;		This is the solenoid test for solwr0.
;
; Purpose:  Verification of solenoid driver logic.
;
; You may check the solenoid outputs at the connectors or at the applicable
; muxes with a scope and or logic probe.  You should see all outputs toggle.
;
; This test will toggle all solenoid outputs related to solwr0 as shown 
; on the schematics.
;
; This test must be run with the mpu connected to the driver board.  All
; solenoid lines should be toggling high and low at their respective muxes.
; This includes the solenable and soldisable signals.
;
; Registers A,B,X are preserved.
;

sol_0_test:
	
	psha			;Save A.
	pshb			;Save B.
	jsr	pshx_svc	;Save X.
	ldx	#sol_loop_cnt	;number of times to loop through test.
	stx	test_loop_cnt	;Do enough times to see with scope or probe.

sol_0_contin:
	ldx	#solenwr1	;We will turn on/off/on solen here.
	ldab	$1		;set up to turn on solenoid enable.
	stab	0,x		;Turn ON solenable based on D0 at solwr0 mux.
	clr	0,x		;Turn OFF solenable based on D0 at solwr0 mux.
	nop			;Give us some OFF time on this one.
	stab	0,x		;Leave ON solenable during test.
	ldab	#sol_0_high	;B contains the sol 0 high select.  set up to
				;turn on highest to lowest solenoids,
				;finally setting soldisable.
sol_0_next:
	ldx	#solwr0		;Addr of solwr0 solenoids.
	stab	0,x		;Turn on sol based on D0 at solwr0 mux.
	nop			;Give us some ON time.
	clr	0,x		;Turn OFF sol based on D0 at solwr0 mux. 
	decb			;decrement to next lowest solenoid.
	bmi	sol_0_next	;at -1 we have done 15 solenoids and disable.

	ldx	test_loop_cnt	;
	dex			;Check the test's loop count, zero if done.
	stx	test_loop_cnt	;This will set CCs.
	bne	sol_0_contin	;Not done do another pass.

	clr	#solenwr1	;We will leave the solenoids disabled on exit.

	jsr	pulx_svc	;Restore X.
	pulb			;Restore B.
	pula			;Restore A.
	rts			;Done; return to caller

;===========================================================================


;===========================================================================

;		The following are support routines.


;===========================================================================

		;This loop does nothing and is an infinite loop on itself
		;as there is no halt instruction for the 6800.

halt_loop:
	clr	#reset_vector	;failure check in.  Write to read only space.
halt_lp:
	clr	watchdog	;clear watchdog timer so we don't get reset
				;when the watchdog is enabled and restart.
	nop
	nop
	bra	#halt_lp	;loop on nothing to simulate a halted program.

;===========================================================================
		;common_return will allow us to save some space rather
		;than repeat code for the restore/rts of some calls.
		;It also helps keep us consistent.

common_return:

	jsr	pulx_svc	;Restore X.
	pulb			;Restore B.
	pula			;Restore A.
	rts			;return to caller.

;===========================================================================

one_sflash:

	pshb			;save b register.
	ldab	#leds_on	;All four LEDs on.
	stab	led_val		;Put in proper place.	
	ldab	#$1		;b now holds the number of times to flash.
	jsr	sflash_svc	;Go flash the light.

	pulb			;restore b register.
	rts			;return to caller.

;===========================================================================

	; flash_svc:
	; flash the LED N times based on contents of b register.
	; Which LEDs are turned on are controlled by the contents
	; of led_val.  This allows us to take advantage of the four
	; LEDs on the atari system to show us things such as switch
	; input values etc... should we wish.  Only the normal flash_svc
	; and sflash_svc routines may do this since they utilize the stack.
	;
	; Registers are preserved.

flash_svc:

	psha			;Save A.
	pshb			;Save B.
	jsr	pshx_svc	;Push X.

flash_loop:

	ldaa	led_val		;aa set to turn on LEDs, based on led_val.
	anda	#led_mask	;mask out bits not associated with LEDs.
	staa	audwr0		;LED should turn on now.
	ldx	#30000		;LED On count.

flash_on:

	dex			; should decrement x by 1.
	bne	#flash_on	; keep on for LED on count. 

;	setup to turn off now to complete flash.

	ldaa	#led_off	;aa set to turn all four LEDs off.
	staa	audwr0		;LED should turn off now.
	ldx	#30000		;LED Off count.

flash_off:
	dex			;should decrement x by 1.
	bne	#flash_off	;keep off for LED off count. 
	decb
	bne	#flash_loop	;go till done flashing.

	jmp	common_return	;common return routine to save some space.
				;and also keep things simple.

;===========================================================================

	; flash_sp_svc
	; flash the LED N times based on contents of b register.
	; On Entry this routine expects the return address in x.
	;
	; This routine should be used prior to stack availability.
	; flash_svc is a called routine that should be used once 
	; the stack has been tested good if you need to save registers.
	; This routine does not save any registers!!

flash_sp_svc:

	txs			; Hold return address in sp.

flash_sp_loop:

	ldaa	#leds_on
	anda	#led_mask	;mask out bits not associated with LEDs.
	staa	audwr0		;LEDs should turn on now.
	ldx	#30000		;LED On count.

flash_sp_on:

	dex			; should decrement x by 1.
	bne	#flash_sp_on	; keep on for LED on count. 

;	setup to turn off now to complete flash.

	ldaa	#led_off	;set all four LEDs OFF.
	staa	audwr0		;LEDs should turn on now.
	ldx	#30000		;LED Off count.

flash_sp_off:
	dex			;should decrement x by 1.
	bne	#flash_sp_off	;keep off for LED off count. 
	decb
	bne	#flash_sp_loop	;go till done flashing.
	tsx			;restore return address from sp.
	jmp	0,x		;return to caller.

;===========================================================================

	; hw_intrpt
	; This routine is entered on a hardware interrupt from IRQ.
	; We will issue WATCHDOG RESET  then INTACK and then return.
	; Note that the writes issued to by intack and watchdog
	; are don't care on the data.  Thus we do not setup data
	; in the A register prior to the write and we don't need
	; to save anything to the stack.
	; 

hw_intrpt:

	clr	watchdog	; clear watchdog counter to avoid RESET.
	clr	intack		; clear the pending interrupt.
	rti			; return from interrupt.

;===========================================================================

;This subroutine does a pull of X from the stack after a pshx_svc call.
;	which is a pain... on the 6800!
;
;The variables sp_place and p_x_place and acc_place must be defined and
;may not be touched by any code except the pshx_svc and pulx_svc subroutines.
;
;A and B registers are preserved.
;
;X is reloaded with the value pushed by the last call to pshx_svc.
;
;Note:  The NMI_routine should be coded such that if interrupts were disabled,
;it will just do a return from interrupt to guarantee that the stack does not
;get mangled.  The NMI routine may modify memory but not any registers, and may;NOT modify the stack in any way.  This code should only be used when necessary
;as it is a lot of instructions to do what should be a simple task but is not
;on the 6800, 6802, and 6808 processors.  Later processors have a psh register
;instruction that includes most all registers as well as X.

pulx_svc:
	clr	watchdog	;clear watchdog timer.

	sei			;set mask, disallow IRQ interrupts.
	sts	sp_place	;hold the stack pointer.
	staa	acc_place	;temp store of A.
	stab	acc_place+1	;temp store of B.
				;no need to store x as we are restoring it!
	ldx	sp_place	;X now has stack_pointer.

	clr	watchdog	;clear watchdog timer.

	ldaa	4,x		;get X low.
	staa	p_x_place+1	;X low now being held.
	ldaa	3,x		;get X high.
	staa	p_x_place	;X high now being held.

	clr	watchdog	;clear watchdog timer.

	pula			;get PC high.
	staa	3,x		;PC high is now where  it was prior to pshx.
	pula			;get PC low.
	staa	4,x		;PC low is now where it was prior to pshx.
				;The pula's should have left SP correct!
	ldx	p_x_place	;X now restored from what was on stack.
	clr	watchdog	;clear watchdog timer.
	ldaa	acc_place	;A now restored.
	ldab	acc_place+1	;B now restored.
	clr	watchdog	;clear watchdog timer.
	jsr	restore_mask	;re-enable interrupts if they should be.
	rts			;return to caller.

;==============================================================================
;This subroutine does a push of X onto the stack.
;	which is a pain... on the 6800!
;
;The variables sp_place and p_x_place and acc_place must be defined and
;may not be touched by any code except the pshx_svc and pulx_svc subroutines.
;
;A,B,X registers are preserved.
;
;Note:  The NMI_routine should be coded such that if interrupts were disabled,
;it will just do a return from interrupt to guarantee that the stack does not
;get mangled.  The NMI routine may modify memory but not any registers, and may;NOT modify the stack in any way.  This code should only be used when necessary
;as it is a lot of instructions to do what should be a simple task but is not
;on the 6800, 6802, and 6808 processors.  Later processors have a psh register
;instruction that includes most all registers as well as X.

pshx_svc:
	clr	watchdog	;clear watchdog timer.
	staa	acc_place	;temp store of A.
	stab	acc_place+1	;temp store of B.
	sei			;set mask, disallow IRQ interrupts.
	sts	sp_place	;hold the stack pointer.
	stx	p_x_place	;hold X.
	clr	watchdog	;clear watchdog timer.
	ldx	sp_place	;X now has stack_pointer.
	ldaa	2,x		;get PC low.
	psha			;PC low now moved down in stack two places.
	ldaa	1,x		;get PC high.
	psha			;PC high now moved down in stack two places.
	clr	watchdog	;clear watchdog timer.
	ldaa	p_x_place+1	;get X low.
	staa	2,x		;X low is now where PC low was.
	ldaa	p_x_place	;get X high.
	staa	1,x		;X high is now where PC high was.
	ldx	p_x_place	;X now restored.
	clr	watchdog	;clear watchdog timer.
	ldaa	acc_place	;A now restored.
	ldab	acc_place+1	;B now restored.
	clr	watchdog	;clear watchdog timer.
	jsr	restore_mask	;re-enable interrupts if they should be.
	rts
;===========================================================================


	; sflash_sp_svc
	; flash the LED N times based on contents of b register.
	; On Entry this routine expects the return address in x.
	;
	; This routine should be used prior to stack availability.
	; sflash_svc is a called routine that should be used once 
	; the stack has been tested good if you need to save registers.
	; This routine does not save any registers!!

sflash_sp_svc:

	txs			; Hold return address in sp.

sflash_sp_loop:

	ldaa	#leds_on	;set all four LEDs ON.
	anda	#led_mask	;mask out bits not associated with LEDs.
	staa	audwr0		;LEDs should turn on now.
	ldx	#9000		;LED On count.

sflash_sp_on:

	dex			; should decrement x by 1.
	bne	#sflash_sp_on	; keep on for LED on count. 

;	setup to turn off now to complete flash.


	ldaa	#led_off	;set all four LEDs OFF.
	staa	audwr0		;LEDs should turn on now.
	ldx	#22000		;LED Off count.

sflash_sp_off:
	dex			; should decrement x by 1.
	bne	#sflash_sp_off	; keep off for LED off count. 
	decb
	bne	#sflash_sp_loop	; go till done flashing.
	tsx			;restore return address to x.
	jmp	0,x		;return to caller.
;===========================================================================

	; sflash_svc:
	; same as flash_svc except this does short flashes.
	; flash the LED N times based on contents of b register.
	; Registers are preserved.


sflash_svc:

	psha			;Save A.
	pshb			;Save B.
	jsr	pshx_svc	;push X.

sflash_loop:

	ldaa	led_val		;aa set to turn on LEDs, based on led_val.
	anda	#led_mask	;mask out bits not associated with LEDs.
	staa	audwr0		;LED should turn on now.
	ldx	#9000		;LED On count.

sflash_on:

	dex			; should decrement x by 1.
	bne	#sflash_on	; keep on for LED on count. 

;	setup to turn off now to complete flash.

	ldaa	#led_off	;aa set to turn all four LEDs off.
	staa	audwr0	;LED should turn off now.
	ldx	#22000		;LED Off count.

sflash_off:
	dex			; should decrement x by 1.
	bne	#sflash_off	; keep off for LED off count. 
	decb
	bne	#sflash_loop	; go till done flashing.

	jmp	common_return	;common return routine to save some space.
				;and also keep things simple.		
;=============================================================================

;This subroutine may be called after disabling interrupts.
;It will re-enable them if they were supposed to be enabled; or
;will leave them disabled.  This assumes the caller has in fact
;disabled the interrupts; if not this code has no effect.
;
;Registers A,B,X are preserved.

restore_mask:
	clr	watchdog	;clear watchdog timer.
	tst	int_status	;if zero interrupts are not supposed to be on.
	beq	restore_no	;branch past interrupt re-enable.
	cli			;clear mask, re-enable interrupts.

restore_no:
	clr	watchdog	;clear watchdog timer.
	rts
;===========================================================================

	; wait_svc:
	; wait for the count in wait_cnt.
	; A,B,X Registers are preserved.

wait_svc:

	psha			;Save A.
	pshb			;Save B.
	jsr	pshx_svc	;Save X.
	ldx	wait_cnt	;x now has the wait count.

wait_loop:

	dex			;dec X
	bne	wait_loop	;wait for count.
	jmp	common_return	;common return routine to save some space.
				;and also keep things simple.
;=============================================================================

	; Vector setup for various interrupts should they occur.

	org	rom3end-7	; start of 6800 vector table.

	; Point unused interrupts to halt_loop.
	; Some day we may toy some more with them given a reason.

	fdb	hw_intrpt	; FFF8,FFF9	IRQ HARDWARE INTERRUPT.
	fdb	halt_loop	; FFFA,FFFB	SWI CURRENTLY NOT USED HERE.
	fdb	show_revision	; FFFC,FFFD	NMI CLEAR SWITCH (SW 33).
	fdb	show_revision	; FFFE,FFFF	RESET.

;===========================================================================
