;
; Questions Comments etc... On this program may be directed to the
; author at the following address (If you wish a reply please send a SASE)
;
; Dallas E. Overturf
; 267 Main St
; Ashland, MA 01721 (USA)
;
; e-mail:  overturf@ultranet.com
;
; Comments from the Author:
;
; If you decide to make your own version of the code either by patching
; the binary or by changing the source; please be considerate and patch
; patch the revision byte to a value of 01, 02, or 03 to indicate that the
; copy is a hacked up copy.  I will not use revisions 01, 02, or 03 in a
; released copy.  The first released copy will be revision 04.
; If you modify the source; then please keep the Modification History 
; and or Bug history sections updated as appropriate.
;
; The code:  This code was not written for efficiency and was done in
; my spare time.  No doubt there are things I did that I could have done
; better. It does work the way I intended it to so I'm not planning
; on re-writing it!  If you submit a bug to me that I deem to be a valid
; bug; I will probably fix it; if it can be reasonably fixed (my opinion).
;
; If you come up with something "great" you want added to this code I may
; be willing to consider doing it if it seems reasonable (my opinion).
; If you should take the time to improve upon it feel free to send me
; a copy :)
;
;		Happy Pinballing!
;					Dallas...
;
;
; Modification history
;
; 12-10-92:  DEO  Rev 4.0. Initial Release to public domain.
;		  
;
;
; End Modification History
; 
; Rev 4.0 removed copyright comments. Still at 4.0
;
; Bug History / Status:
;
; 06-15-93:  DEO Rev 4.0.  The digits test did not work when actually
; tested on a bally machine live (7 or 6 digit game) though it works
; in my test jig Rev 3.0 same.  Need to retry with live mpu and live display.
; Notes:  pia_test_3: if takes last error leaves LED on solid.
; 11-28-96 Fixed by Oring led_off with the
; cra ddr select value; see notes at pia_test3.
;
; Thoughts on additional tests:
; A full pia test may be possible by making some special loopback cables
; for the J2 and J3 connectors.  Also need to try out interrupts as well.
; Possibly this should be a new test in a different prom; suggestions are;
; Check interupts, then displays, then do full PIA test using some sort of
; loopbacks.  We may be able to put out bad pia PA/PB lines in the displays
; as well as flash a code for them?  Need to look at schematics.
;
; Testing Notes:
;
;	This test prom is placed into socket U6 to be used.
;	Note socket U6 must be jumpered to match the prom type used.
;	If this prom code is put into a 2732 it may be used in a U6
;	socket that is jumpered as a 2716.
;
;	If this prom code is put into a 2532 EPROM; then U6 jumpering
;	must be for a 9332 or 2532.
;
;	Special note on Bally AS2518-17 MPUs.  If the board is jumpered
;	for a single 2732 to run the game in a socket other than U6; then
;	the test prom should be run in that socket.
;
;	The test prom should be in the socket that will be enabled by
;	an address of FFFE.  (the reset vector of the 6800).
;
;
;	Programming the code into an EPROM:
;	For a 2716 just program it in normal (IE: start address is 0000).
;	If your burner requires an end address, it is 07FF.
;
;	For a 2732 or a 2532 the start address is 0800 (this means 0000-07FF
;	remain blank.  If your burner requires an end address, it is 0FFF.
;	(note in this case you are using the 2732 as a 2716)
;
;	Note:  If the binary file of this program is larger than 2048 bytes;
;	and the file starts at address 0000; then you will need to give your
;	prom programmer a starting address within the binary file as well to
;	start taking data from.  In this case you need to give it address
;	5800 which will be the offset (in hexadecimal) of the start of code
;	in the binary file. (This is for a 2732 or 2532)
;	You may use such a 2732 but not a 2532 in a U6  socket jumpered for
;	either a 2732 or for a 2716 provided that only the upper half of
;	the prom is programmed.  A 2532 must be jumpered as a 2532.
;	Remember that the 6800's boot vector is in upper memory; thus
;	this prom is written in upper memory and currently does not
;	exceed the size of a 2716.  Thus it is good on both -17 and -35 mpus.
;
;
;	It is recommended that you verify the prom in a known good board
;	prior to attempting to use it on a bad board!
;
;	Things that may keep you from comming up and working correctly
;	on the game rom that are untested in this game.
;	INTERRUPTS!!  This test program specifically does not test program
;	interrupts (the IRQ lines on the 6800 and PIAs).
;
;
;	Some basic suggestions when trouble shooting a board:
;
;	If the test prom does not work at all; any remaning game rom in
;	U1,U2,U3,U5 should also be removed.  This will help prove that
;	there is not a bad game rom on the board.  If this test prom can
;	get through it's set of tests on your mpu, and your game rom cannot
;	flash the LED at least three times; then it is recommended that
;	you try a different set of KNOWN GOOD game rom in your mpu to
;	verify that your game rom are good.  Also it is recommended that
;	you verify that the jumpers on the board are properly configured
;	for the type of prom that you are using; especially in the case of
;	a board that you have never seen working before.
;	Note that if the Bally game rom turns off the LED this indicates
;	that they have passed a checksum test and are likely to be good.
;	If they cannot turn off the LED and this test prom passes; they
;	are likely bad or a signal is bad at one of the game rom sockets
;	they use, other than at socket U6.  Also verify jumpers match
;	the prom you are using if you changed them to use the test prom.
;
;	With regards to this test prom:
;	To get the initial "two short flashes" indicating the program is
;	active, the only essential "normally" socketed chips on the board
;	are as follows:  U9 (6800 processor) U11 (6820 or 6821) PIA and
;	this test prom.
;
;	"Normally" socketed chips are:  U1 through U11 inclusive.
;
;	The first four flashes only require the following "normally"
;	socketed chips in the board; The 6800 processor at U9, the 6810
;	Static Ram in U7 and the U11 the (6820 or 6821) PIA.  U9 is in
;	actuality only required to see the LED FLASH in this case.  It
;	is not otherwise tested till much later.
;	
;	U8 the 5101 is additionally required to get through seven flashes.
;
;	U10 is additionally required to get through ten flashes.
;	Note:  if not running MPU 200 tests you will also see flashes 
;	11 and 12 occur; they are dummy flashes for consistency in this case.
;	(so I don't have to re-document things!)
;
;	U13 is required to get through 12 flashes STERN MPU 200 only and
;	only if test_mpu_200 is set to 01.  If it is set to 00 then
;	flashes 11 and 12 will occur as dummy flashes.  Note that
;	the dummy flashes are the flashes with a short "LED ON"
;	time as opposed to the longer normal flashes.  This will
;	allow the user to recognize when he is or is not really
;	testing U13.
;
;	With the above in mind; if the board fails you may wish to remove
;	all normally socketed chips not needed for the current failing test.
;	example:  You do not get the two short flashes indicating the board
;	is running or the LED stays on.  Remove all but the test chip in U6,
;	U9, and U11.  If you still do not get the short flashes you have 
;	elimiated many possible causes such as a chip being mistakenly
;	selected or bad.  If you then do get the two short flashes with the
;	excess chips out; you now have some idea that on of the chips removed
;	is bad or being mistakenly selected.
;
; 
;	This prom is intended to loop on errors and continue if the error
;	goes away.  Thus it is not usefull for running overnight to see
;	if there is a single intermittent error, as if the error goes away
;	there will be no indication of it as the test will continue
;	(note you could do this with a logic analzer attached).
;	Note:  This action is affected by the setting of the "error_action"
;	byte, described later.
;
;	Since the program in the prom must go over the same bus to the
;	mpu as anything else on the board it is not very usefull for
;	trouble shooting shorts on a data line or address line (EXCEPT
;	with a logic analyzer and this listing) as the test program 
;	probably will not be able to run.
;
;	This test program was designed with the intent of finding problems
;	that are due to the battery corroding on the board.  This normally
;	means an OPEN (broken) etch or bad connection within a socket.
;	
;	It is also designed to deal with LED on solid type problems; however
;	to use this in most LED on solid conditions you will need to have
;	a 24 channel logic analyzer at your disposal (8 data, 16 address)
;	and of course this listing so you can see what went wrong.
;
;	This program checks the main address/data paths on Bally MPUs:
; 	AS-2518-17, AS-2518-35, and AS-2518-133.
;	
;	Further it also will run on STERN MPU 100 and STERN MPU 200 MPUs.
;
;	Assuming the program code can be read out of the prom:
;
;	This prom will loop on errors and continue if the error
;	goes away.  Thus it is not usefull for running overnight to see
;	if there is a single intermittent error, as if the error goes away
;	there will be no indication of it as the test will continue
;	(note you could do this with a logic analzer attached).
;	Note:  This is the default setup of the program.  This can
;	be change by changing the "error_action" byte.  See below.
;
;
;	Since the program in the prom must go over the same bus to the
;	mpu as anything else on the board it is not very usefull for
;	trouble shooting shorts on a data line or address line (EXCEPT
;	with a logic analyzer and this listing) as the test program 
;	probably will not be able to run.
;
;	The design of this test prom was for three main purposes:
;
;	1.  Allow troubleshooting of bus problems relatively quickly
;		and easily using a 24 channel logic analyzer.  
;
;	2.  Also attempt to actively loop on an error so that the error
;		may be more easily seen using an osciloscope or logic probe.
;
;	3.  Attempt (when possible) to be able to give the user a more 
;		definitive visual clue to what was wrong with the bus.
;		Ie:  data and or control was bad or the address lines
;		are bad.
;		Note:  The data test is always first for a part followed
;		by addressing tests.  The initial data test will not fail
;		(normally)due to an open address line to a component under
;		test.
;		It will fail due to a bad data line, or a bad control
;		line.  Control lines are chip selects; R/W and Power/Ground.
;		Thus if you pass the data test and fail the first addressing
;		test you know you will most likely be looking for a bad
;		address line.  If you fail a component uniqueness test
;		then you are looking for a bad address/control line
;		to one of the components that failed.
;
;
;		Most problems on the bally MPU are due to corrosion from the
;		battery (which normally results in open etches/pins).
;		In most cases these problems involve U7, U8, and U11 and
;		sometimes U6 as they are nearest the battery.  
;		Problems involving the U11 PIA cannot easily be diagnosed
;		without a logic	analyzer due to the fact that flashing the 
;		LED requires the U11 PIA.  If the U6 socket is suspect due
;		to corrosion you should check continuity of all pins back
;		back to the processor, and control signals to their origins.
;		Also verify that you have power and ground connections to U6. 
;		
;
;
;	Note:  Upon start of the program there will be two short flashes.
;	This is done as a sanity check so that the user can see that the 
;	actual test code has in fact started to run or is re-running.
;
;	Note:  for those using a logic analyzer the following occurs prior
;	to seeing the two sanity check flashes:
;
;	Four data patterns are written out immediately after reset;
;	they are: 00, FF, 55, and AA are written to address 0000.  
;	They are immediately followed by four reads from the following
;	addresses:  0000, FFFF, 5555, and AAAA.  (data here is random and 
;	of no interest.)
;	This is to allow analyzer users to verify basic data and address bus
;	integrity.  If either is being held down or up; it should be
;	immediately visable here using an analyzer.  The next thing that
;	occurs will be the two short flashes that act as a sanity check for
;	the user to verify that the program is running followed by the first
;	test.
;
;
;	The "error_action" byte settings located at program address 5800 or
;	physical EPROM addresses 0000 (for 2716) or 0800 for (2732/2532).
;
;	If set to "restart_val" (02):
;	Then all errors will, after writing to error_check in; jmp to an 
;	error_flash routine which is responsible for flashing the LED 5
;	times quickly and then restarting the test (based on further decode
;	of the data) from the beginning.   This setting may be usefull for
;	looking at intermittent problems.
;
;	If set to "halt_val":
;	Then all errors will, after writing to error_check in; jmp to an 
;	error_flash routine which is responsible for flashing the LED 5
;	times quickly and then the 6800 processor will be put into an
;	infinite do nothing loop (simulates a halt) that does absolutely 
;	nothing.  This would allow the user to see that the test failed 
;	an overnight run.  Since the test is no longer running the normal
;	tests; the LED will not be flashing.
;
;	If set to loop_val (00):  
;	The test will not flash five times nor will it restart.  Instead it
;	attempts to loop on any error found until the error goes away.
;	This setting is the most usefull for trouble shooting with a scope
;	or logic probe if a logic analyzer is unavailble.  It is also fine
;	for using a logic analyzer as well.
;
;
;	The error_action byte will by default, be set to 00 (loop_val)
;	when the EPROM is made.  The user may wish to make an archival
;	copy of this prom and or set this byte to whatever he/she desires.
;	It may be preferrable to keep two copies of the test prom handy;
; 	each with a different setting of this byte.
;
;
;	Error Check In Feature:   (For use with a logic analyzer)
;
;	On an error the program will attempt to write to the reset_vector
;	of the 6800 (Address FFFE) in an attempt to allow triggering of
;	an analyzer on an intermittent error.  This is known as the
;	"error check in" location, and is READ ONLY.  Set the anlyzer to
;	End trace on any write to address FFFE.  Data should be
;	set to a don't care state.  The data you see is of no interest.
;	The only thing that counts is that you do a write to address FFFE.
;	You should be able to see what failed shortly before the error
;	check in location is written.
;
;	Another good qualifier to use for any trace if you have more than 24
;	channels available, is to trace on the address and or data you
;	are interested in and qualify it with reset being false.
;	Note:  Both Write (R/W) and Reset are low true signals; thus a
;	logic "1" high indicates you are reading or not reset respectively;
;	a 0 (low) would indicate you are writing or reseting respectively.
;
;
;	
;		Current # of tests/flashes and their meaning:
;
;		(The short double flash at start does NOT count as a flash.)
;
;		Note:  A flash indicates that a test passed, prior to a flash,
;		you are running the testing associated with that flash.
;		Thus seeing two short flashes at the start and not the "first"
;		flash would indicate you are failing the U7 data test.
;
;
;		All tests below will attempt to loop on an error; assuming
;		the error_action byte is set to loop_val (00).
;			
;		0.  Two short flashes of LED indicate program start. **
;		    Whether it be from power up or from the test finishing
;		    a pass and simply looping back to the start.
;
;		    ** Note:  Will be preceeded by short flashes indicating
;		    the Revision Number of the program if switch 33 is pushed.
;			
;		If the program successfully started you will see two short
;		flashes prior to the start of the first test.
;		Note:  In counting the number of flashes do not count these
;		two short flashes.  They simply mean the program is active
;		and also indicate the start of the program.  Every time the
;		program loops through 10 flashes successfully, you should
;		again see these two short flashes. 
;
;		The test program will run forever and these two short
;		flashes can be used to identify the start of another pass
;		as well as indicate that the program has not gone haywire
;		due to some problem; watch for them!
;
;		The only other time short flashes are used currently is:
;
;		1.  If switch 33 is pushed as described above.
;		
;		2.  For an error report (based on the "error_action byte;
;		described previously) 
;
;		3.  During certain tests as described in the test descriptions
;		below, all indicate an error and all will be 10 flashes.
;
;
;		If the LED did not blink twice you should suspect that the
;		program is not running.   
;
;		If it did blink two short times then you should now
;		be running test 1.  If the test passes succesfully; 
;		you should see the first real flash.
;
;
;		Flash 1.  U7 data test passed:
;			All data patterns checked at all locations
;			in the 6810.
;
;			Note:  As this is the first test run, any failure
;			can get hit here.  An open address line to a ram,
;			may cause this test to fail if it is floating
;			at a border line state.  Indications of this would
;			be failing data/address changing during different
;			passes through this test.  If the address line is
;			floating steadily high or steadily low then this
;			problem will normally be caught in the address tests.
;
;			Probable cause of failure:  almost anything at this
;			point.  Check data lines, U7 enables, and R/W line 
;			at U7 and finally check address lines.  Enables on U7
;			should be active as well as R/W (error_action = loop).
;			Data and address can most easily be checked via a 
;			logic analyzer or may be ohmed out to the 6800 
;			processor.
;			
;			Now running test 2.
;
;		Flash 2. The first of two U7 address uniqueness tests passed.
;			All addresses checked in u7 for uniqness to
;			u7_min address and u7_min+1 address.
;
;
;			Note:  This is a weird way to do it but it is done
;			before address equal data test since due to the lack
;			of any stack space (we cannot trust U7 yet) and due
;			to lack of an extra register in the 6800 we cannot
;			get into an error loop from address equal data
;			until we have two "trusted" bytes in the 6810.
;			This will catch almost any address uniqness problem
;			at the 6810, except possibly a bad 6810.  That should
;			be caught by the address equal data test that follows.
;
;			Probable cause of failure is a faulty address line.
;
;			Now running test 3.
;
;		3.  Flash 3.  Address equal data test of U7.  passed.
;			This is the second address uniqueness test of U7.
;
;			A loop is entered on failure.
;			Now writing zeroes to all of 5101 verify 6810 is
;			not touched.  If test fails check 6810 enable circuit
;			followed by checking for short between 5101 enable
;			signals and 6810 enable circuit.  Also verify
;			R/W line is not shorted to 6810 enable circuitry.
;			This test also looks for an error if it cannot find
;			it, it will short flash the LED 10 times and re-start.
;
;			Probable cause of failure:  
;			Open or shorted address line to U7.
;
;			Now running test 4.
;
;		4.  Flash 4.  U7 uniq from U8 test 1 of 2 passed.
;
;			Write 00 to 5101 check 6810 untouched.
;
;			Probable cause of failure:
;			Check 6810 enable circuit followed by checking
;			for short between 5101 enable signals and 6810
;			enable circuit.  Also verify
;			R/W line is not shorted to 6810 enable circuitry.
;			Possible though unlikely, data line shorted.
;
;			Now running test 5.
;
;		5.  Flash 5.  U7 uniq from U8 test 2 of 2 passed.
;
;			Write FF to 5101 check 6810 untouched.
;
;			Probable cause of failure:
;			open or shorted chip selects feeding U7 an or U8.
;			Check U7 an U8 enable circuits followed by checking
;			for short between the U7 and U8 chip selects.
;			Also verify R/W line is not shorted to 6810 enable 
;			circuitry.  Possible though unlikely shorted data line.
;
;			Now running test 6.
;
;		6.  Flash 6.  U8 data test passed.
;
;			Check all data patterns to U8 CMOS RAM.
;			on the error.  If it is unsuccessfull; the error
;			routine will short flash the LED 10 times and the
;			program will attempt to restart itself from scratch.
;
;			Probable cause of failure:
;			Open data line, open or shorted chip select or 
;			open or shorted R/W line to U8 CMOS RAM.
;
;			Now running test 7.
;
;		7.  Flash 7.  U8 address uniqueness test passed.
;
;			All addresses checked on 5101.  Test passed.
;
;			Note:  The address uniqueness section of this test
;			will attempt to find the conflicting address to loop
;			on the error.  If it is unsuccessfull; the error
;			routine will short flash the LED 10 times and the
;			program will attempt to restart itself from scratch.
;
;			Probable cause of error:  open address line,
;			open or shorted chip select to U8.
;
;			Now running test 8.
;
;		8.  Flash 8.  U11 PIA DDR test passed.
;			Check data direction registers A,B in U11 PIA.
;			Simple data patterns of $55 and $AA are used.  Verify
;			that the two registers can be uniquely addressed
;			within the PIA.
;
;			Probable cause of failure:
;			Open address, enable, Data, R/W or reset line to PIA.
;
;			Now running test 9.
;
;		9.  Flash 9.  U10 PIA DDR test passed.
;			Check data direction registers A,B in U10 PIA.
;			Simple data patterns of $55 and $AA are used.  Verify
;			that the two registers can be uniquely addressed
;			within the PIA.
;
;			Probable cause of failure:
;			Open address, enable, Data, R/W or reset line to PIA.
;
;			Now running test 10.
;
;		10. Flash 10.  Check that we can address U11 and U10 uniquely.
;			We write $55 to the ddra in U11 and $AA to ddra in
;			U10.  We then read back U11 and make sure it still
;			has $55.  Loop entered on failure.
;
;			Probable cause of failure:  Pin 22 of PIA Open or:
;			Open address, enable, Data, R/W or reset line to 
;			either U10 or U11.
;
;			Now runing test 11.
;			
;		11.  Flash 11.  U13 data test passed.  MPU 200 ONLY.
;			Note: location test_mpu_200 must be set to 01.
;			to run this test.
;
;			Note: location test_mpu_200 must be set to 01 to run.
;			If set to 00 this test is not run; however a short
;			flash will occur as flash 11 indicating that it was
;			not run.
;
;			Check all data patterns to U13 CMOS RAM.
;
;			Probable cause of failure:
;			Open data line, open or shorted chip select or 
;			open or shorted R/W line to U13 CMOS RAM.
;
;			Now running test 12.
;
;		12.  Flash 12.  U13 addr uniqueness test passed. MPU 200 ONLY.
;
;			All addresses checked on U13 CMOS RAM.
;
;			Note: location test_mpu_200 must be set to 01 to run.
;			If set to 00 this test is not run; however a short
;			flash will occur as flash 12 indicating that it was
;			not run.
;
;
;			Note:  The address uniqueness section of this test
;			will attempt to find the conflicting address to loop
;			on the error.  If it is unsuccessfull; the error
;			routine will short flash the LED 10 times and the
;			program will attempt to restart itself from scratch.
;
;			Probable cause of error:  open address line,
;			open or shorted chip select to U8.
;
;			The test at this time has finished and will loop back
;			to the start and indicate this by two short flashes.
;
;           10 Quick Flashes on the tests that indicate this function
;			occur because an error loop aka scope loop was entered
;           and the error loop determined the failure stopped occuring.
;           This is typical of intermittent signals or signals on "the edge".
;			Maybe you pressed on the board or probed an iff signal.
;
;
;		Note:  All of the above "Probable cause" suggestions assume
;			that you have  "Known Good" Parts in the sockets.
;			If you do not; you would be well advised to change
;			the part in most cases before looking for opens or
;			shorts in the board. 
;			Also in the case of sockets U6, U7, U8, and U11 in
;			particular; verify that the socket(s) are not 
;			corroded from the NiCad battery leaking.  If a 
;			socket looks bad; replace it with a good one.
;
;
;		Program re-start may be accomplished by activating the
;		switch 33 pushbutton at any time.  This will cause
;		a Non-Maskable Interrupt whose vector points back to
;		"show_revision".  Show_revision will cause the LED
;		To blink n times rapidly to correspond with the revision
;		number of the program.  This will be followed by a wait
;		of 2-3 seconds.  After which the program will restart.
;		Program restart can be recognized by its two short flashes.
;		
;
;
;Power on or a "forced" reset (you ground and release reset on 6800)
;Sanity Check: (2 short flashes).
;flash #1.  U7 data bits are unique.
;flash #2.  U7 addresses unique part 1.
;flash #3.  U7 addr uniq part 2. U7 is good.
;flash #4.  Writing U8 did not affect U7.
;flash #5.  U7/U8 uniq. Stack is now useable.
;flash #6.  U8 data bits are unique.
;flash #7.  U8 addresses are unique.
;flash #8.  ddra/b reg in u11 basically checked.
;flash #9.  ddra/b reg in u10 basically checked.
;flash #10. U10 and U11 are unique.
;flash #11. MPU 200 only. U13 data bits unique. (short flash if not run)
;flash #12. MPU 200 only. U13 addresses unique. (short flash if not run)
;A short display test now occurs that does NOT use interrupts!
;Start over from beginning at sanity check flashes. 

yes		equ	$1	;
no		equ	$0	;
u6_strt		equ	$5800	;start addr = 0 for bally level 17 mpu for U6.
u6_end		equ	$5FFF	;end addr = ffff for bally level 17 mpu for U6.
u2_strt		equ	$5000	;start addr = 0 for bally level 17 mpu for U2.
u2_end		equ	$57FF	;end addr = ffff for bally level 17 mpu for U2.
u7_min		equ	$0	;low addr for 6810.
u7_max		equ	$7F	;high addr for 6810.
u8_min		equ	$200	;low addr for 5101.
u8_max		equ	$2FF	;high addr for 5101.
u11_mask	equ	%0000000010010011	;mask for u11 PIA. 93x
u11_sel		equ	%0000000010010000	;addr enable for u11 PIA. 90x
u10_mask	equ	%0000000010001011	;mask for u10 PIA.8Bx
u10_sel		equ	%0000000010001000	;addr enable for u10 PIA. 88x
u13_min		equ	$200	;low addr for 5101. U13 is STERN MPU 200
u13_max		equ	$2FF	;high addr for 5101. U13 is STERN MPU 200
led_on		equ	$38  ;should turn on LED.
led_off		equ	$30  ;should turn off LED.
cra_sel		equ	$01  ;control register A.
crb_sel		equ	$03  ;control register B.
pia_reg_a_sel	equ	$00  ;Register A slections in PIA.  For DDRA and PRA
pia_reg_b_sel	equ	$02  ;Register B slections in PIA.  For DDRB and PRB
pia_ddr_reg	equ	%00000000 ; access output reg access.
pia_out_reg	equ	%00000100 ; access data direction reg access.
player_latches	equ	$0F	;players 1-4.  status/match not included.
digit_bcd_bits	equ	$FF	;PA0-7. BCD bits for all players.
irq_vector	equ	$FFF8	; FFF8,FFF9  IRQ interrupt.
swi_vector	equ	$FFFA	; FFFA,FFFB SWI	interrupt.
nmi_vector	equ	$FFFC	; FFFC,FFFD NMI CLEAR SWITCH (SW 33).
reset_vector	equ	$FFFE	; FFFE,FFFF RESET vector.
loop_val	equ	$0	; see error_action settings, or flash_error.
restart_val	equ	$1	; see error_action settings, or flash_error.
halt_val	equ	$2	; see error_action settings, or flash_error.
blank_time	equ	$1F	; toggling blanking for RC network R30/C18.
revision_val	equ	4	; Revision.
stack_strt	equ	$4F	; stack will be from 4f to 0, with the
;				; top $30 locations reserved for variables.
;
; THE FOLLOWING LOCATIONS ARE RESERVED IN THE 6810 AS PROGRAM VARIABLES
; THEY MAY NOT BE ACCESSED UNTIL AFTER WE HAVE DECIDED THE 6810 IS OK.
;
; Variable region exists from addr $50 to addr $7f.
var_1		equ	$50	; two bytes for first variable in call
var_2		equ	$52	; two bytes for second variable in call
var_3		equ	$54	; two bytes for third variable in call
x_place		equ	$56	; two bytes for allowing push of x.
temp		equ	$58	; two bytes used for local temp storage.
disp_bcd	equ	$5A	; two bytes for display info.
blank_cnt	equ	$5C	; current blanking status.
disp_cnt	equ	$5E	; for looping.

; define  some important space for address uniqness testing of 6810 RAM.
temp_0		equ	u7_min	; two bytes of space at 00 and 01 in RAM.



	org 	u6_strt		;start of program.

;	Note:  "error_action" is intended to always be at u6_strt
;	for convenience and is the first byte of u6.  The next byte will
;	be the test_mpu_200 variable.  If set to #no (zero) we do not run
;	U13 tests.  If set to #yes (one) we run u13 tests.  Default of the
;	code unless modified shall be to not run u13 tests.  Thus we will
;	run on all Bally AS-2518-xx and both Stern MPU 100/200 less the 
;	U13 test.  Note to allow conitinued additions to this test, if the
;	MPU 200 tests are disabled the test will still flash as if it passed;
;	however the flash will be a short flash when disabled instead of the
;	normal longer flash.  This will allow us to always associate a given
;	flash with a specific test.  The next byte contains the revision 
;	number.  Then a few bytes which contain a copyright notice.


error_action:
	fcb	#loop_val	;set to 00 will cause 5 flashes on error and
				;restart.  set to non-zero will allow loop
 				;on error.  See description of error_action.

test_mpu_200:
	fcb	#yes	;This byte decides if we run mpu 200 tests.
				;If no; then we do not run U13 specific tests.
revision:
	fcb	#revision_val	;Put rev in rom also in case we want to know.


;		Now put in my copyright notice.
copyright:
	fcc	" WRITTEN 1992 BY DALLAS E. OVERTURF "

	fcc	" MY BALLY/STERN TEST PROM "

;	A Non Maskable Interrupt (switch 33) will restart at show_revision.

show_revision:

	ldx	#30000		;wait a moment prior to flashing rev.
wait_rev:
	dex			;dec count.
	bne	wait_rev	;keep waiting.

	ldab	#revision_val	;b now has revision.
	ldx	#revision_ret	;x has return address.
	jmp	sflash_sp_svc	;go flash the rev.
revision_ret:
	ldaa	#2		;two times will be a very long wait.
revision_lp:
	ldx	#30000		;long wait.
rev_x_lp:
	dex			;dec wait count
	bne	rev_x_lp	;
	deca			;dec loop count.
	bne	#revision_lp	;branch if not done or fall through to start.

start:
	;1st write and or read some data and address patterns to see
	;that the bus is ok on the analyzer.

	ldx	#$0000		;An address to write to write to. = 0000.
	ldaa	#$00		;some data to write= 00.
	staa	0,x		;unchecked data only to look at bus.
	coma			;some data to write= FF.
	staa	0,x		;unchecked data only to look at bus.
	ldaa	#$55		;some data to write= 55.
	staa	0,x		;unchecked data only to look at bus.
	coma			;some data to write= AA.
	staa	0,x		;unchecked data only to look at bus.
	ldx	#$0000		;address pattern to read from.
	ldaa	0,x		;read address, don't care about data.
	ldx	#$FFFF		;address pattern to read from.
	ldaa	0,x		;read address, don't care about data.
	ldx 	#$5555		;address pattern to read from.
	ldaa	0,x		;read address, don't care about data.
	ldx 	#$AAAA		;address pattern to read from.
	ldaa	0,x		;read address, don't care about data.

	;Now turn off LED so we know we are running code.

	ldx	#u11_sel	;x has u11 enable base addr.
	ldaa	#led_off	;aa set to enable CA2 as output, LED off.
	staa	cra_sel,x	;LED should turn off now.
	staa	cra_sel,x	;again in case b5 in cra must be set first.

	ldx	#32700		;hold off so we can see the LED is off.

start_delay:
	dex
	bne	#start_delay

pass:

	ldx	#pass_return	;x has return address.
	ldab	#2		;flash count.  Double quick blink so we know
	jmp	sflash_sp_svc	;we are in code and where start is on a pass.
pass_return:

	ldx	#u7_min		; Start addr of 6810.
	ldaa	#$0
u7_data:

;	u7_data attempts to check that each address can hold any data
;	pattern in the 6810.  Address uniqueness has not been qualified yet.


	ldx	#u7_min

u7_d_nextadr:
	ldaa	#$0		;aa will be expected data
	ldab	#$ff		;ab will be actual. Init to anything else; we
				;start at zero so this must get zero on a read
	bra	#u7_d_datloop	;

u7_d_errloop:
	clr	#reset_vector	;failure check in.  Write to read only space.
	ldab	error_action	;sets CCs.
	beq	#u7_d_datloop	;no flash error.
	jmp	flash_error	;

u7_d_datloop:
	staa	0,x		;write to the 6810.
	ldab	0,x		;read from the 6810.
	cba			;compare a and b accumulators.
	bne	#u7_d_errloop	;loop on the error till it clears.
	inca			;increment a.
	bne	#u7_d_datloop	;data is 00-FF on FF we roll to 00 indicating
				;we have checked all data on current address.
	inx			;inc x to next addr.
	cpx	#u7_max+1	;if past max addr then we are done.
	bne	#u7_d_nextadr	;not done; do next address.
	ldab	#$1		;flash #1.  U7 data bits are unique.

	ldx	#u7_d_return	;x now has return address.
	jmp	#flash_sp_svc	;go flash, then fall through to next test.
u7_d_return:			;This label must be here to return.

;===========================================================================

;	This test will verify that locations zero and one in u7 are unique
;	from all other locations in u7.  Thus we will have identified two
;	bytes of memory that can be used as stack space or temp storage in
;	the full u7 address uniqueness test which is an address equal data
;	test.
;	By using all data patterns we verify that there is no data line(s)
;	shorted to an address line which would later confuse things for the
;	four addresses in question.  Probably most address uniqueness problems
;	will be caught here; however we will do adress equal data test later
;	to be very safe.  Note:  Between this test and the u7_data_test; we
;	really should not see data shorted to an address.


	clra			;aa holds pattern to write.  patterns. 00-FF.
u7_au0_part:
	ldx	#u7_min+2
	bra	#u7_au0_loop

u7_au0_err:
	clr	#reset_vector	;failure check in.  Write to read only space.
	ldab	error_action	;sets CCs.
	beq	#u7_au0_loop	;no flash error.
	jmp	flash_error	;

u7_au0_loop:
	clrb			;to clear temp locs. We need some temp space.
	stab	u7_min		;temp loc 0.  Part of temp_0.  Used for
	stab	u7_min+1	;temp loc 1.  Part of temp_0.  Future storage.
	staa	0,x		;write test data to all other u7 locations.
	ldab	u7_min		;temp loc 0.  This will set CCs.
	bne	u7_au0_err	;loop on error.
	ldab	u7_min+1	;temp loc 1.  This will set CCs.
	bne	u7_au0_err	;loop on error.
	inca			;inc aa to next data pattern.
	beq	#u7_au0_ckdone
	bra	#u7_au0_loop	;Do next data pattern.

u7_au0_ckdone:
	cpx	#u7_max		; check if really done.
	beq	u7_au0_done	; yes we are really done.
	inx	
	clra			; Start pattern at 00 again.
	bra	#u7_au0_loop	;not done continue looping.

u7_au0_done:
	ldab	#$1		;flash #2.  U7 addresses unique part 1.
				;we don't call flash_svc yet till we check
				;that all addresses are unique which occurs
				;next.
	ldx	#u7_au0_return	;x has return address.
	jmp	#flash_sp_svc	;go flash.
u7_au0_return:			;This label must be here to return.

	ldx	#20000		; since we passed hold off a bit so we can 
				; see the next flash occur for next test.
u7_au0_delay:
	dex
	bne	#u7_au0_delay


;===========================================================================

;	U7 Address Equal Data test.  Final adress uniqness test of u7.
;	It is not expected that this test will normally fail except
;	in the case of a bad 6810 or an intermittent board.  However
;	this test can loop on error in case strange circumstances are
;	encountered, and we fail due to something other than a bad 6810.

u7_aed_test:
	ldaa	#u7_max
	ldx	#u7_max
u7_aed_dec:
	staa	0,x		;write pattern to address.
	deca			;next pattern.
	dex			;next address.
	cpx	#temp_0+1	;leave our two byte space alone!
	bne	u7_aed_dec	;done if equal else continue.

;	U7 should now be completely address equal data less first two locs.

	ldaa	#temp_0+2	;1st address in range we write to.
	ldx	#temp_0+2	;1st address in range we write to.
u7_aed_inc:
	ldab	0,x		;read 1st address.
	cba			;compare expected in a, to actual in b.
	bne	u7_aed_find_loc	;Try to find the conflict to loop on error.
	inca			;next pattern.
	inx			;next address.
	cpx	#u7_max+1	;done when we reach u7_max.
	bne	u7_aed_inc	;continue if not done or fall through and pass.

;	U7 should now be completely address equal data, less first two locs.

	jmp	u7_aed_passed	;no error, we passed.

;	On entry x is the location in error found as follows:
;	The routine prior to this is expected to write data from max to
;	min addresses and then do the check from min to max so that 
;	temp_0 - 1 will be the highest address needed to find the error.
;	If this policy is not adhered to then this routine's ability to
;	localize the error will be compromised.
;	Note:  Temp_0 occupies the first two bytes of u7 thus it MUST BE
;	gauranteed to to be uniqe to u7 prior to using this routine.

u7_aed_find_loc:

	clra			;
	staa	0,x		;clear error location.
	stx	temp_0		;store away error loc for now.
	ldaa	#$AA		;Compliment of first pattern.  We do two.

u7_aed_next_dat:
	coma			;compliment so 1st patter will be 55.
	ldx	#temp_0+2	;initial loc we write to

u7_aed_next_adr:
	staa	0,x		;write to location then check error loc.
	txs			;store x in sp for now.
	ldx	temp_0		;get error loc into x.
	ldab	0,x		;check it.  This will set CCs.
	tsx			;restore x from sp. This will not change CCs.
	bne	u7_aed_found	;we will now go and try to loop on the error.
	inx			;next address.
	cpx	temp_0		;If >= temp_0, we've not yet found the problem.
	blt	u7_aed_next_adr	;continue to next addr.
	cmpa	#$AA		;if AA then we are done.
	bne	u7_aed_next_dat	;if only 55 then we need to do AA still.
	bra	u7_aed_unot	;If here we could not isolate the error.
				;in such a case let them know.
	
;	u7_f_loop expects the bad address in temp_0 and conflict addr in x.
;	data written to find error will be in aa still.  aa has the data.
;	Note:  we enter at found:  we loop to found_lp.  This allows us to
;	see the problem occur immediately before error check in.
u7_aed_foundlp:
	clr	#reset_vector	;failure check in.  Write to read only space.
	ldab	error_action	;sets CCs.
	beq	#u7_aed_found	;no flash error.
	jmp	flash_error	;Set to flash on error.

u7_aed_found:			;This is initial entry point.
	txs			;hold x in sp.	
	clrb
	ldx	temp_0		;x has error loc.
	stab	0,x		;clear error loc.
	tsx			;restore x.
	staa	0,x		;write to x, we expect to overwrite error loc.
	txs			;hold x in sp.
	ldx	temp_0		;x has error loc.
	ldab	0,x		;this will set CCs.
	tsx			;restore x.  This does not affect CCs.
	bne	u7_aed_foundlp	;if not equal we are seeing the error.

u7_aed_unot:

	ldab	#10		;setup to flash 10 times so we know that
				;we could not locate the error.
				;We will restart.
	ldx	u7_aedf_return	;x now has return address.
	jmp	sflash_sp_svc	;Let them know we cannot isolate error.
u7_aedf_return:

	ldx	#20000		;delay count.

u7_aed_delay:
	dex
	bne	#u7_aed_delay
	jmp	start		;There was an error so restart from scratch.


;	The next test expects that u7 has addr equal data pattern in it.
;	make it so.

u7_aed_passed:
	ldaa	#u7_min
	ldx	#u7_min
set_loop:
	staa	0,x
	inx			;next address.
	inca			;next pattern.
	cpx	#u7_max+1	;if equal then we are done.
	bne	#set_loop	;go write pattern.

;	U7 should now be completely address equal data.

	ldab	#$1		;flash #3.  U7 addr uniq part 2. U7 is good.
				;we don't call flash svc yet till we check
				;the 5101 addressing; as we want to check 
				;that the 5101 addressing does not affect
				;the 6810 at U7.
	ldx	#u7_aed_return	;x now has return address.
	jmp	#flash_sp_svc	;go flash.
u7_aed_return:

;	No delay loop is needed here.  We can see the LED just fine.

;===========================================================================

;	Verify that writing to u8 does not write to u7.

u7u8_uniq:
	ldx	#u7_min		; start of u7.
	ldaa	#$0		; zero to clear.

u7u8_clr:			; setup u7 data to known 00 is simplest.
	staa	0,x		; clear u7.
	inx
	cpx	#u7_max+1	; check if done.
	bne	#u7u8_clr	; continue or fall through.
	lds	#u8_max		; high addr of device we are checking against.
				; sp grows from high to low.
u7u8_lp:
	ldaa	#$FF		; Any pattern besides zero will do here.
	psha			; This will write a byte to u8.
	tsx			; x now has a copy of sp+1. sp is unchanged.
	cpx	#u8_min		; because got sp+1 this is a straight cmpare.
	beq	#u7u8_pass	; if done then pass.

u7u8_chk:
	ldx	#u7_min		; Setup to check out u7, it should still be 00.

u7u8_chklp:
	ldab	0,x		; read device location.  This will set CCs.
	bne	#u7u8_err	; if not zero goto error routine.

u7u8_reent:			; if error goes away in errloop then re-enter.
	inx			; inc x.
	cpx	#u7_max+1	; check if done.
	bne	#u7u8_chklp	; loop if not done
	bra	#u7u8_lp	; current did not overwrite, next u8 address.

u7u8_err:
	clr	#reset_vector	;failure check in.  Write to read only space.
	ldab	error_action	;sets CCs.
	beq	#u7u8_errlp	;no flash error.
	jmp	flash_error	;

u7u8_errlp:
	clr	0,x		; clear overwritten location.
	des			; sp will always be fail addr+1. fix it.
	psha			; write to suspect location.
	ldab	0,x		; check for failure.  This sets CCs.
	bne	#u7u8_errlp	; continue to loop on error if failing.
	bra	#u7u8_reent	;failure not solid;continue.

u7u8_pass:

	ldab	#$1		;flash #4.  u7 is unique from other devices.

	ldx	#u7u8_return	;x now has return address.
	jmp	#flash_sp_svc	;go flash.
u7u8_return:

	ldx	#20000		; since we passed hold off a bit so we can 
				; see the next flash occur for next test.
u7u8_delay:
	dex
	bne	#u7u8_delay ; Delay to see flash.

;===========================================================================

	lds	#stack_strt	;initialize the stack for use.

;	This will be the first time we use the stack.  We shall hope
;	that it is in good enough shape to use!

	jsr	one_flash	;flash #5.  U7/U8 uniq. Stack is now useable.
				;note: U7 has not been proven unique against
				;U10 at this point.  Will not likely matter.

	ldx	#20000		; since we passed hold off a bit so we can 
u78_s_delay:
	dex			; see the next flash occur for next test.
	bne	#u78_s_delay

;===========================================================================

	; From this point we will call (jsr) the rest of the tests
	; for convenience.  The only thing not specifically tested against
	; the stack is U10.  We have to flash through U11 so it has by nature,
	; been tested.  U10 has in theory been prove unique to the stack
	; by virtue of the fact nothing else has interfered with anything
	; at this point.

;===========================================================================

do_calls:

; Test U8 data.

	jsr	u8_data_test	;
	jsr	one_flash	;flash #6.  U8 data bits are unique.

;===========================================================================

; Test U8 addressing unique.

	jsr	u8_address_test	;
	jsr	one_flash	;flash #7.  U8 addresses are unique.

;===========================================================================

; Test U11 pia.

	ldx	#u11_sel	;u11 enable base address.
	stx	var_1		;passed argument.
	jsr	pia_test	;go test u11 pia.
	jsr	one_flash	;flash #8.  U11 PIA has been checked.

;===========================================================================

; Test bally/stern U10 pia.

	ldx	#u10_sel	;u10 enable base address.
	stx	var_1		;passed argument.
	jsr	pia_test	;go test u10 pia.
	jsr	one_flash	;flash #9.  U10 PIA has been checked.

;===========================================================================

; Test bally/stern u10 and u11 pia's are uniquely addressable.

	ldx	#u11_sel	;u11 enable base address.
	stx	var_1		;this will be pia1 in pia_uniq test.
	ldx	#u10_sel	;u10 enable base address.
	stx	var_2		;this will be pia2 in pia_uniq test.
	jsr	pia_uniq	;test that two pia's are uniquely addressed.
	jsr	one_flash	;flash #10. U10 and U11 are unique.

; Check to see if we are supposed to run the U13 test on the Stern MPU200.
	ldaa	test_mpu_200	;get status byte and check if mpu200 enabled.
	cmpa	#yes
	beq	run_mpu_200	;Do not run MPU 200 tests.

fake_flash_11:	

	ldx	#20000		;delay to differentiate from two flash start
fake_11a:			;of test marker.
	dex			;
	bne	fake_11a	;

	ldab	#1		;One flash.
	jsr	sflash_svc	;flash #11.  Short "fake" flash.

	ldx	#5000		;delay to differentiate from two flash start
fake_11b:			;of test marker.
	dex			;
	bne	fake_11b	;

fake_flash_12:
	
	ldab	#1		;One flash.
	jsr	sflash_svc	;flash #12.  Short "fake" flash.

	ldx	#20000		;delay to differentiate from two flash start
fake_12:			;of test marker.
	dex			;
	bne	fake_12		;
	jmp	past_mpu_200	;skip mpu200 u13 tests.


run_mpu_200:		;mpu 200 tests are the U13 tests.

;===========================================================================
; Test U13 data.   STERN MPU 200 only test.

	jsr	u13_data_test	;
	jsr	one_flash	;flash #11.  U13 data bits unique.


;===========================================================================

; Test U13 addressing unique.  STERN MPU 200 only test.

	jsr	u13_addr_test
	jsr	one_flash	;flash #12.  U13 addresses unique.

;===========================================================================

past_mpu_200:

;===========================================================================

	jmp	start		;For now we are done; start all over again.
	jsr	display_test	;Run display test.

;===========================================================================


; This is a short digits test.  Visual mode only for the fun of it.
; We should see digits regardless of the state of the interrupts.
; Beware that this test does not use interrpts and is only an indication
; of the data path to the displays.

display_test:
	ldx	#2		;run the digits through twice.
digit_lp:
	ldab	#00
digit_lp1:
	jsr	display_svc	;pump out digits onto player displays.
	addb	#$10		;BCD data is on high nibble!  Inc B reg.
	cmpb	#$A0		;max digit is nine; done if 10.
	bne	digit_lp1	;continue if not 10.
	dex			;check count in x.
	bne	digit_lp	;

;===========================================================================

;	End of testing; restart the test from the beginning.

	jmp	start		;For now we are done; start all over again.

;===========================================================================


;	This is the start of all common routines.


	; This is the U8 test.  It will verify data and address uniqueness.
	; because the 5101 CMOS Ram is only 4 bits wide; we take some
	; great pains to see to it that we find conflicting addresses
	; in the address uniqueness test for this part so we can loop on
	; error.  The pain however is well worth the gain
	; as it may make using a scope easier; and certainly makes things
	; crystal clear on a logic analyzer.

u8_data_test:

;	u8_data attempts to check that each cell can hold any
;	data pattern in the 5101.  address has not been qualified yet.


	ldx	#u8_min

u8_d_nextadr:
	ldaa	#$0		;aa will be expected data
	ldab	#$ff		;ab will be actual. Init to anything else; we
				;start at zero so this must get zero on a read
	bra	u8_d_datloop	

u8_d_errloop:
	clr	#reset_vector	;failure check in.  Write to read only space.
	ldab	error_action	;sets CCs.
	beq	#u8_d_datloop	;no flash error.
	jmp	flash_error	;

u8_d_datloop:
	anda	#$f0		;5101 is only 4 bits wide.  Mask unused bits.
	staa	0,x		;write to the 5101.
	ldab	0,x		;read from the 5101.
	andb	#$f0		;5101 is only 4 bits wide.  Mask unused bits.
	cba			;compare a and b accumulators.
	bne	#u8_d_errloop	;loop on the error till it clears.
	adda	#$10		;inca on MSB side, lsb 4 bits stay zero.
	anda	#$f0		;just to be sure.
	bne	#u8_d_datloop	;data is 00-FF on FF we roll to 00 indicating
				;we have checked all data on current address.
	inx			;inc x to next addr.
	cpx	#u8_max+1	;if past max addr then we are done.
	bne	#u8_d_nextadr	;not done; do next address.

	rts			;return to caller.


u8_address_test:
	ldx	#u8_min		;5101 starting address
	ldaa	#$0		;aa will be expected data
	staa	var_1		;Clear variable located in 6810 used for uniq.
	ldab	#$ff		;ab will be actual. Init to anything else; we
				;start at zero so this must get zero on a read
	bra	#u8_a_datloop

u8_a_errloop:
	clr	#reset_vector	;failure check in.  Write to read only space.
	ldab	error_action	;sets CCs.
	beq	#u8_a_datloop	;no flash error.
	jmp	flash_error	;

u8_a_datloop:
	adda	var_1		;A+M->A var_1 allows us to do uniqueness.
	anda	#$f0		;5101 is only 4 bits wide.  Mask unused bits.
	staa	0,x		;write to the 5101.
	ldab	0,x		;read/verify data.
	andb	#$f0		;5101 is only 4 bits wide.  Mask unused bits.
	cba			;compare a and b accumulators.
	bne	#u8_a_errloop	;loop on the error till it clears.
	adda	#$10		;inca. msb, already has the uniq cnt offset.
	anda	#$f0		;5101 is only 4 bits wide.  Mask unused bits.
	bne 	#u8_continue	;on 0 we will increment var_1.
	ldab	var_1		;get current cnt in b.
	addb	#$10		;inc b (really cnt) used to provide addr uniq.
	andb	#$f0		;5101 is only 4 bits wide.  Mask unused bits.
	stab	var_1		;put new count back in cnt.
u8_continue:
	inx			;inc x to next addr.
	cpx	#u8_max+1	;if past max addr then we are done.
	bne	#u8_a_datloop	;not done; do next address.

u8_addr_uniq:

u8_uclr:
	ldx	#u8_min		;Start addr of 5101.
	ldaa	#$0

u8_uclr_addr:
	staa	0,x		;clear 5101.
	inx			;inc x to next addr.
	cpx	#u8_max+1	;if past max addr then we are done.
	bne	#u8_uclr_addr	;not done.

	ldx	#u8_min
	ldaa	#$f0		;pattern to write to 5101.

u8_uloop:
	ldab	0,x		;check before writing.
	andb	#$f0		;mask out unused bits.
	cmpb	#$0		;verify loc is zero.	
	bne	u8_uerror	;go try to loop on error.
	staa	0,x		;loc was zero; write pattern to loc.
	inx			;next loc.
	cpx	#u8_max+1	;check if done.
	bne	u8_uloop	;not done.

	rts			;return to caller.


u8_uerror:
	stx	var_1		;loc that was in error.
	clr	0,x		;clear error loc.
	ldx	#u8_min		;setup to find location that overwrote
				;the location just stored in var_1.
	ldaa	#$f0		;reload a with same data.

u8_find_loc:

	staa	0,x
	txs			;hold x in sp for a moment.
	ldx	var_1		;now get fail addr to check.
	ldab	0,x		;
	tsx			;put x held in sp back into x.
	andb	#$f0		;mask out unused bits.
	stx	var_2		;Addr in var_2 confilicts with addr in var_1.
	cmpb	#$0		;check error loc was still zero.  If not,
	bne	u8_uerr_loop	;then try again.
	inx			;try next loc.
	cpx	var_1		;don't need to go past addr in var_1.
	beq	u8_unot		;did not find bad address.	

	ldaa	#$f0		;to be safe.

;	Note:  we enter at u8_uerr_loop, we loop to checkin.  This allows us to
;	see the problem occur immediately before error check in.
u8_checkin_lp:
	clr	#reset_vector	;failure check in.  Write to read only space.
	ldab	error_action	;sets CCs.
	beq	#u8_uerr_loop	;no flash error.
	jmp	flash_error	;Set to flash on error.

u8_uerr_loop:
	clr	0,x		;clear the addr in var_2.
	ldx	var_1		;address that destroyed data in loc var_2.
	staa	0,x		;write to var_1 should wipe data in var_2 addr.
	ldx	var_2		;check we destroyed data in var_2 addr.
	ldab	0,x		;read the data in var_2 addr.
	andb	#$f0		;mask unused.
	bne	#u8_checkin_lp	;stay in as long as we are failing.

u8_unot:
	ldab	#10
	jsr	sflash_svc	;Flash 10 times short so we know the error was 
	jmp	start		;not located, we will re-start the test.


	; This is the Stern u13 test.  It will verify data and address 
	; uniqueness.  because the 5101 CMOS Ram is only 4 bits wide; we take
	; some great pains to see to it that we find conflicting addresses
	; in the address uniqueness test for this part so we can loop on
	; error.  The pain however is well worth the gain
	; as it may make using a scope easier; and certainly makes things
	; crystal clear on a logic analyzer.
	; Note: this test should only be run if a Stern MPU-200 is being
	; tested.  It should not be called when testing Bally MPUs or
	; when testing Stern MPU-100 MPUs.
	; U13 is occupies the same address space as U8.  U8 uses data bits
	; 4 - 7 while U13 uses data bits 0 - 3.

u13_data_test:

;	u13_data attempts to check that each cell can hold any
;	data pattern in the 5101.  address has not been qualified yet.


	ldx	#u13_min

u13_d_nextadr:
	ldaa	#$0		;aa will be expected data
	ldab	#$ff		;ab will be actual. Init to anything else; we
				;start at zero so this must get zero on a read
	bra	u13_d_datloop	

u13_d_errloop:
	clr	#reset_vector	;failure check in.  Write to read only space.
	ldab	error_action	;sets CCs.
	beq	#u13_d_datloop	;no flash error.
	jmp	flash_error	;

u13_d_datloop:
	anda	#$0f		;5101 is only 4 bits wide.  Mask unused bits.
	staa	0,x		;write to the 5101.
	ldab	0,x		;read from the 5101.
	andb	#$0f		;5101 is only 4 bits wide.  Mask unused bits.
	cba			;compare a and b accumulators.
	bne	#u13_d_errloop	;loop on the error till it clears.
	adda	#$01		;inca on MSB side, lsb 4 bits stay zero.
	anda	#$0f		;just to be sure.
	bne	#u13_d_datloop	;data is 00-FF on FF we roll to 00 indicating
				;we have checked all data on current address.
	inx			;inc x to next addr.
	cpx	#u13_max+1	;if past max addr then we are done.
	bne	#u13_d_nextadr	;not done; do next address.

	rts			;return to caller.


u13_addr_test:
	ldx	#u13_min	;5101 starting address
	ldaa	#$0		;aa will be expected data
	staa	var_1		;Clear variable located in 6810 used for uniq.
	ldab	#$ff		;ab will be actual. Init to anything else; we
				;start at zero so this must get zero on a read
	bra	#u13_a_datloop

u13_a_errloop:
	clr	#reset_vector	;failure check in.  Write to read only space.
	ldab	error_action	;sets CCs.
	beq	#u13_a_datloop	;no flash error.
	jmp	flash_error	;

u13_a_datloop:
	adda	var_1		;A+M->A var_1 allows us to do uniqueness.
	anda	#$0f		;5101 is only 4 bits wide.  Mask unused bits.
	staa	0,x		;write to the 5101.
	ldab	0,x		;read/verify data.
	andb	#$0f		;5101 is only 4 bits wide.  Mask unused bits.
	cba			;compare a and b accumulators.
	bne	#u13_a_errloop	;loop on the error till it clears.
	adda	#$01		;inca. msb, already has the uniq cnt offset.
	anda	#$0f		;5101 is only 4 bits wide.  Mask unused bits.
	bne 	#u13_continue	;on 0 we will increment var_1.
	ldab	var_1		;get current cnt in b.
	addb	#$01		;inc b (really cnt) used to provide addr uniq.
	andb	#$0f		;5101 is only 4 bits wide.  Mask unused bits.
	stab	var_1		;put new count back in cnt.
u13_continue:
	inx			;inc x to next addr.
	cpx	#u13_max+1	;if past max addr then we are done.
	bne	#u13_a_datloop	;not done; do next address.

u13_addr_uniq:

u13_uclr:
	ldx	#u13_min	;Start addr of 5101.
	ldaa	#$0

u13_uclr_addr:
	staa	0,x		;clear 5101.
	inx			;inc x to next addr.
	cpx	#u13_max+1	;if past max addr then we are done.
	bne	#u13_uclr_addr	;not done.

	ldx	#u13_min
	ldaa	#$0f		;pattern to write to 5101.

u13_uloop:
	ldab	0,x		;check before writing.
	andb	#$0f		;mask out unused bits.
	cmpb	#$0		;verify loc is zero.	
	bne	u13_uerror	;go try to loop on error.
	staa	0,x		;loc was zero; write pattern to loc.
	inx			;next loc.
	cpx	#u13_max+1	;check if done.
	bne	u13_uloop	;not done.

	rts			;return to caller.


u13_uerror:
	stx	var_1		;loc that was in error.
	clr	0,x		;clear error loc.
	ldx	#u13_min		;setup to find location that overwrote
				;the location just stored in var_1.
	ldaa	#$0f		;reload a with same data.

u13_find_loc:

	staa	0,x
	txs			;hold x in sp for a moment.
	ldx	var_1		;now get fail addr to check.
	ldab	0,x		;
	tsx			;put x held in sp back into x.
	andb	#$0f		;mask out unused bits.
	stx	var_2		;Addr in var_2 confilicts with addr in var_1.
	cmpb	#$0		;check error loc was still zero.  If not,
	bne	u13_uerr_loop	;then try again.
	inx			;try next loc.
	cpx	var_1		;don't need to go past addr in var_1.
	beq	u13_unot	;did not find bad address.	

	ldaa	#$0f		;to be safe.

;	Note:  we enter at u13_uerr_loop, we loop to checkin.  This allows us to
;	see the problem occur immediately before error check in.
u13_checkin_lp:
	clr	#reset_vector	;failure check in.  Write to read only space.
	ldab	error_action	;sets CCs.
	beq	#u13_uerr_loop	;no flash error.
	jmp	flash_error	;Set to flash on error.

u13_uerr_loop:
	clr	0,x		;clear the addr in var_2.
	ldx	var_1		;address that destroyed data in loc var_2.
	staa	0,x		;write to var_1 should wipe data in var_2 addr.
	ldx	var_2		;check we destroyed data in var_2 addr.
	ldab	0,x		;read the data in var_2 addr.
	andb	#$0f		;mask unused.
	bne	#u13_checkin_lp	;stay in as long as we are failing.

u13_unot:
	ldab	#10
	jsr	sflash_svc	;Flash 10 times short so we know the error was 
	jmp	start		;not located, we will re-start the test.




	; This pretty much completes addr uniqueness except for
	; the pia's selection and rom selection.
	; Writing to a PIA could, in theory cause a write
	; to the 6810 or the 5101; though this is highly unlikely,
	; there for we will not test it.

;===========================================================================


	; This is the PIA test for U10 and U11.  U11 is already pretty
	; much known to work at this point as we have been using
	; it to flash the LED.  U10 has never been touched however; nor
	; do we know that they are unique yet.


	; pia_test is a called routine.
	; var_1 on entry contains the base enable addr for the pia to test.
	; On error this call will loop on the error. On Pass it will return.
	; to the caller.

pia_err:
	clr	#reset_vector	;failure check in.  Write to read only space.
	ldab	error_action	;sets CCs.
	beq	#pia_test	;no flash error.
	jmp	flash_error	;

pia_test:
	ldx	var_1		;x has pia enable base addr.
	ldaa	#pia_ddr_reg	;access ddr reg.
	oraa	#led_off	;fix bug where LED stays on in pia_test3
	staa	cra_sel,x	;set ddr access.  PIA is now setup.
	clra			;
	staa	pia_reg_b_sel,x	;should clear ddrb reg.
	ldaa	#$AA		;test pattern.
	staa	pia_reg_a_sel,x	;should write to ddra reg.
	ldab	pia_reg_a_sel,x	;should read from ddra reg.
	cba			;compare expected in A to actual in B.
	bne	#pia_err	;loop on error.

	clra			;clear a reg to verify ddrb was unique in pia.
	ldab	pia_reg_b_sel,x	;ddrb should still be clear if unique.
	cba			;compare expected in A to actual in B.
	bne	#pia_test	;loop on error.  Not unique.
	bra	#pia_test_1

pia_err_1:
	clr	#reset_vector	;failure check in.  Write to read only space.
	ldab	error_action	;sets CCs.
	beq	#pia_test_1	;no flash error.
	jmp	flash_error	;

pia_test_1:

	ldaa	#$55		;test pattern.
	staa	pia_reg_a_sel,x	;should write to ddra reg.
	ldab	pia_reg_a_sel,x	;should read from ddra reg.
	cba			;compare expected in A to actual in B.
	bne	#pia_err_1	;loop on error.
	bra	#pia_test_2

pia_err_2:
	clr	#reset_vector	;failure check in.  Write to read only space.
	ldab	error_action	;sets CCs.
	beq	#pia_test_2	;no flash error.
	jmp	flash_error	;

pia_test_2:			;start checking drb reg.
	clra			;
	staa	pia_reg_a_sel,x	;clear ddra for uniqueness check.
	ldaa	#$55		;test pattern.
	staa	pia_reg_b_sel,x	;should write to ddrb reg.
	ldab	pia_reg_b_sel,x	;should read from ddrb reg.
	cba			;compare expected in A to actual in B.
	bne	#pia_err_2	;loop on error.
	bra	#pia_test_3

pia_err_3:
	clr	#reset_vector	;failure check in.  Write to read only space.
	ldab	error_action	;sets CCs.
	beq	#pia_test_3	;no flash error.
	jmp	flash_error	;

pia_test_3:	
	ldaa	#$AA		;test pattern.
	staa	pia_reg_b_sel,x	;should write to ddrb reg.
	ldab	pia_reg_b_sel,x	;should read from ddrb reg.
	cba			;compare expected in A to actual in B.
	bne	#pia_err_3	;loop on error.
	ldab	pia_reg_a_sel,x	;ddra should still be clear. Sets CCs.
	bne	#pia_err_2	;ddra/b not unique loop to err_2 to see writes.
				;Note:  This error will leave LED ON if cra
				;was not loaded with ddra sel OR'd w/ led_off.
				;Problem was cra set to CA2 input which is
				;pulled up on the board thus turning ON!
				;Now OR'd with led_off at start of pia_test.
	rts			;If here we passed.  Return to caller.

;===========================================================================

	;this test tests that two pia'pia's are uniquely addressable.
	;var_1 contains the addr of the first pia to check.
	;var_2 contains the addr of the second pia to check.
	;on error this test loops.
	;on Pass this test returns to the caller.

pia_uniq_err:
	clr	#reset_vector	;failure check in.  Write to read only space.
	ldab	error_action	;sets CCs.
	beq	#pia_uniq	;no flash error.
	jmp	flash_error	;

pia_uniq:
	ldx	var_1		;x has pia1 enable base addr.
	ldaa	#pia_ddr_reg	;access ddr reg.
	staa	cra_sel,x	;set ddr access.  PIA is now setup.
	clra			;
	staa	pia_reg_a_sel,x	;should clear ddra reg in pia1.
	staa	pia_reg_b_sel,x	;should clear ddrb reg in pia1.

	ldx	var_2		;x has pia2 enable base addr.
	ldaa	#pia_ddr_reg	;access ddr reg.
	staa	cra_sel,x	;set ddr access.  PIA is now setup.

	ldaa	#$AA		;test pattern.
	staa	pia_reg_a_sel,x	;write one pattern to ddra in pia2.
	staa	pia_reg_b_sel,x	;write one pattern to ddrb in pia2.
	ldaa	#$55		;test pattern.
	staa	pia_reg_a_sel,x	;write one pattern to ddra in pia2.
	staa	pia_reg_b_sel,x	;write one pattern to ddrb in pia2.

	ldx	var_1		;x has pia1 enable base addr.
	clra			;
	ldab	pia_reg_a_sel,x	;ddra reg in pia1 should still be zero.
	cba			;compare expected in A to actual in B.
	bne	#pia_uniq_err	;loop on error.
	ldab	pia_reg_b_sel,x	;ddrb reg in pia1 should still be zero.
	cba			;compare expected in A to actual in B.
	bne	#pia_uniq_err	;loop on error.

	rts			;return to caller.


	;This completes the checking of the PIA for now.
	;The PIA testing currently only attempts to verify the
	;two data direction registers ddra and ddrb.

;===========================================================================

;		The following are support routines.


;===========================================================================

	; flash_error.
	; This will set the b register to 5 flashes and jmp to sflash_sp_svc.
	; This routine does not return, it restarts.


flash_error:			;Currently we do not return so we do much.
				;and the heck with the stack.  It matters not.
	ldx	#20000		;delay so we can tell the difference.
				;when looping.
flash_err_delay:
	dex
	bne	#flash_err_delay ; Delay to see flashes clearly.

	ldab	#$5		;b now holds the number of times to flash.
	ldx	#flash_err_ret	;x has return address.
	jmp	sflash_sp_svc	;Go flash the light 5 short times.
				;sflash_sp returns to our caller.
flash_err_ret:
	ldaa	error_action	;Find out what to do here.
	cmpa	#halt_val	;
	beq	#halt_loop	;effectively halt the testing.
	cmpa	#restart_val	;
	beq	#flash_error_rst	;go restart tests.
				;any other value will currently restart also,
				;but is not guaranteed to do so in later
				;revisions.
flash_error_rst:
	jmp	start		;flash complete, now restart from scratch.

;===========================================================================

		;This loop does nothing and is an infinite loop on itself
		;as there is no halt instruction for the 6800.

halt_loop:
	clr	#reset_vector	;failure check in.  Write to read only space.
halt_lp:
	nop
	nop
	bra	#halt_lp	;loop on nothing to simulate a halted program.

;===========================================================================
		;common_return will allow us to save some space rather
		;than repeat code for the restore/rts of some calls.
		;It also makes us consistent.

common_return:

	pulb			;restoring x.
	pula			;restoring x.
	stab	x_place+1	;restoring x.
	staa	x_place		;restoring x.
	ldx	x_place		;x is now restored.
	pulb			;restore b.
	pula			;restore a.
	rts			;return to caller.

;===========================================================================

one_sflash:

	pshb			;save b register.

	ldab	#$1		;b now holds the number of times to flash.
	jsr	sflash_svc	;Go flash the light.

	pulb			;restore b register.
	rts			;return to caller.

;===========================================================================

	; one_flash is a called routine.  It will flash the led
	; N number of times based on the value in the b register.
	; there are no passed parameters.
one_flash:

	pshb			;save b register.

	ldab	#$1		;b now holds the number of times to flash.
	jsr	flash_svc	;Go flash the light.

	pulb			;restore b register.
	rts			;return to caller.

;===========================================================================

	; flash_svc:
	; flash the LED N times based on contents of b register.
	; Registers are preserved.

flash_svc:

	psha			;Save A.
	pshb			;Save B.
	stab	temp		;b has count need to hold b reg temporarily.
	stx	x_place		; setup to push x also.
	ldaa	x_place		; 
	ldab	x_place+1	;
	psha			; x high onto stack
	pshb			; x low onto stack
	ldab	temp		; b should now have the flash count again.

flash_loop:

	ldx	#u11_sel	;x has u11 enable base addr.
	ldaa	#led_on		;aa set to enable CA2 as output, LED on.
	staa	cra_sel,x	;LED should turn on now.
	ldx	#30000		;LED On count.

flash_on:

	dex			; should decrement x by 1.
	bne	#flash_on	; keep on for LED on count. 

;	setup to turn off now to complete flash.

	ldx	#u11_sel	;x has u11 enable base addr.
	ldaa	#led_off	;aa set to enable CA2 as output, LED on.
	staa	cra_sel,x	;LED should turn off now.
	ldx	#30000		;LED Off count.

flash_off:
	dex			;should decrement x by 1.
	bne	#flash_off	;keep off for LED off count. 
	decb
	bne	#flash_loop	;go till done flashing.

	jmp	common_return	;common return routine to save some space.
				;and also keep things simple.

;===========================================================================

	; sflash_svc:
	; same as flash_svc except this does short flashes.
	; flash the LED N times based on contents of b register.
	; Registers are preserved.


sflash_svc:

	psha			;Save A.
	pshb			;Save B.
	stab	temp		;b has count need to hold b reg temporarily.
	stx	x_place		; setup to push x also.
	ldaa	x_place		; 
	ldab	x_place+1	;
	psha			; x high onto stack
	pshb			; x low onto stack
	ldab	temp		; b should now have the flash count again.

sflash_loop:

	ldx	#u11_sel	;x has u11 enable base addr.
	ldaa	#led_on		;aa set to enable CA2 as output, LED on.
	staa	cra_sel,x	;LED should turn on now.
	ldx	#9000		;LED On count.

sflash_on:

	dex			; should decrement x by 1.
	bne	#sflash_on	; keep on for LED on count. 

;	setup to turn off now to complete flash.

	ldx	#u11_sel	;x has u11 enable base addr.
	ldaa	#led_off	;aa set to enable CA2 as output, LED on.
	staa	cra_sel,x	;LED should turn off now.
	ldx	#22000		;LED Off count.

sflash_off:
	dex			; should decrement x by 1.
	bne	#sflash_off	; keep off for LED off count. 
	decb
	bne	#sflash_loop	; go till done flashing.

	jmp	common_return	;common return routine to save some space.
				;and also keep things simple.		

;===========================================================================

	; flash_sp_svc
	; flash the LED N times based on contents of b register.
	; On Entry this routine expects the return address in x.
	;
	; This routine should be used prior to stack availability.
	; flash_svc is a called routine that should be used once 
	; the stack has been tested good if you need to save registers.
	; This routine does not save any registers!!

flash_sp_svc:

	txs			; Hold return address in sp.

flash_sp_loop:

	ldx	#u11_sel	;x has u11 enable base addr.
	ldaa	#led_on		;aa set to enable CA2 as output, LED on.
	staa	cra_sel,x	;LED should turn on now.
	ldx	#30000		;LED On count.

flash_sp_on:

	dex			; should decrement x by 1.
	bne	#flash_sp_on	; keep on for LED on count. 

;	setup to turn off now to complete flash.

	ldx	#u11_sel	;x has u11 enable base addr.
	ldaa	#led_off	;aa set to enable CA2 as output, LED on.
	staa	cra_sel,x	;LED should turn off now.
	ldx	#30000		;LED Off count.

flash_sp_off:
	dex			;should decrement x by 1.
	bne	#flash_sp_off	;keep off for LED off count. 
	decb
	bne	#flash_sp_loop	;go till done flashing.
	tsx			;restore return address from sp.
	jmp	0,x		;return to caller.

;===========================================================================

	; sflash_sp_svc
	; flash the LED N times based on contents of b register.
	; On Entry this routine expects the return address in x.
	;
	; This routine should be used prior to stack availability.
	; sflash_svc is a called routine that should be used once 
	; the stack has been tested good if you need to save registers.
	; This routine does not save any registers!!

sflash_sp_svc:

	txs			; Hold return address in sp.

sflash_sp_loop:

	ldx	#u11_sel	;x has u11 enable base addr.
	ldaa	#led_on		;aa set to enable CA2 as output, LED on.
	staa	cra_sel,x	;LED should turn on now.
	ldx	#9000		;LED On count.

sflash_sp_on:

	dex			; should decrement x by 1.
	bne	#sflash_sp_on	; keep on for LED on count. 

;	setup to turn off now to complete flash.

	ldx	#u11_sel	;x has u11 enable base addr.
	ldaa	#led_off	;aa set to enable CA2 as output, LED on.
	staa	cra_sel,x	;LED should turn off now.
	ldx	#22000		;LED Off count.

sflash_sp_off:
	dex			; should decrement x by 1.
	bne	#sflash_sp_off	; keep off for LED off count. 
	decb
	bne	#sflash_sp_loop	; go till done flashing.
	tsx			;restore return address to x.
	jmp	0,x		;return to caller.


	;display_svc displays a digit in all player displays.
	;currently we do not do the match/ball in play display
	;aka status display.  On entry B hold the digit value.
	;registers are preserved.

display_svc:
;	attempt to display digits in displays.

	psha			;save A reg.
	pshb			;save B reg.
	stab	temp		;b holds value to display. IE: 0-9.
	stx	x_place		; setup to push x also.
	ldaa	x_place		;
	ldab	x_place+1	;
	psha			; x high onto stack
	pshb			; x low onto stack
	ldab	temp		; display value should be restored.
	comb			; compliment BCD to the display.

	ldx	#u11_sel	;x has u11 enable base addr.
	ldaa	#led_off	;aa set to enable U11 CA2 as output.
	oraa	#pia_ddr_reg	;add in ddr sel.  Note we keep the LED off!
	staa	cra_sel,x	;CA2 is now low; selecting ddra.
	ldaa	#$FF		;setup to enable PA0-PA7 out.
	staa	pia_ddr_reg,x	;PA0-PA7 of U11 now enabled as outputs.
	ldaa	#$00		;All digit strobes (Negated) ignore match/stat.
				;Note: A $C0 would shut off the left digits. 
	staa	pia_reg_a_sel,x	;set digit strobes; match status display off.

	;set blanking by setting CA2 LOW to enable displays to latch data.

	ldx	#u10_sel	;x has u10 enable base addr.
	ldaa	#led_off	;aa set to enable U10 CA2 as output.
	oraa	#pia_ddr_reg	;add in ddr sel.
	staa	cra_sel,x	;CA2 is now low and blanking; selecting ddra.
	ldaa	#$FF		;setup to enable PA0-PA7 out.
	staa	pia_ddr_reg,x	;PA0-PA7 U10 enabled. BCD and player latches.
	andb	#$F0		;Guarantee BCD data is valid in high nibble.
	orab	#$0F		;BCD with latches cleared.
	stab	pia_reg_a_sel,x	;BCD from b reg now ouputing at U10 PA4-7.
	andb	#$F0		;BCD with latches set.
	stab	pia_reg_a_sel,x	;player latches go high.
	orab	#$0F		;ready to reset player latches.
	stab	pia_reg_a_sel,x	;BCD with latches cleared.
	ldaa	#led_on		;aa set to enable U10 CA2 as output.
	staa	cra_sel,x	;CA2 high, blanking off, data now on displays.

	ldaa	#blank_time	;loops before toggle blank for RC network.
	staa	blank_cnt	;Serves as a counter for now.
	stx	x_place		;holding x pointing to U10.
	ldx	#10000		;pause in an attempt to delay to see digits.
display_lp:

	dec	blank_cnt	;
	bne	display_lp2	;go around and continue.  No blank this time.
	stx	temp		;storing count.
	ldx	x_place		;restore x pointing to U10.
	ldaa	#led_off	;set to blank.
	staa	cra_sel,x	;CA2 now low; blanking is on.
	ldaa	#led_on		;set to not blank.
	staa	cra_sel,x	;CA2 now high; blanking is off displays on.
	ldx	temp		;restore count into x.

display_lp2:

	tst	blank_cnt	;
	bne	display_lp3	;
	ldaa	#blank_time	;loops before toggle blank for RC network.
	staa	blank_cnt	;reloading blank_count.
display_lp3:

	dex			;dec count in x
	bne	display_lp	;delay if x not zero.

	;set blanking to turn off displays prior to exiting.

	ldx	#u10_sel	;x has u10 enable base addr.
	ldaa	#led_off	;aa set to enable U10 CA2 as output.
	oraa	#pia_ddr_reg	;add in ddr sel.
	staa	cra_sel,x	;CA2 is now low; selecting ddra.

	ldx	#5000		;pause to see that we blanked succesfully.
display_lp1:

	dex
	bne	display_lp1	;delay if x not zero.

	jmp	common_return	;common return routine to save some space.
				;and also keep things simple.


;===========================================================================

	; Vector setup for various interrupts should they occur.

	org	u6_end-7	; start of 6800 vector table.

	; Point unused interrupts to halt_loop.
	; Some day we may toy some more with them given a reason.

	fdb	halt_loop	; FFF8,FFF9 IRQ	CURRENTLY NOT USED HERE.
	fdb	halt_loop	; FFFA,FFFB SWI	CURRENTLY NOT USED HERE.
	fdb	show_revision	; FFFC,FFFD NMI CLEAR SWITCH (SW 33).
	fdb	start		; FFFE,FFFF RESET.

;===========================================================================
