;
; Written by Dallas Overturf 12-9-92
;
; DEO:  Released to Public Domain Nov. 16, 1995
;
; Questions Comments etc... On this program should be directed to the
; author at the following address (If you wish a reply please send a SASE)
;
; Dallas E. Overturf
; 267 Main St
; Ashland, MA 01721 (USA)
;
; e-mail:  overturf@vos.stratus.com
;
;
;		Memory Map for Williams Level 3-7 MPU
;		(Includes all PIAs on mpu and driver)
;
;			ADDRESS
;Device	| 15 14 13 12 11 10 9 8 7 6 5 4 3 2 1 0
;==============================================================================
;IC19	|     0  0  0       0 1 x x x x x x x x	 5101	100-1FF Level 3-7 CMOS
;------------------------------------------------------------------------------
;IC13	|     0  0            0 0 x x x x x x x	 6810	00-7F Level 3-6 ram
;------------------------------------------------------------------------------
;IC13	|     0  0  1       x x x x x x x x x x	 2114	1000-13FF Level 7 ram
;------------------------------------------------------------------------------
;IC16	|     0  0            0 1 x x x x x x x	 6810	00-7F Level 3-6 ram
;------------------------------------------------------------------------------
;IC16	|     0  0  1       x x x x x x x x x x	 2114	1000-13FF Level 7 ram
;------------------------------------------------------------------------------
;PIA 1  |     0  1     1                    x x         2800-2803 Displays
;------------------------------------------------------------------------------
;PIA 2  |     0  1  1                       x x         3000-3003 switches
;------------------------------------------------------------------------------
;PIA 3  |     0  1        1                 x x         2400-2403 lamps
;------------------------------------------------------------------------------
;PIA 4  |     0  1          1               x x         2200-2203 solenoids
;------------------------------------------------------------------------------
;PIA 5  |     0  1            1             x x         2100-2103 L7 snd/commas
;------------------------------------------------------------------------------
;ic17_L7      1  1  1  x  x x x x x x x x x x x	 IC17 L7 7000-7FFF Program upper addresses.
;------------------------------------------------------------------------------
;ic17_L36     1  1  1  1  x x x x x x x x x x x  IC17 L346 7800-7FFF Program upper addresses.
;------------------------------------------------------------------------------
;ic20   |     1  1  1  x  x x x x x x x x x x x	 IC20	6800-6FFF Program.
;------------------------------------------------------------------------------
;ic14	|     1  1  0  0  x x x x x x x x x x x  IC14 	6000-67FF Program.
;------------------------------------------------------------------------------
;=============================================================================

;Our Test prom code goes into IC17.burn into address 7800-7FFF.  

;=============================================================================
;Notes:  An 'x' indicates used for addressing, Nothing indicates don't care;
; a '1' or a '0' indicates TRUE or FALSE address line state to enable the part.
;CMOS map shown will work for levels 3-7; and is combined via commonalities
;between 3-6 and 7.  Looking at just 3-6 or 7 you won't see this unless you
;look at both.
; 
; This code will do a thorough test of the Williams MPU Level 3-6
; memory/bus; with scope loops on errors.  This should be a
; reasonably sufficient check of the board prior to booting with actual
; Williams Rom.  If this program completes without error then the tests
; in the actual Williams Game Rom should be sufficient to find other
; problems which are not likely to be memory/bus related such as 
; stuck switch, display, and solenoid problems.
; What this test will not do:
;
;
; 1.  	This test will not run with shorted data lines and will likely not
; 	run with shorted address lines.  It can easily be used to identify
;	most shorted address and/or data lines with the help of a logic
;	analyzer and this listing.
;
; 2.	Contrary to Williams statements; that the mpu will pass selftest
;	with the driver board disconnected; you MAY need the driver board 
;	plugged in to run this test or the Williams Self Tests.  The
;	reasoning behind this is as follows:
;	Looking at a Williams Level 6 MPU you will find two unused locations
;	at the bottom of the board near the driver board connector.  These
;	two locations used to be at one time (according to the schematics
;	part list) 4.7k pullups.  I have had MPUs fail the Williams
;	self test on the 5101 ram with the driver disconnected but run
;	with it connected.  This would imply that the driver is having
;	an effect on the mpu; likely pulling up the address and data lines
;	that these pullups once went to.  I have in fact partially proven
;	this by adding in pullups on just the data lines to a board that
;	was otherwise known good.  The board got past the 5101 check at
;	that point and was partially able to strobe digits.  
;	That was all the proof I needed and all I offer.  Beware and use a
;	known good driver board when checking an mpu once the mpu has been
;	partially checked out by itself and especially if it fails with
;	a CMOS ram failure indication.
;
; 3.	This test is most effective when used with a logic analyzer
;	such as an HP 1630 or some other such analyzer.  With an analyzer
;	and this listing it should be possible to see what bits are shorted
;	in the case of a short.  With just a scope that will be difficult to
;	see; though opens will be detectable provided the program prom 
;	address	and data lines are intact to the mpu, the clock to the
;	microprocessor is running, reset has been release and various other
;	signals to/from the microprocessor are functioning.  The first thing
;	to check if the program does not run is to check that as reset
;	goes away from the microprocessor, that the processor does a fetch
;	from address FFFE and FFFF and that the data comming out is in fact
;	a 7800.  The 7800 is the starting address of this program.  The user
;	should be able to follow the program from there and watch
;	the microprocessor fetch and execute if necessary by tracing on
;	selected points in the program till you find where the program goes
;	into hyperspace.
;
;	Note:  All findable errors report in by writing once to the
;		reset_vector of the microprocessor, address FFFE.  Error
;		loops are setup to continue the program if the error goes
;		away and the loop gets good data.  Thus a flakey problem is
;		best caught using an analyzer and tracing on a write to
;		address FFFE.
;
;		Note that error loops may or may not continually write to 
;		the error check in address (FFFE).  All errors found will
;		write to the error check in address at least once.  
;		Thus when using	a logic analyzer, it is recommended that
;		you look for error check in starting from power up so that
;		there is no chance that you miss it by already being in
;		an error loop that does not repeat error checkin.
;
; Testing Notes:
;
;	This test prom is placed into socket U17 to be used.
;	Note socket u17 must be jumpered to match the prom type used.
;	If this prom code is put into a 2732 it may be used in a u17
;	socket that is jumpered as a 2716 by programming the upper half
;	of the 2732 from 0800 - 0FFF
;
;	The test prom should be in the socket that will be enabled by
;	an address of FFFE.  (the reset vector of the 6800).
;
;	Programming the code into an EPROM:
;	For a 2716 just program it in normal (IE: start address is 0000).
;	If your burner requires an end address, it is 07FF.
;
;	For a 2732 or a 2532 the start address is 0800 (this means 0000-07FF
;	remain blank.  If your burner requires an end address, it is 0FFF.
;
;	NOTE:  If the binary file of this program is larger than 2048 bytes;
;	and the file starts at address 0000; then you will need to give your
;	prom programmer a starting address within the binary file as well to
;	start taking data from.  In this case you need to give it address
;	7800 which will be the offset (in hexadecimal) of the start of code
;	in the binary file.
;
;
;	It is recommended that you verify the prom in a known good board
;	prior to attempting to use it on a bad board!
;
;	Things of that may keep you from comming up and working correctly
;	on the game rom that are untested in this game.
;	INTERRUPTS!!  This test program specifically does not test program
;	interrupts (the IRQ lines on the 6800 and PIAs).
;
;
; Modification history
;
;		12-28-92 Last development file is wilt18.asm
;		Remove Development history.  Set revision_val to 2.
;		Add in revision and error_action routines.
;		setup jmp to flash_error after checkin.
;		File is wilt19.asm  Revision is 2. 
;
;		04-15-93 Added up front dummy reads and writes to allow
;		us to basically verify addr and data buses are okay
;		prior to hitting any real routines.  These are unchecked.
;
;		05-26-96 By DEO:  Fix a problem added in upgrading
;		pia testing I believe.  pia_loop3 should not have
;		done a comare just prior to the last bne before rts.
;		The expected data was 0 the ldab will set the CCs and
;		that is all that is needed.  We had a cba and were comparing
;		AA or 55 to 0 which of course will always fail!
;
;
; End Modification history
;
;
;
;	Notes:
;		1.  When a 6802 microprocessor is used and ram1 is not
;			per Williams schematics notes: if any ram1 test
;			fails; then the processor should be changed if
;			it is not absolutely known good as ram1 exists
;			within the 6802 processor.  Also it would be
;			suggested that the processor jumper for ram1 out
;			is checked to be correct.  (Pin 36 of the 6802
;			"RE" must be pulled up to enable the 6802
;			internal RAM or if not used Pin 36 should be
;			tied to ground.)
;			
;
;
;
;
;

ic17_strt	equ	$7800	
ic17_end	equ	$7fff	
ic20_strt	equ	$7000	
ic20_end	equ	$77ff	
ram1_min	equ	$0	;low addr for 6810. In 6802 or 6810 if 6808. 
ram1_max	equ	$7f	;high addr for 6810. In 6802 or 6810 if 6808.  
ram2_min	equ	$80	;low addr for 6810. Second Ram.
ram2_max	equ	$ff	;high addr for 6810. Second Ram.
cmos_min	equ	$100	;low addr for 5101.
cmos_max	equ	$1ff	;high addr for 5101.
pia1_mask	equ	$2803 	;mask for pia1. Displays/LED
pia1_sel	equ	$2800 	;Enable for pia1. Display/LED
pia2_mask	equ	$3003 	;mask for pia2. Switch Matrix.
pia2_sel	equ	$3000 	;Enable for pia2. Switch Matrix.
pia3_mask	equ	$2403 	;mask for pia3.  Lamps.
pia3_sel	equ	$2400 	;Enable for pia3. Lamps.
pia4_mask	equ	$2203 	;mask for pia4.  Solenoids.
pia4_sel	equ	$2200	;Enable for pia4. Solenoids.
pia_reg_a_sel	equ	$00  ;Register A slections in PIA.  For DDRA and PRA
cra_sel		equ	$01  ;control register A.
pra_sel		equ	$00  ;For Peripheral Registrer A.
ddra_sel	equ	$00  ;For DDRA.
pia_reg_b_sel	equ	$02  ;Register B slections in PIA.  For DDRB and PRB
crb_sel		equ	$03  ;control register B.
prb_sel		equ	$02  ;For Peripheral Registrer B.
ddrb_sel	equ	$02  ;For DDRA.
;  NOTE these next two are specified in binary so you know the syntax.
;  specifying in hex is probably easier to look at and read.% is binary. 
pia_ddr_reg	equ	%00000000 ; access data direction reg access.
pia_out_reg	equ	%00000100 ; access output reg access. binary = $0004

led_1_bit	equ	$20	;Turns On LED 1. Upper LED set in ddra and pra.
led_2_bit	equ	$10	;Turns On LED 2. Lower LED set in ddra and pra.
led_on		equ	$38	;should turn on LED.  Sets CRA-2,3,4,5.
led_off		equ	$30	;should turn off LED.
led_bits	equ	$30	;led bits are PA4, PA5 set in ddra to use pra.
led_bits_on	equ	$30	;led bits; PA4, PA5 set to light; set in pra.
led_bits_off	equ	$00	;led bits; PA4, PA5 reset to clear; set in pra.
irq_vector	equ	$FFF8	; FFF8,FFF9  IRQ interrupt.
swi_vector	equ	$FFFA	; FFFA,FFFB SWI	interrupt.
nmi_vector	equ	$FFFC	; FFFC,FFFD NMI CLEAR SWITCH (SW 33).
reset_vector	equ	$FFFE	; FFFE,FFFF RESET vector for 68xx processors.
loop_val	equ	$0	; see error_action settings, or flash_error.
restart_val	equ	$1	; see error_action settings, or flash_error.
halt_val	equ	$2	; see error_action settings, or flash_error.
revision_val	equ	$2	; Revision.

;
; THE FOLLOWING LOCATIONS ARE RESERVED IN THE 6810 AS PROGRAM VARIABLES
; THEY MAY NOT BE ACCESSED UNTIL AFTER WE HAVE DECIDED THE 6810 IS OK.
;

stack_strt	equ	$4f	; stack will be from 4f to 0, with the
				; top $30 locations reserved for variables.

; Variable region exists from addr $50 to addr $7f.
var_1		equ	$50	; two bytes for first variable in call
var_2		equ	$52	; two bytes for second variable in call
var_3		equ	$54	; two bytes for third variable in call
x_place		equ	$56	; two bytes for allowing push of x.
temp		equ	$58	; two bytes used for local temp storage.

; define  some important space for address uniqness testing of 6810 RAM.
temp_0		equ	ram1_min	; two bytes of space at 00 and 01 in RAM.

;=============================================================================

	org 	ic17_strt	;start of program.

;	Note:  "error_action" is intended to always be at u6_strt
;	for convenience and is the first byte of u6.  The next few
;	bytes or so are the REV string.  Followed by copyright notice.

error_action:
	fcb	#loop_val	;set to restart_val; will cause 5 flashes on
				;error and then restart the test.  Set to 
				;loop_val; will loop on error.  If the error
				;goes away, the test will continue.  Set to
				;halt_val; will effectively halt the test.
				;See description of error_action.

	fcb	#revision_val	;Put rev in rom also in case we want to know.

;		Now put in my copyright notice.

	fcc	"MY WILLIAMS TEST PROM LEVEL 3,4,6 "
	fcc	"WRITTEN 1992 BY DALLAS E. OVERTURF "

;	A Non Maskable Interrupt (switch 33) will restart at show_revision.

show_revision:

	ldab	#revision_val	;b now has revision.
	ldx	#revision_ret	;x has return address.
	jmp	sflash_sp_svc	;
revision_ret:
	ldaa	#2		;two times will be a very long wait.
revision_lp:
	ldx	#30000		;long wait.
rev_x_lp:
	dex			;dec wait count
	bne	rev_x_lp	;
	deca			;dec loop count.
	bne	#revision_lp	;branch if not done or fall through to start.

start:

	;1st write and or read some data and address patterns to see that
	;the address and data buses are basically ok on the analyzer.

	ldx	#$0000		;An address to write to write to. = 0000.
	ldaa	#$0		;some data to write= 00.
	staa	0,x		;unchecked data only to look at bus.
	coma			;some data to write= FF.
	staa	0,x		;unchecked data only to look at bus.
	ldaa	#$55		;some data to write= 55.
	staa	0,x		;unchecked data only to look at bus.
	coma			;some data to write= AA.
	staa	0,x		;unchecked data only to look at bus.
	ldx	#$0000		;address pattern to read from.
	ldaa	0,x		;read address, don't care about data.
	ldx	#$FFFF		;address pattern to read from.
	ldaa	0,x		;read address, don't care about data.
	ldx 	#$5555		;address pattern to read from.
	ldaa	0,x		;read address, don't care about data.
	ldx 	#$AAAA		;address pattern to read from.
	ldaa	0,x		;read address, don't care about data.


;	The LED is ON, when the board is reset, for about 1/2 - 1 second.
;	That is not to be confused with the first flash.
;	Turn it off so we know we successfully entered our code.
;	Successfull entry into prom will show two short flashes 
;	not counting the flash for reset.  If there is any question of 
;	the number of flashes, the diagnostic pushbutton on the mpu
;	(lower of the two buttons) may be depressed to cause a non-maskable
;	interrupt at any time to restart the program.  Since there is no
;	reset the only flashes seen at that point should be real.


	ldab	#$1		;Flash routines expect a count in b.
	ldx	#start_return	;x now has return address.
	txs			;sp now has return address.
	jmp	#flash_sp_ofent	;go turn off the light.

; 	Now we will do our 2 short flashes to show we have entered the code.

pass:

	ldx	#pass_return	;x has return address.
	ldab	#2		;flash count.  Double quick blink so we know
	jmp	sflash_sp_svc	;we are in code and where start is on a pass.
pass_return:




;	We init data in the CMOS ram to make sure that it will interfere
;	with either ram1 or ram2 if by chance it is not unique. Thus if
;	bus is corrupted during ram1 or ram2 testing; the user may
;	remove the cmos ram to see if the diagnostic will get past that
;	point.  If so the cmos enable circuitry should be check out.

	ldx	#cmos_min	;setup to init cmos.
	ldaa	#$0		;
cmos_init:
	anda	#$0f		;CMOS uses low 4 bits of data. Mask out unused.
	staa	0,x		;write to cmos.  If cmos bad we find out later.
	adda	#$3		;this will cause bits to vary so odd even addr
				;sees all bits set and cleared.
	inx			;next addr.
	cpx	#cmos_max+1	;check range.
	bne	#cmos_init	;branch if not done; else fall through.

;=============================================================================

start_return:

	ldab	#$02		;Two flashes on entry.
	ldx	#start_return1	;x now has return address.
	txs			;sp now has return address.
	jmp	#sflash_sp_svc	;Do flashes.
start_return1:


; The first real test starts here.

	ldx	#ram1_min		; Start addr of 6810.
	ldaa	#$0
ram1_data:
;	ram1_data attempts to check that each cell can hold any
;	data pattern in the 6810.  address has not been qualified yet.


	ldx	#ram1_min

ram1_d_nextadr:
	ldaa	#$0		;aa will be expected data
				;start at zero so this must get zero on a read
	bra	#ram1_d_datlp	;

ram1_d_errlp:
	clr	#reset_vector	;failure check in.  Write to read only space.
	ldab	error_action	;sets CCs.
	beq	#ram1_d_datlp	;no flash error.
	jmp	flash_error	;

ram1_d_datlp:
	staa	0,x		;write to the 6810.
	ldab	0,x		;read actual data from the 6810.
	cba			;compare a and b accumulators.
	bne	#ram1_d_errlp	;loop on the error till it clears.
	inca			;increment a.
	bne	#ram1_d_datlp	;data is 00-FF on FF we roll to 00 indicating
				;we have checked all data on current address.
	inx			;inc x to next addr.
	cpx	#ram1_max+1	;if past max addr then we are done.
	bne	#ram1_d_nextadr	;not done; do next address, or fall through.

	ldab	#$1		;flash #1.  ram1 data bits are unique.

	ldx	#ram1_d_return	;x now has return address.
	jmp	#flash_sp_svc	;go flash.
ram1_d_return:			;return here.

;=============================================================================

;	This test will verify that locations zero and one in ram1 are unique
;	from all other locations in ram1.  Thus we will have identified two
;	bytes of memory that can be used as stack space or temp storage in
;	the full ram1 address uniqueness test which is an address equal data
;	test.
;	By using all data patterns we verify that there is no data line(s)
;	shorted to an address line which would later confuse things for the
;	four addresses in question.  Probably most address uniqueness problems
;	will be caught here; however we will do adress equal data test later
;	to be very safe.  Note:  Between this test and the ram1_data_test; we
;	really should not see data shorted to an address.


	clra		;aa holds pattern to write. Do all patterns. 00-FF.
ram1_au0_part:
	ldx	#ram1_min+2
	bra	#ram1_au0_loop

ram1_au0_err:
	clr	#reset_vector	;failure check in.  Write to read only space.
	ldab	error_action	;sets CCs.
	beq	#ram1_au0_loop	;no flash error.
	jmp	flash_error	;

ram1_au0_loop:
	clrb			;to clear temp locs. We need some temp space.
	stab	ram1_min		;temp loc 0.  Part of temp_0.  Used for
	stab	ram1_min+1	;temp loc 1.  Part of temp_0.  Future storage.
	staa	0,x		;write test data to all other ram1 locations.
	ldab	ram1_min		;temp loc 0.  This will set CCs.
	bne	ram1_au0_err	;loop on error.
	ldab	ram1_min+1	;temp loc 1.  This will set CCs.
	bne	ram1_au0_err	;loop on error.
	inca			;inc aa to next data pattern.
	beq	#ram1_au0_ckdone
	bra	#ram1_au0_loop	;Do next data pattern.

ram1_au0_ckdone:
	cpx	#ram1_max	; check if really done.
	beq	ram1_au0_done	; yes we are really done.
	inx	
	clra			; Start pattern at 00 again.
	bra	#ram1_au0_loop	;not done continue looping.

ram1_au0_done:
;	ram1 Address Equal Data test.  Final adress uniqness test of ram1.
;	It is not expected that this test will ever likely fail except
;	in the case of a bad 6810.  However this test has a scope loop
;	in case strange circumstances are encountered and we fail due
;	to something other than a bad 6810.
;	The scope loop is only a repetative read of the failed location.
;	Since the data should equal the address, then using a 2 channel
;	scope it should be easy to look at the address being read; and
;	what the data comming out is.  The data comming out will most
;	likely not equal the address by a single bit.  That bit will
;	most likely be the address bit that is bad going into the
;	6810; assuming that the 6810 is known good, the address line
;	in question should be checked for shorts to surronding signals.

ram1_aed_test:
	ldaa	#ram1_max
	ldx	#ram1_max

ram1_aed_dec:
	staa	0,x		;write pattern to address.
	deca			;next pattern.
	dex			;next address.
	cpx	#temp_0+1	;leave our two byte space alone!
	bne	ram1_aed_dec	;done if equal else continue.

;	ram1 should now be completely address equal data less first two locs.

	ldaa	#temp_0+2	;get initial value in a register.
	ldx	#temp_0+2	;get start address from temp storage.

ram1_aed_inc:
	ldab	0,x		;read 1st address.
	cba			;compare expected in a, to actual in b.
	bne	ram1_find_loc	;Try to find the conflict for a scope loop.
	inca			;next pattern.
	inx			;next address.
	cpx	#ram1_max+1	;done when we reach ram1_max.
	bne	ram1_aed_inc	;continue if not done or fall through and pass.

;	ram1 should now be completely address equal data, less first two locs.

	jmp	ram1_aed_passed	;no error, we passed.

;	On entry x is the location in error found as follows:
;	The routine prior to this is expected to write data from max to
;	min addresses and then do the check from min to max so that 
;	temp_0 - 1 will be the highest address needed to find the error.
;	If this policy is not adhered to then this routine's ability to
;	localize the error will be compromised.
;	Note:  Temp_0 occupies the first two bytes of ram1 thus it MUST BE
;	gauranteed to to be uniqe to ram1 prior to using this routine.

ram1_find_loc:
	clra			;
	staa	0,x		;clear error location.
	stx	temp_0		;store away error loc for now.
	ldaa	#$AA		;Compliment of first pattern.  We do two.

ram1_au_nxt_dat:
	coma			;compliment so 1st pattern will be 55.
	ldx	#temp_0+2	;initial loc we write to.

ram1_au_nxt_adr:
	staa	0,x		;write to location then check error loc.
	txs			;store x in sp for now.
	ldx	temp_0		;get error loc into x.
	ldab	0,x		;check it.  This will set CCs.
	tsx			;restore x from sp. This will not change CCs.
	bne	ram1_aed_found	;we will now go and try to loop on the error.
	inx			;next address.  If x temp_0, we wrote up
	cpx	temp_0		;to temp_0 - 1 and cannot find the error.
	blt	ram1_au_nxt_adr	;continue to next addr.
	cmpa	#$AA		;if AA then we are done.
	bne	ram1_au_nxt_dat	;if only 55 then we need to do AA still.
	bra	ram1_aed_unot	;We should never exit this routine unless
				;we could not find error for some reason.
				;there for we branch, flash and restart.

;	ram1_f_loop expects the bad address in temp_0 and conflict addr in x.
;	data written to find error will be in aa still.  aa has the data.

ram1_aed_floop:
	clr	#reset_vector	;failure check in.  Write to read only space.
	ldab	error_action	;sets CCs.
	beq	#ram1_aed_found	;no flash error.
	jmp	flash_error	;

ram1_aed_found:
	txs			;hold x in sp.  Note x is unchanged.
	clrb			;
	ldx	temp_0		;x has error loc.
	stab	0,x		;clear error loc.
	tsx			;restore x.
	staa	0,x		;write to x, we expect to overwrite error loc.
	txs			;hold x in sp.
	ldx	temp_0		;x has error loc.
	ldab	0,x		;this will set CCs.
	tsx			;restore x from sp.  This does not affect CCs.
	bne	ram1_aed_floop	;if not equal we are seeing the error.

ram1_aed_unot:

	ldab	#$3		;setup to flash three times so we know
				;we lost the error.  We will restart.

ram1_zflash:
;	attempt to flash the light. n times based on contents of ab.

	ldx	#pia1_sel	;x has pia1 enable base addr.  addr of pia1
	ldaa	#pia_ddr_reg+led_on	;set to select ddra, cra 3,4,5 set.
	staa	cra_sel,x	;ddra is now selected, cra 3,4,5 now set.
	ldaa	#led_bits	;set bits 4 and 5 for led select in ddr.
	staa	ddra_sel,x	;setup mask in ddra.
	ldaa	#pia_out_reg	;bits for CRA to access PRA.
	staa	cra_sel,x	;set CRA.
	ldaa	#led_bits_on	;try to light Both LEDs.
	staa	pra_sel,x	;set bits in PRA.
	ldaa	#pia_out_reg+led_on	;Should be 3C.
	staa	cra_sel,x	;do it.  Should now be accessing outreg.
	staa	cra_sel,x	;do it.  Twice to be sure. CA2 should be high.

	ldx	#10000		;LED On count.

ram1_zflash_on:

	dex			; should decrement x by 1.
	bne	#ram1_zflash_on	; keep on for LED on count. 

;	setup to turn off now to complete flash.

	ldx	#pia1_sel	;x has pia1 enable base addr.  addr of pia1
	ldaa	#pia_ddr_reg+led_off	;set to select ddra, cra 2,3,4 set.
	staa	cra_sel,x	;ddra is now selected, cra 2,3,4 now set.
	ldaa	#pia_out_reg+led_off	;Should be 34.
	staa	cra_sel,x	;do it.  Should now be accessing outreg.
	staa	cra_sel,x	;do it.  Twice to be sure. CA2 should be high.
	ldaa	#led_bits_off	;to reset pa4 pa5 and turn off LEDs.
	staa	pra_sel,x	;PA4 AND PA5 SHOULD HAVE GONE LOW. LEDs OFF.

	ldx	#10000		;LED Off count.

ram1_zflash_off:
	dex			; should decrement x by 1.
	bne	#ram1_zflash_off	; keep off for LED off count. 
	decb
	bne	#ram1_zflash	; go till done flashing; then fall through.

	jmp	start		;we will restart from scratch.


;	The next test expects that ram1 has addr equal data pattern in it.
;	make it so.

ram1_aed_passed:
	ldaa	#$0
	ldx	#$0
ram1_set_loop:
	staa	0,x
	inx			;next address.
	inca			;next pattern.
	cmpa	#ram1_max+1	;if equal then we are done.
	bne	#ram1_set_loop	;go write pattern.

;	ram1 should now be completely address equal data.

	ldab	#$1		;flash #2.  ram1 addresses are unique.
				;we don't call flash svc yet till we check
				;that the ram is unique from other devices.
				;such as ram2 and the 5101 CMOS ram.
	ldx	#ram1_aed_return	;x now has return address.
	jmp	#flash_sp_svc	;go flash.
ram1_aed_return:

;=============================================================================

;	Verify that writing to ram2 does not write to ram1.

	ldx	#ram1_min	;
	ldaa	#$00		;setup ram1 data to known 00 is simplest.
ram1_u_clr:			; 
	staa	0,x		; clear.
	inx
	cpx	#ram1_max+1	; set CCs. equal if done.
	bne	#ram1_u_clr	; continue or fall through.

ram1_u_test:
	lds	#ram2_max	; high addr of device we are checking against.
	ldaa	#$ff		; data pattern we will write.

ram1ram2u_lp:
	psha			; This will write a byte to ram2.
	tsx			; x now has a copy of sp+1 sp is unchanged.
	cpx	#ram2_min	; because got sp+1 this is a straight cmpare.
	beq	#ram1ram2_unxt	; done goto next test; or continue.
	bra	ram1ram2u_lp	;

ram1ram2u_chk:
	ldx	#ram1_min	;

ram1ram2u_chklp:
	ldab	0,x		; read device location.  This will set CCs.
	bne	#ram1ram2_err	; if not zero goto error routine.

ram1ram2u_reent:
	cpx	#ram1_max	; check if done.
	bne	#ram1ram2u_chklp	; loop if not done
	inx			; inc x.
	bra	#ram1ram2u_chklp	;

ram1ram2_err:
	clr	#reset_vector	;failure check in.  Write to read only space.
	ldab	error_action	;sets CCs.
	beq	#ram1ram2_errlp	;no flash error.
	jmp	flash_error	;

ram1ram2_errlp:
	clr	0,x		; clear overwritten location.
	des			; sp will always be fail addr+1. fix it.
	psha			; write to suspect location.
	ldab	0,x		; check for failure.  This sets CCs.
	bne	#ram1ram2_errlp	; continue to loop on error if failing.
	bra	#ram1ram2u_reent	;failure not solid;continue.

ram1ram2_unxt:

;	Verify that writing to cmos does not write to ram1.

ram1cmos_uniq:
	ldx	#ram1_min	; start of ram1.
	ldaa	#$0		; zero to clear.

ram1cmos_clr:			; setup ram1 data to known 00 is simplest.
	staa	0,x		; clear ram1.
	inx
	cpx	#ram1_max+1	; check if done.
	bne	#ram1cmos_clr	; continue or fall through.
	lds	#cmos_max	; high addr of device we are checking against.
				; sp grows from high to low.
ram1cmos_lp:
	ldaa	#$FF		; Any pattern besides zero will do here.
	psha			; This will write a byte to cmos.
	tsx			; x now has a copy of sp+1. sp is unchanged.
	cpx	#cmos_min	; because got sp+1 this is a straight cmpare.
	beq	#ram1cmos_pass	; if done then pass.

ram1cmos_chk:
	ldx	#ram1_min	; Setup to check out ram1.  All addrs SB 00.

ram1cmos_chklp:
	ldab	0,x		; read device location.  This will set CCs.
	bne	#ram1cmos_err	; if not zero goto error routine.

ram1cmos_reent:			; if error goes away in errloop then re-enter.
	inx			; inc x.
	cpx	#ram1_max+1	; check if done.
	bne	#ram1cmos_chklp	; loop if not done
	bra	#ram1cmos_lp	; current did not overwrite, next cmos address.

ram1cmos_err:
	clr	#reset_vector	;failure check in.  Write to read only space.
	ldab	error_action	;sets CCs.
	beq	#ram1cmos_errlp	;no flash error.
	jmp	flash_error	;

ram1cmos_errlp:
	clr	0,x		; clear overwritten location.
	des			; sp will always be fail addr+1. fix it.
	psha			; write to suspect location.
	ldab	0,x		; check for failure.  This sets CCs.
	bne	#ram1cmos_errlp	; continue to loop on error if failing.
	bra	#ram1cmos_reent	; failure not solid;continue.

ram1cmos_pass:

	ldab	#$1		;flash #3.  ram1 unique from ram2 & cmos.
				; by ram2 or by cmos.
	ldx	#ram1cmos_return	;x now has return address.
	jmp	#flash_sp_svc	;go flash.
ram1cmos_return:

;=============================================================================

	ldx	#ram2_min		; Start addr of 6810.
	ldaa	#$0
ram2_data:
;	ram2_data attempts to check that each cell can hold any
;	data pattern in the 6810.  address has not been qualified yet.


	ldx	#ram2_min

ram2_d_nextadr:
	ldaa	#$0		;aa will be expected data
				;start at zero so this must get zero on a read
	bra	#ram2_d_datlp	;

ram2_d_errlp:
	clr	#reset_vector	;failure check in.  Write to read only space.
	ldab	error_action	;sets CCs.
	beq	#ram2_d_datlp	;no flash error.
	jmp	flash_error	;

ram2_d_datlp:
	staa	0,x		;write to the 6810.
	ldab	0,x		;read actual data from the 6810.
	cba			;compare a and b accumulators.
	bne	#ram2_d_errlp	;loop on the error till it clears.
	inca			;increment a.
	bne	#ram2_d_datlp	;data is 00-FF on FF we roll to 00 indicating
				;we have checked all data on current address.
	inx			;inc x to next addr.
	cpx	#ram2_max+1	;if past max addr then we are done.
	bne	#ram2_d_nextadr	;not done; do next address, or fall through.

	ldab	#$1		;flash #4.  ram2 data bits are unique.

	ldx	#ram2_d_return	;x now has return address.
	jmp	#flash_sp_svc	;go flash.
ram2_d_return:			;return here.

;=============================================================================

;	This test will verify that locations zero and one in ram2 are unique
;	from all other locations in ram2.  Thus we will have identified two
;	bytes of memory that can be used as stack space or temp storage in
;	the full ram2 address uniqueness test which is an address equal data
;	test.
;	By using all data patterns we verify that there is no data line(s)
;	shorted to an address line which would later confuse things for the
;	four addresses in question.  Probably most address uniqueness problems
;	will be caught here; however we will do adress equal data test later
;	to be very safe.  Note:  Between this test and the ram2_data_test; we
;	really should not see data shorted to an address.


	clra		;aa holds pattern to write. Do all patterns. 00-FF.
ram2_au0_part:
	ldx	#ram2_min+2
	bra	#ram2_au0_loop

ram2_au0_err:
	clr	#reset_vector	;failure check in.  Write to read only space.
	ldab	error_action	;sets CCs.
	beq	#ram2_au0_loop	;no flash error.
	jmp	flash_error	;

ram2_au0_loop:
	clrb			;to clear temp locs. We need some temp space.
	stab	ram2_min		;temp loc 0.  Part of temp_0.  Used for
	stab	ram2_min+1	;temp loc 1.  Part of temp_0.  Future storage.
	staa	0,x		;write test data to all other ram2 locations.
	ldab	ram2_min		;temp loc 0.  This will set CCs.
	bne	ram2_au0_err	;loop on error.
	ldab	ram2_min+1	;temp loc 1.  This will set CCs.
	bne	ram2_au0_err	;loop on error.
	inca			;inc aa to next data pattern.
	beq	#ram2_au0_ckdone
	bra	#ram2_au0_loop	;Do next data pattern.

ram2_au0_ckdone:
	cpx	#ram2_max	; check if really done.
	beq	ram2_au0_done	; yes we are really done.
	inx	
	clra			; Start pattern at 00 again.
	bra	#ram2_au0_loop	;not done continue looping.

ram2_au0_done:
;	ram2 Address Equal Data test.  Final adress uniqness test of ram2.
;	It is not expected that this test will ever likely fail except
;	in the case of a bad 6810.  However this test has a scope loop
;	in case strange circumstances are encountered and we fail due
;	to something other than a bad 6810.
;	The scope loop is only a repetative read of the failed location.
;	Since the data should equal the address, then using a 2 channel
;	scope it should be easy to look at the address being read; and
;	what the data comming out is.  The data comming out will most
;	likely not equal the address by a single bit.  That bit will
;	most likely be the address bit that is bad going into the
;	6810; assuming that the 6810 is known good, the address line
;	in question should be checked for shorts to surronding signals.

ram2_aed_test:
	ldaa	#ram2_max
	ldx	#ram2_max

ram2_aed_dec:
	staa	0,x		;write pattern to address.
	deca			;next pattern.
	dex			;next address.
	cpx	#temp_0+1	;leave our two byte space alone!
	bne	ram2_aed_dec	;done if equal else continue.

;	ram2 should now be completely address equal data less first two locs.

	ldaa	#temp_0+2	;get initial value in a register.
	ldx	#temp_0+2	;get start address from temp storage.

ram2_aed_inc:
	ldab	0,x		;read 1st address.
	cba			;compare expected in a, to actual in b.
	bne	ram2_find_loc	;Try to find the conflict for a scope loop.
	inca			;next pattern.
	inx			;next address.
	cpx	#ram2_max+1	;done when we reach ram2_max.
	bne	ram2_aed_inc	;continue if not done or fall through and pass.

;	ram2 should now be completely address equal data, less first two locs.

	jmp	ram2_aed_passed	;no error, we passed.

;	On entry x is the location in error found as follows:
;	The routine prior to this is expected to write data from max to
;	min addresses and then do the check from min to max so that 
;	temp_0 - 1 will be the highest address needed to find the error.
;	If this policy is not adhered to then this routine's ability to
;	localize the error will be compromised.
;	Note:  Temp_0 occupies the first two bytes of ram2 thus it MUST BE
;	gauranteed to to be uniqe to ram2 prior to using this routine.

ram2_find_loc:
	clra			;
	staa	0,x		;clear error location.
	stx	temp_0		;store away error loc for now.
	ldaa	#$AA		;Compliment of first pattern.  We do two.

ram2_au_nxt_dat:
	coma			;compliment so 1st pattern will be 55.
	ldx	#temp_0+2	;initial loc we write to.

ram2_au_nxt_adr:
	staa	0,x		;write to location then check error loc.
	txs			;store x in sp for now.
	ldx	temp_0		;get error loc into x.
	ldab	0,x		;check it.  This will set CCs.
	tsx			;restore x from sp. This will not change CCs.
	bne	ram2_aed_found	;we will now go and try to loop on the error.
	inx			;next address.  If x temp_0, we wrote up
	cpx	temp_0		;to temp_0 - 1 and cannot find the error.
	blt	ram2_au_nxt_adr	;continue to next addr.
	cmpa	#$AA		;if AA then we are done.
	bne	ram2_au_nxt_dat	;if only 55 then we need to do AA still.
	bra	ram2_aed_unot	;We should never exit this routine unless
				;we could not find error for some reason.
				;there for we branch, flash and restart.

;	ram2_f_loop expects the bad address in temp_0 and conflict addr in x.
;	data written to find error will be in aa still.  aa has the data.

ram2_aed_floop:
	clr	#reset_vector	;failure check in.  Write to read only space.
	ldab	error_action	;sets CCs.
	beq	#ram2_aed_found	;no flash error.
	jmp	flash_error	;

ram2_aed_found:
	txs			;hold x in sp.  Note x is unchanged.
	clrb			;
	ldx	temp_0		;x has error loc.
	stab	0,x		;clear error loc.
	tsx			;restore x.
	staa	0,x		;write to x, we expect to overwrite error loc.
	txs			;hold x in sp.
	ldx	temp_0		;x has error loc.
	ldab	0,x		;this will set CCs.
	tsx			;restore x from sp.  This does not affect CCs.
	bne	ram2_aed_floop	;if not equal we are seeing the error.

ram2_aed_unot:

	ldab	#$3		;setup to flash three times so we know
				;we lost the error.  We will restart.

ram2_zflash:
;	attempt to flash the light. n times based on contents of ab.

	ldx	#pia1_sel	;x has pia1 enable base addr.  addr of pia1
	ldaa	#pia_ddr_reg+led_on	;set to select ddra, cra 3,4,5 set.
	staa	cra_sel,x	;ddra is now selected, cra 3,4,5 now set.
	ldaa	#led_bits	;set bits 4 and 5 for led select in ddr.
	staa	ddra_sel,x	;setup mask in ddra.
	ldaa	#pia_out_reg	;bits for CRA to access PRA.
	staa	cra_sel,x	;set CRA.
	ldaa	#led_bits_on	;try to light Both LEDs.
	staa	pra_sel,x	;set bits in PRA.
	ldaa	#pia_out_reg+led_on	;Should be 3C.
	staa	cra_sel,x	;do it.  Should now be accessing outreg.
	staa	cra_sel,x	;do it.  Twice to be sure. CA2 should be high.

	ldx	#10000		;LED On count.

ram2_zflash_on:

	dex			; should decrement x by 1.
	bne	#ram2_zflash_on	; keep on for LED on count. 

;	setup to turn off now to complete flash.

	ldx	#pia1_sel	;x has pia1 enable base addr.  addr of pia1
	ldaa	#pia_ddr_reg+led_off	;set to select ddra, cra 2,3,4 set.
	staa	cra_sel,x	;ddra is now selected, cra 2,3,4 now set.
	ldaa	#pia_out_reg+led_off	;Should be 34.
	staa	cra_sel,x	;do it.  Should now be accessing outreg.
	staa	cra_sel,x	;do it.  Twice to be sure. CA2 should be high.
	ldaa	#led_bits_off	;to reset pa4 pa5 and turn off LEDs.
	staa	pra_sel,x	;PA4 AND PA5 SHOULD HAVE GONE LOW. LEDs OFF.

	ldx	#10000		;LED Off count.

ram2_zflash_off:
	dex			; should decrement x by 1.
	bne	#ram2_zflash_off	; keep off for LED off count. 
	decb
	bne	#ram2_zflash	; go till done flashing; then fall through.

	
	jmp	start		;we will restart from scratch.



;	The next test expects that ram2 has addr equal data pattern in it.
;	make it so.

ram2_aed_passed:
	ldaa	#$0
	ldx	#$0
ram2_set_loop:
	staa	0,x
	inx			;next address.
	inca			;next pattern.
	cmpa	#ram2_max+1	;if equal then we are done.
	bne	#ram2_set_loop	;go write pattern.

;	ram2 should now be completely address equal data.

	ldab	#$1		;flash #5.  ram2 addresses are unique.
				;we don't call flash svc yet till we check
				;that the ram is unique from other devices.
				;such as ram1 and the 5101 CMOS ram.
	ldx	#ram2_aed_return	;x now has return address.
	jmp	#flash_sp_svc	;go flash.
ram2_aed_return:

;=============================================================================

;	Verify that writing to ram1 does not write to ram2.

ram2ram1_uniq:
	ldx	#ram2_min	; start of ram2.
	ldaa	#$0		; zero to clear.

ram2ram1_clr:			; setup ram2 data to known 00 is simplest.
	staa	0,x		; clear ram2.
	inx
	cpx	#ram2_max+1	; check if done.
	bne	#ram2ram1_clr	; continue or fall through.
	lds	#ram1_max	; high addr of device we are checking against.
				; sp grows from high to low.
ram2ram1_lp:
	ldaa	#$FF		; Any pattern besides zero will do here.
	psha			; This will write a byte to ram1.
	tsx			; x now has a copy of sp+1. sp is unchanged.
	cpx	#ram1_min	; because got sp+1 this is a straight cmpare.
	beq	#ram2ram1_pass	; if done then pass.

ram2ram1_chk:
	ldx	#ram2_min	; Setup to check out ram2.  All addrs SB 00.

ram2ram1_chklp:
	ldab	0,x		; read device location.  This will set CCs.
	bne	#ram2ram1_err	; if not zero goto error routine.

ram2ram1_reent:			; if error goes away in errloop then re-enter.
	inx			; inc x.
	cpx	#ram2_max+1	; check if done.
	bne	#ram2ram1_chklp	; loop if not done
	bra	#ram2ram1_lp	; current did not overwrite, next ram1 address.

ram2ram1_err:
	clr	#reset_vector	;failure check in.  Write to read only space.
	ldab	error_action	;sets CCs.
	beq	#ram2ram1_errlp	;no flash error.
	jmp	flash_error	;

ram2ram1_errlp:
	clr	0,x		; clear overwritten location.
	des			; sp will always be fail addr+1. fix it.
	psha			; write to suspect location.
	ldab	0,x		; check for failure.  This sets CCs.
	bne	#ram2ram1_errlp	; continue to loop on error if failing.
	bra	#ram2ram1_reent	; failure not solid;continue.

ram2ram1_pass:

;	Verify that writing to cmos does not write to ram2.

ram2cmos_uniq:
	ldx	#ram2_min	; start of ram2.
	ldaa	#$0		; zero to clear.

ram2cmos_clr:			; setup ram2 data to known 00 is simplest.
	staa	0,x		; clear ram2.
	inx
	cpx	#ram2_max+1	; check if done.
	bne	#ram2cmos_clr	; continue or fall through.
	lds	#cmos_max	; high addr of device we are checking against.
				; sp grows from high to low.
ram2cmos_lp:
	ldaa	#$FF		; Any pattern besides zero will do here.
	psha			; This will write a byte to cmos.
	tsx			; x now has a copy of sp+1. sp is unchanged.
	cpx	#cmos_min	; because got sp+1 this is a straight cmpare.
	beq	#ram2cmos_pass	; if done then pass.

ram2cmos_chk:
	ldx	#ram2_min	; Setup to check out ram2.  All addrs SB 00.

ram2cmos_chklp:
	ldab	0,x		; read device location.  This will set CCs.
	bne	#ram2cmos_err	; if not zero goto error routine.

ram2cmos_reent:			; if error goes away in errloop then re-enter.
	inx			; inc x.
	cpx	#ram2_max+1	; check if done.
	bne	#ram2cmos_chklp	; loop if not done
	bra	#ram2cmos_lp	; current did not overwrite, next cmos address.

ram2cmos_err:
	clr	#reset_vector	;failure check in.  Write to read only space.
	ldab	error_action	;sets CCs.
	beq	#ram2cmos_errlp	;no flash error.
	jmp	flash_error	;

ram2cmos_errlp:
	clr	0,x		; clear overwritten location.
	des			; sp will always be fail addr+1. fix it.
	psha			; write to suspect location.
	ldab	0,x		; check for failure.  This sets CCs.
	bne	#ram2cmos_errlp	; continue to loop on error if failing.
	bra	#ram2cmos_reent	; failure not solid;continue.

ram2cmos_pass:

	ldab	#$1		;flash #6.  ram2 unique from ram1 & cmos.
				; by ram1 or by cmos.
	ldx	#ram2cmos_return	;x now has return address.
	jmp	#flash_sp_svc	;go flash.
ram2cmos_return:

;=============================================================================


	lds	#stack_strt	;initialize stack pointer; stack is ready.

; This is the start of called routines; Ie the stack must be ready.


;=============================================================================

	jsr	cmos_data_test	;go test cmos data.
	ldab	#$01		;setup for one flash.
	jsr	flash_svc	;flash #7.  cmos data test passed.

	jsr	cmos_addr_test	;go test cmos addressing.
	ldab	#$01		;setup for one flash.
	jsr	flash_svc	;flash #8.  cmos address test passed.

	ldx	#pia1_sel	;setup to test the pia on the mpu.
	stx	var_1		;var_1 now has pia base address.
	jsr	pia_test	;go do the test.
	ldab	#$01		;setup for one flash.
	jsr	flash_svc	;flash #9.  pia1 the mpu pia test passed.


	ldx	#30000

delay:		;delay flashing for a moment before next pass of tests.

	dex			; should decrement x by 1.
	bne	#delay		; delay between passes.
	jmp	start		; goto start.


;=============================================================================


	; This is start of cmos testing of both data and addressing.
	; Because the 5101 CMOS Ram is only 4 bits wide; we take some
	; great pains to see to it that we find conflicting addresses
	; in the address uniqueness test for this part for a scope loop.
	; the pain however is well worth the gain for using a scope or
	; especially an analyzer.
	;
	; Note:  The CMOS is assumed to be a unique device at this point as
	; it has not interfered with any other test.  It most likely
	; would have messed up some other device test were it not unique.

cmos_data_test:

;	cmos_data attempts to check that each cell can hold any
;	data pattern in the 5101.  address has not been qualified yet.


	ldx	#cmos_min

cmos_d_nextadr:
	ldaa	#$0		;aa will be expected data
	ldab	#$ff		;ab will be actual. Init to anything else; we
				;start at zero so this must get zero on a read
	bra	cmos_d_datloop	

cmos_d_errloop:
	clr	#reset_vector	;failure check in.  Write to read only space.
	ldab	error_action	;sets CCs.
	beq	#cmos_d_datloop	;no flash error.
	jmp	flash_error	;

cmos_d_datloop:
	anda	#$0f		;5101 is only 4 bits wide.  Mask unused bits.
	staa	0,x		;write to the 5101.
	ldab	0,x		;read from the 5101.
	andb	#$0f		;5101 is only 4 bits wide.  Mask unused bits.
	cba			;compare a and b accumulators.
	bne	#cmos_d_errloop	;loop on the error till it clears.
	adda	#$01		;inca on MSB side, lsb 4 bits stay zero.
	anda	#$0f		;just to be sure.
	bne	#cmos_d_datloop	;data is 00-FF on FF we roll to 00 indicating
				;we have checked all data on current address.
	inx			;inc x to next addr.
	cpx	#cmos_max+1	;if past max addr then we are done.
	bne	#cmos_d_nextadr	;not done; do next address.

	rts			;return to caller.

;===========================================================================

cmos_addr_test:
	ldx	#cmos_min		;5101 starting address
	ldaa	#$0		;aa will be expected data
	staa	var_1		;Clear variable located in 6810 used for uniq.
	ldab	#$ff		;ab will be actual. Init to anything else; we
				;start at zero so this must get zero on a read
	bra	#cmos_a_datloop

cmos_a_errloop:
	clr	#reset_vector	;failure check in.  Write to read only space.
	ldab	error_action	;sets CCs.
	beq	#cmos_a_datloop	;no flash error.
	jmp	flash_error	;

cmos_a_datloop:
	adda	var_1		;A+M->A var_1 allows us to do uniqueness.
	anda	#$0f		;5101 is only 4 bits wide.  Mask unused bits.
	staa	0,x		;write to the 5101.
	ldab	0,x		;read/verify data.
	andb	#$0f		;5101 is only 4 bits wide.  Mask unused bits.
	cba			;compare a and b accumulators.
	bne	#cmos_a_errloop	;loop on the error till it clears.
	adda	#$01		;inca. msb, already has the uniq cnt offset.
	anda	#$0f		;5101 is only 4 bits wide.  Mask unused bits.
	bne 	cmos_continue	;on 0 we will increment var_1.
	ldab	var_1		;get current cnt in b.
	addb	#$01		;inc b (really cnt) used to provide addr uniq.
	andb	#$0f		;5101 is only 4 bits wide.  Mask unused bits.
	stab	var_1		;put new count back in cnt.
cmos_continue:
	inx			;inc x to next addr.
	cpx	#cmos_max+1	;if past max addr then we are done.
	bne	#cmos_a_datloop	;not done; do next address.

cmos_addr_uniq:

cmos_uclr:
	ldx	#cmos_min		;Start addr of 5101.
	ldaa	#$0

cmos_uclr_addr:
	staa	0,x		;clear 5101.
	inx			;inc x to next addr.
	cpx	#cmos_max+1	;if past max addr then we are done.
	bne	#cmos_uclr_addr	;not done.

	ldx	#cmos_min
	ldaa	#$0f		;pattern to write to 5101.

cmos_uloop:
	ldab	0,x		;check before writing.
	andb	#$0f		;mask out unused bits.
	cmpb	#$0		;verify loc is zero.	
	bne	cmos_uerror	;go try to loop on error.
	staa	0,x		;loc was zero; write pattern to loc.
	inx			;next loc.
	cpx	#cmos_max+1	;check if done.
	bne	cmos_uloop	;not done.

	rts			;return to caller.

cmos_uerror:
	stx	var_1		;loc that was in error.
	clr	0,x		;clear error loc.
	ldx	#cmos_min	;setup to find location that overwrote
				;the location just stored in var_1.
	ldaa	#$0f		;reload a with same data.

cmos_find_loc:

	staa	0,x
	txs			;hold x in sp for a moment.
	ldx	var_1		;now get fail addr to check.
	ldab	0,x		;
	tsx			;put x held in sp back into x.
	andb	#$0f		;mask out unused bits.
	stx	var_2		;Addr in var_2 confilicts with addr in var_1.
	cmpb	#$0		;check error loc was still zero.  If not,
	bne	cmos_uerr_loop	;then try again.
	inx			;try next loc.
	cpx	var_1		;don't need to go past addr in var_1.
	beq	cmos_unot	;did not find bad address.	

	ldaa	#$0f		;to be safe.

cmos_uerr_lloop:
	clr	#reset_vector	;failure check in.  Write to read only space.
	ldab	error_action	;sets CCs.
	beq	#cmos_uerr_loop	;no flash error.
	jmp	flash_error	;

cmos_uerr_loop:
		
	clr	0,x		;clear the addr in var_2.
	ldx	var_1		;address that destroyed data in loc var_2.
	staa	0,x		;write to var_1 should wipe data in var_2 addr.
	ldx	var_2		;check we destroyed data var_2 addr.
	ldab	0,x		;read.
	andb	#$0f		;mask unused.
	bne	cmos_uerr_lloop	;stay in as long as we are failing.

cmos_unot:
	ldab	#$3
	jsr	sflash_svc	;Flash 3 short so we know that the error was 
	jmp	start		;not located, we will re-start the test.

;===========================================================================

	; pia_test is a called routine.
	; var_1 on entry contains the base enable addr for the pia to test.
	; On error this call will loop on the error. On Pass it will return.
	; to the caller (depending on the setting of error_action).

pia_test_err:
	clr	#reset_vector	;failure check in.  Write to read only space.
	ldab	error_action	;sets CCs.
	beq	#pia_test	;no flash error.
	jmp	flash_error	;

pia_test:
	ldx	var_1		;x has pia enable base addr.
	ldaa	#pia_ddr_reg	;access ddr reg.
	staa	cra_sel,x	;set ddr access.  PIA is now setup.
	clra			;
	staa	pia_reg_b_sel,x	;should clear ddrb reg.
	ldaa	#$AA		;test pattern.
	staa	pia_reg_a_sel,x	;should write to ddra reg.
	ldab	pia_reg_a_sel,x	;should read from ddra reg.
	cba			;compare expected in A to actual in B.
	bne	#pia_test_err	;loop on error.

	clra			;clear a reg to verify ddrb was unique in pia.
	ldab	pia_reg_b_sel,x	;ddrb should still be clear if unique.
	cba			;compare expected in A to actual in B.
	bne	#pia_test_err	;loop on error.  Not unique.
	bra	#pia_loop1	;continue test.

pia_loop1_err:
	clr	#reset_vector	;failure check in.  Write to read only space.
	ldab	error_action	;sets CCs.
	beq	#pia_loop1	;no flash error.
	jmp	flash_error	;

pia_loop1:
	ldaa	#$55		;test pattern.
	staa	pia_reg_a_sel,x	;should write to ddra reg.
	ldab	pia_reg_a_sel,x	;should read from ddra reg.
	cba			;compare expected in A to actual in B.
	bne	#pia_loop1_err	;loop on error.
	bra	#pia_loop2	;continue test.

pia_loop2_err:
	clr	#reset_vector	;failure check in.  Write to read only space.
	ldab	error_action	;sets CCs.
	beq	#pia_loop2	;no flash error.
	jmp	flash_error	;

pia_loop2:			;start checking ddrb reg.
	clra			;
	staa	pia_reg_a_sel,x	;clear ddra for uniqueness check.
	ldaa	#$55		;test pattern.
	staa	pia_reg_b_sel,x	;should write to ddrb reg.
	ldab	pia_reg_b_sel,x	;should read from ddrb reg.
	cba			;compare expected in A to actual in B.
	bne	#pia_loop2_err	;loop on error.
	bra	#pia_loop3	;continue testing.

pia_loop3_err:
	clr	#reset_vector	;failure check in.  Write to read only space.
	ldab	error_action	;sets CCs.
	beq	#pia_loop3	;no flash error.
	jmp	flash_error	;

pia_loop3:	
	ldaa	#$AA		;test pattern.
	staa	pia_reg_b_sel,x	;should write to ddrb reg.
	ldab	pia_reg_b_sel,x	;should read from ddrb reg.
	cba			;compare expected in A to actual in B.
	bne	#pia_loop3_err	;loop on error.
	ldab	pia_reg_a_sel,x	;ddra should still be clear.  Set CCs.
	bne	#pia_loop2_err	;we loop back to 2 to see all writes in loop.

	rts			;If here we passed.  Return to caller.

;===========================================================================

;=============================================================================
;
;	Service type routines are below such as flashing the LEDs.
;
;=============================================================================

;===========================================================================

	; flash_error.
	; This will set the b register to 5 flashes and jmp to
	; sflash_sp_svc.  This routine does not return, it restarts.


flash_error:			;we do not plan to return so we do much.
				;and the heck with the stack.  It matters not.
	ldx	#20000		;delay so we can tell the difference.
				;when looping.
flash_err_delay:
	dex
	bne	#flash_err_delay ; Delay to see flashes clearly.

	ldab	#$5		;b now holds the number of times to flash.
	ldx	#flash_err_ret	;x has return address.
	jmp	sflash_sp_svc	;Go flash the light 5 short times.
				;sflash_sp returns to caller in x.
flash_err_ret:
	ldaa	error_action	;Find out what to do here.
	cmpa	#loop_val	;
	beq	#flash_error_rst	;go restart tests ie: loop.
	cmpa	#halt_val	;
	beq	#halt_loop	;effectively halt the testing.
	cmpa	#restart_val	;
	beq	#flash_error_rst	;go restart tests.
				;any other value will currently restart also,
				;but is not guaranteed to do so in later
				;revisions.
flash_error_rst:
	jmp	start		;flash complete, now restart from scratch.

;===========================================================================

		;This loop does nothing and is an infinite loop on itself
		;as there is no halt instruction for the 6800.
halt_loop:
	clr	#reset_vector	;failure check in.  Write to read only space.
halt_lp:
	nop
	nop
	bra	#halt_lp	;loop on nothing to simulate a halted program.


;===========================================================================



;===========================================================================

flash_sp_svc:
;	Attempt to flash the light n times based on contents of ab.
;
;	On entry x has return address.  We need to store it into the
;	sp and then we can restore it.  This routine does not use
;	any stack space nor does it push or pull anything to or from
;	the stack.  The SP itself is utilized as a holding register.
;	This routine may not change the x register prior to the txs
;	instruction and may not subsequently change the sp register.

	txs			; x is holding the return address, save in sp.

flash_sp_lp:

	ldx	#pia1_sel	;x has pia1 enable base addr.  addr of pia1
	ldaa	#pia_ddr_reg+led_on	;set to select ddra, cra 3,4,5 set.
	staa	cra_sel,x	;ddra is now selected, cra 3,4,5 now set.
	ldaa	#led_bits	;set bits 4 and 5 for led select in ddr.
	staa	ddra_sel,x	;setup mask in ddra.
	ldaa	#pia_out_reg	;bits for CRA to access PRA.
	staa	cra_sel,x	;set CRA.
	ldaa	#led_bits_on	;try to light Both LEDs.
	staa	pra_sel,x	;set bits in PRA.
	ldaa	#pia_out_reg+led_on	;Should be 3C.
	staa	cra_sel,x	;do it.  Should now be accessing outreg.
	staa	cra_sel,x	;do it.  Twice to be sure. CA2 should be high.

	ldx	#30000		;LED On count.

flash_sp_on:

	dex			; should decrement x by 1.
	bne	#flash_sp_on	; keep on for LED on count. 

;	setup to turn off now to complete flash.

flash_sp_ofent:			;flash_sp_ofent expects return addr in sp.
	ldx	#pia1_sel	;x has pia1 enable base addr.  addr of pia1
	ldaa	#pia_ddr_reg+led_off	;set to select ddra, cra 2,3,4 set.
	staa	cra_sel,x	;ddra is now selected, cra 2,3,4 now set.
	ldaa	#pia_out_reg+led_off	;Should be 34.
	staa	cra_sel,x	;do it.  Should now be accessing outreg.
	staa	cra_sel,x	;do it.  Twice to be sure. CA2 should be high.
	ldaa	#led_bits_off	;to reset pa4 pa5 and turn off LEDs.
	staa	pra_sel,x	;PA4 AND PA5 SHOULD HAVE GONE LOW. LEDs OFF.

	ldx	#30000		;LED Off count.

flash_sp_off:
	dex			; should decrement x by 1.
	bne	#flash_sp_off	; keep off for LED off count. 
	decb
	bne	#flash_sp_lp	; go till done flashing.

	tsx			; restore return address from the sp.
	jmp	0,x		; return to caller.

;===========================================================================

sflash_sp_svc:
;	Attempt to flash the light n times based on contents of ab.
;
;	On entry x has return address.  We need to store it into the
;	sp and then we can restore it.  This routine does not use
;	any stack space nor does it push or pull anything to or from
;	the stack.  The SP itself is utilized as a holding register.
;	This routine may not change the x register prior to the txs
;	instruction and may not subsequently change the sp register.

	txs			; x is holding the return address, save in sp.

sflash_sp_lp:

	ldx	#pia1_sel	;x has pia1 enable base addr.  addr of pia1
	ldaa	#pia_ddr_reg+led_on	;set to select ddra, cra 3,4,5 set.
	staa	cra_sel,x	;ddra is now selected, cra 3,4,5 now set.
	ldaa	#led_bits	;set bits 4 and 5 for led select in ddr.
	staa	ddra_sel,x	;setup mask in ddra.
	ldaa	#pia_out_reg	;bits for CRA to access PRA.
	staa	cra_sel,x	;set CRA.
	ldaa	#led_bits_on	;try to light Both LEDs.
	staa	pra_sel,x	;set bits in PRA.
	ldaa	#pia_out_reg+led_on	;Should be 3C.
	staa	cra_sel,x	;do it.  Should now be accessing outreg.
	staa	cra_sel,x	;do it.  Twice to be sure. CA2 should be high.

	ldx	#10000		;LED On count.

sflash_sp_on:

	dex			; should decrement x by 1.
	bne	#sflash_sp_on	; keep on for LED on count. 

;	setup to turn off now to complete flash.

sflash_sp_ofent:		;sflash_sp_ofent expects return addr in sp.
	ldx	#pia1_sel	;x has pia1 enable base addr.  addr of pia1
	ldaa	#pia_ddr_reg+led_off	;set to select ddra, cra 2,3,4 set.
	staa	cra_sel,x	;ddra is now selected, cra 2,3,4 now set.
	ldaa	#pia_out_reg+led_off	;Should be 34.
	staa	cra_sel,x	;do it.  Should now be accessing outreg.
	staa	cra_sel,x	;do it.  Twice to be sure. CA2 should be high.
	ldaa	#led_bits_off	;to reset pa4 pa5 and turn off LEDs.
	staa	pra_sel,x	;PA4 AND PA5 SHOULD HAVE GONE LOW. LEDs OFF.

	ldx	#10000		;LED Off count.

sflash_sp_off:
	dex			; should decrement x by 1.
	bne	#sflash_sp_off	; keep off for LED off count. 
	decb
	bne	#sflash_sp_lp	; go till done flashing.

	tsx			; restore return address from the sp.
	jmp	0,x		; return to caller.

;===========================================================================

flash_svc:
;	attempt to flash the light. n times based on contents of b register.


	stab	temp		;b has count need to hold b reg temporarily.
	psha			; 
	stx	x_place		; setup to push x also.
	ldaa	x_place		;
	ldab	x_place+1	;
	psha			; x high onto stack
	pshb			; x low onto stack
	ldab	temp		; b should now have the flash count again.

flash_svc_lp:
	ldx	#pia1_sel	;x has pia1 enable base addr.  addr of pia1
	ldaa	#pia_ddr_reg+led_on	;set to select ddra, cra 3,4,5 set.
	staa	cra_sel,x	;ddra is now selected, cra 3,4,5 now set.
	ldaa	#led_bits	;set bits 4 and 5 for led select in ddr.
	staa	ddra_sel,x	;setup mask in ddra.
	ldaa	#pia_out_reg	;bits for CRA to access PRA.
	staa	cra_sel,x	;set CRA.
	ldaa	#led_bits_on	;try to light Both LEDs.
	staa	pra_sel,x	;set bits in PRA.
	ldaa	#pia_out_reg+led_on	;Should be 3C.
	staa	cra_sel,x	;do it.  Should now be accessing outreg.
	staa	cra_sel,x	;do it.  Twice to be sure. CA2 should be high.

	ldx	#30000		;LED On count.

flash_on:

	dex			; should decrement x by 1.
	bne	#flash_on	; keep on for LED on count. 

;	setup to turn off now to complete flash.

	ldx	#pia1_sel	;x has pia1 enable base addr.  addr of pia1
	ldaa	#pia_ddr_reg+led_off	;set to select ddra, cra 2,3,4 set.
	staa	cra_sel,x	;ddra is now selected, cra 2,3,4 now set.
	ldaa	#pia_out_reg+led_off	;Should be 34.
	staa	cra_sel,x	;do it.  Should now be accessing outreg.
	staa	cra_sel,x	;do it.  Twice to be sure. CA2 should be high.
	ldaa	#led_bits_off	;to reset pa4 pa5 and turn off LEDs.
	staa	pra_sel,x	;PA4 AND PA5 SHOULD HAVE GONE LOW. LEDs OFF.

	ldx	#30000		;LED Off count.

flash_off:
	dex			; should decrement x by 1.
	bne	#flash_off	; keep off for LED off count. 
	decb
	bne	#flash_svc_lp	; go till done flashing.

	pulb			;restoring x.
	pula			;restoring x.
	stab	x_place+1	;restoring x.
	staa	x_place		;restoring x.
	ldx	x_place		;x is now restored.
	pula			;restore a.
	rts			; return to caller.

;===========================================================================

sflash_svc:
;	attempt to flash the light. n times based on contents of b register.


	stab	temp		;b has count need to hold b reg temporarily.
	psha			; 
	stx	x_place		; setup to push x also.
	ldaa	x_place		; 
	ldab	x_place+1	;
	psha			; x high onto stack
	pshb			; x low onto stack
	ldab	temp		; b should now have the flash count again.

sflash_svc_lp:
	ldx	#pia1_sel	;x has pia1 enable base addr.  addr of pia1
	ldaa	#pia_ddr_reg+led_on	;set to select ddra, cra 3,4,5 set.
	staa	cra_sel,x	;ddra is now selected, cra 3,4,5 now set.
	ldaa	#led_bits	;set bits 4 and 5 for led select in ddr.
	staa	ddra_sel,x	;setup mask in ddra.
	ldaa	#pia_out_reg	;bits for CRA to access PRA.
	staa	cra_sel,x	;set CRA.
	ldaa	#led_bits_on	;try to light Both LEDs.
	staa	pra_sel,x	;set bits in PRA.
	ldaa	#pia_out_reg+led_on	;Should be 3C.
	staa	cra_sel,x	;do it.  Should now be accessing outreg.
	staa	cra_sel,x	;do it.  Twice to be sure. CA2 should be high.

	ldx	#30000		;LED On count.

sflash_on:

	dex			; should decrement x by 1.
	bne	#sflash_on	; keep on for LED on count. 

;	setup to turn off now to complete flash.

	ldx	#pia1_sel	;x has pia1 enable base addr.  addr of pia1
	ldaa	#pia_ddr_reg+led_off	;set to select ddra, cra 2,3,4 set.
	staa	cra_sel,x	;ddra is now selected, cra 2,3,4 now set.
	ldaa	#pia_out_reg+led_off	;Should be 34.
	staa	cra_sel,x	;do it.  Should now be accessing outreg.
	staa	cra_sel,x	;do it.  Twice to be sure. CA2 should be high.
	ldaa	#led_bits_off	;to reset pa4 pa5 and turn off LEDs.
	staa	pra_sel,x	;PA4 AND PA5 SHOULD HAVE GONE LOW. LEDs OFF.

	ldx	#30000		;LED Off count.

sflash_off:
	dex			; should decrement x by 1.
	bne	#sflash_off	; keep off for LED off count. 
	decb
	bne	#sflash_svc_lp	; go till done flashing.

	pulb			;restoring x.
	pula			;restoring x.
	stab	x_place+1	;restoring x.
	staa	x_place		;restoring x.
	ldx	x_place		;x is now restored.
	pula			;restore a.
	rts			; return to caller.

;===========================================================================

	org	ic17_end-7	; start of 6800 vector table.

	; currently we will point all vectors to start for now.
	; maybe at some time we will mess with them.


	; Currently we will point most vectors to start for now.
	; Some day we may toy some more with them.

	fdb	halt_loop	; FFF8,FFF9 IRQ	CURRENTLY NOT USED, HALT.
	fdb	halt_loop	; FFFA,FFFB SWI	CURRENTLY NOT USED, HALT.
	fdb	show_revision	; FFFC,FFFD NMI CLEAR SWITCH (SW 33)
	fdb	start		; FFFE,FFFF RESET


;===========================================================================
